@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -81,6 +81,7 @@ def _getBuiltinExceptionNames():
 builtin_exception_names, builtin_exception_values = _getBuiltinExceptionNames()
 
 # Just to make sure it's covering these cases correctly.
+assert "TypeError" in builtin_exception_names
 assert "ValueError" in builtin_exception_names
 assert "StopIteration" in builtin_exception_names
 assert "GeneratorExit" in builtin_exception_names
@@ -105,6 +106,9 @@ def _getBuiltinNames():
     names.remove("__name__")
     names.remove("__package__")
 
+    if "__loader__" in names:
+        names.remove("__loader__")
+
     warnings = []
 
     for builtin_name in names:
@@ -130,7 +134,7 @@ def getBuiltinTypeNames():
     result = []
 
     for builtin_name in builtin_names:
-        if isinstance(__builtins__[builtin_name],type):
+        if isinstance(__builtins__[builtin_name], type):
             result.append(builtin_name)
 
     return tuple(sorted(result))
@@ -169,6 +173,8 @@ def _getAnonBuiltins():
     }
 
     if python_version < 300:
+        # There are only there for Python2,
+        # pylint: disable=I0021,no-name-in-module
         from types import ClassType, InstanceType, MethodType
 
         anon_names["classobj"] = ClassType
