@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -30,14 +30,28 @@ import subprocess
 import sys
 from logging import info, warning
 
+from nuitka.finalizations.FinalizeMarkups import getImportedNames
 from nuitka.importing import Importing, Recursion
 from nuitka.Options import getPythonFlags
 from nuitka.plugins.Plugins import Plugins
-from nuitka.PythonVersions import isUninstalledPython, python_version
+from nuitka.PythonVersions import (
+    getSupportedPythonVersions,
+    isUninstalledPython,
+    python_version,
+    python_version_str
+)
 from nuitka.tree import SyntaxErrors
-from nuitka.utils import InstanceCounters, MemoryUsage, Utils
+from nuitka.utils import Execution, InstanceCounters, MemoryUsage, Utils
+from nuitka.utils.AppDirs import getCacheDir
+from nuitka.utils.FileOperations import (
+    deleteFile,
+    hasFilenameExtension,
+    listDir,
+    makePath,
+    removeDirectory
+)
 
-from . import ModuleRegistry, Options, Tracing, TreeXML
+from . import ModuleRegistry, Options, TreeXML
 from .build import SconsInterface
 from .codegen import CodeGeneration, ConstantCodes
 from .finalizations import Finalization
@@ -70,16 +84,19 @@ def createNodeTree(filename):
     # harm.
     source_dir = getSourceDirectoryPath(main_module)
 
-    if not Options.shallOnlyExecCppCall():
+    if not Options.shallOnlyExecCCompilerCall():
         cleanSourceDirectory(source_dir)
 
     # Prepare the ".dist" directory, throwing away what was there before.
     if Options.isStandaloneMode():
         standalone_dir = getStandaloneDirectoryPath(main_module)
-        shutil.rmtree(standalone_dir, ignore_errors = True)
-        Utils.makePath(standalone_dir)
+        removeDirectory(
+            path          = standalone_dir,
+            ignore_errors = True
+        )
+        makePath(standalone_dir)
 
-    Utils.deleteFile(
+    deleteFile(
         path       = getResultFullpath(main_module),
         must_exist = False
     )
@@ -100,7 +117,16 @@ def createNodeTree(filename):
     # Then optimize the tree and potentially recursed modules.
     Optimization.optimize()
 
-    return main_module
+    if Options.isExperimental("check_xml_persistence"):
+        for module in ModuleRegistry.getRootModules():
+            if module.isMainModule():
+                return module
+
+        assert False
+    else:
+        # Main module might change behind our back, look it up again.
+        return main_module
+
 
 def dumpTreeXML(tree):
     xml_root = tree.asXml()
@@ -123,7 +149,7 @@ def getSourceDirectoryPath(main_module):
     assert main_module.isCompiledPythonModule()
 
     return Options.getOutputPath(
-        path = Utils.basename(
+        path = os.path.basename(
             getTreeFilenameWithSuffix(main_module, ".build")
         )
     )
@@ -131,7 +157,7 @@ def getSourceDirectoryPath(main_module):
 
 def getStandaloneDirectoryPath(main_module):
     return Options.getOutputPath(
-        path = Utils.basename(
+        path = os.path.basename(
             getTreeFilenameWithSuffix(main_module, ".dist")
         )
     )
@@ -141,15 +167,15 @@ def getResultBasepath(main_module):
     assert main_module.isCompiledPythonModule()
 
     if Options.isStandaloneMode():
-        return Utils.joinpath(
+        return os.path.join(
             getStandaloneDirectoryPath(main_module),
-            Utils.basename(
+            os.path.basename(
                 getTreeFilenameWithSuffix(main_module, "")
             )
         )
     else:
         return Options.getOutputPath(
-            path = Utils.basename(
+            path = os.path.basename(
                 getTreeFilenameWithSuffix(main_module, "")
             )
         )
@@ -159,10 +185,7 @@ def getResultFullpath(main_module):
     result = getResultBasepath(main_module)
 
     if Options.shallMakeModule():
-        if Utils.getOS() == "Windows":
-            result += ".pyd"
-        else:
-            result += ".so"
+        result += Utils.getSharedLibrarySuffix()
     else:
         result += ".exe"
 
@@ -170,35 +193,17 @@ def getResultFullpath(main_module):
 
 
 def cleanSourceDirectory(source_dir):
-    if Utils.isDir(source_dir):
-        for path, _filename in Utils.listDir(source_dir):
-            if Utils.getExtension(path) in (".cpp", ".hpp", ".c", ".o", ".os",
-                                            ".obj", ".bin", ".res", ".rc",
-                                            ".manifest"):
-                Utils.deleteFile(path, True)
+    if os.path.isdir(source_dir):
+        for path, _filename in listDir(source_dir):
+            if hasFilenameExtension(
+                path       = path,
+                extensions = (".c", ".h", ".o", ".os", ".obj",
+                              ".bin", ".res", ".rc", ".S", ".cpp",
+                              ".manifest")
+            ):
+                deleteFile(path, must_exist = True)
     else:
-        Utils.makePath(source_dir)
-
-    static_source_dir = Utils.joinpath(
-        source_dir,
-        "static"
-    )
-
-    if Utils.isDir(static_source_dir):
-        for path, _filename in Utils.listDir(static_source_dir):
-            if Utils.getExtension(path) in (".o", ".os", ".obj"):
-                Utils.deleteFile(path, True)
-
-    win32_source_dir = Utils.joinpath(
-        static_source_dir,
-        "win32_ucontext_src"
-    )
-
-    if Utils.getOS() == "Windows":
-        Utils.deleteFile(
-            Utils.joinpath(win32_source_dir, "fibers_win32.obj"),
-            False
-        )
+        makePath(source_dir)
 
 
 def pickSourceFilenames(source_dir, modules):
@@ -209,7 +214,7 @@ def pickSourceFilenames(source_dir, modules):
     module_filenames = {}
 
     def getFilenames(module):
-        base_filename =  Utils.joinpath(
+        base_filename =  os.path.join(
             source_dir,
             "module." + module.getFullName()
               if not module.isInternalModule() else
@@ -219,7 +224,7 @@ def pickSourceFilenames(source_dir, modules):
         # Note: Could detect if the file system is cases sensitive in source_dir
         # or not, but that's probably not worth the effort. False positives do
         # no harm at all.
-        collision_filename = Utils.normcase(base_filename)
+        collision_filename = os.path.normcase(base_filename)
 
         return base_filename, collision_filename
 
@@ -255,26 +260,27 @@ def pickSourceFilenames(source_dir, modules):
 
         base_filename += hash_suffix
 
-        module_filenames[module] = base_filename + ".cpp"
+        module_filenames[module] = base_filename + ".c"
 
     return module_filenames
 
 
 standalone_entry_points = []
 
+
 def makeSourceDirectory(main_module):
     """ Get the full list of modules imported, create code for all of them.
 
     """
     # We deal with a lot of details here, but rather one by one, and split makes
-    # no sense, pylint: disable=R0912,R0914
+    # no sense, pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     assert main_module.isCompiledPythonModule()
 
     # The global context used to generate code.
     global_context = CodeGeneration.makeGlobalContext()
 
-    assert main_module in ModuleRegistry.getDoneModules()
+    # assert main_module in ModuleRegistry.getDoneModules()
 
     # We might have chosen to include it as bytecode, and only compiled it for
     # fun, and to find its imports. In this case, now we just can drop it. Or
@@ -294,7 +300,8 @@ def makeSourceDirectory(main_module):
                 # module. Some uncompiled modules may have been asked by the user
                 # or technically required. By default, frozen code if it exists
                 # is preferred, as it will be from standalone mode adding it.
-                if uncompiled_module.isUserProvided():
+                if uncompiled_module.isUserProvided() or \
+                   uncompiled_module.isTechnical():
                     ModuleRegistry.removeDoneModule(module)
                 else:
                     ModuleRegistry.removeUncompiledModule(uncompiled_module)
@@ -302,6 +309,9 @@ def makeSourceDirectory(main_module):
     # Lets check if the recurse-to modules are actually present, and warn the
     # user if one of those was not found.
     for any_case_module in Options.getShallFollowModules():
+        if '*' in any_case_module or '{' in any_case_module:
+            continue
+
         for module in ModuleRegistry.getDoneUserModules():
             if module.getFullName() == any_case_module:
                 break
@@ -331,9 +341,9 @@ def makeSourceDirectory(main_module):
 
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            cpp_filename = module_filenames[module]
+            c_filename = module_filenames[module]
 
-            prepared_modules[cpp_filename] = CodeGeneration.prepareModuleCode(
+            prepared_modules[c_filename] = CodeGeneration.prepareModuleCode(
                 global_context = global_context,
                 module         = module,
                 module_name    = module.getFullName(),
@@ -341,14 +351,14 @@ def makeSourceDirectory(main_module):
 
             # Main code constants need to be allocated already too.
             if module is main_module and not Options.shallMakeModule():
-                prepared_modules[cpp_filename][1].getConstantCode(0)
+                prepared_modules[c_filename][1].getConstantCode(0)
 
     # Second pass, generate the actual module code into the files.
     for module in ModuleRegistry.getDoneModules():
         if module.isCompiledPythonModule():
-            cpp_filename = module_filenames[module]
+            c_filename = module_filenames[module]
 
-            template_values, module_context = prepared_modules[cpp_filename]
+            template_values, module_context = prepared_modules[c_filename]
 
             source_code = CodeGeneration.generateModuleCode(
                 module_context  = module_context,
@@ -356,14 +366,14 @@ def makeSourceDirectory(main_module):
             )
 
             writeSourceCode(
-                filename    = cpp_filename,
+                filename    = c_filename,
                 source_code = source_code
             )
 
             if Options.isShowInclusion():
                 info("Included compiled module '%s'." % module.getFullName())
         elif module.isPythonShlibModule():
-            target_filename = Utils.joinpath(
+            target_filename = os.path.join(
                 getStandaloneDirectoryPath(main_module),
                 *module.getFullName().split('.')
             )
@@ -373,10 +383,10 @@ def makeSourceDirectory(main_module):
             else:
                 target_filename += ".so"
 
-            target_dir = Utils.dirname(target_filename)
+            target_dir = os.path.dirname(target_filename)
 
-            if not Utils.isDir(target_dir):
-                Utils.makePath(target_dir)
+            if not os.path.isdir(target_dir):
+                makePath(target_dir)
 
             shutil.copy(
                 module.getFilename(),
@@ -384,7 +394,11 @@ def makeSourceDirectory(main_module):
             )
 
             standalone_entry_points.append(
-                (target_filename, module.getPackage())
+                (
+                    module.getFilename(),
+                    target_filename,
+                    module.getPackage()
+                )
             )
         elif module.isUncompiledPythonModule():
             pass
@@ -392,7 +406,10 @@ def makeSourceDirectory(main_module):
             assert False, module
 
     writeSourceCode(
-        filename    = Utils.joinpath(source_dir, "__constants.cpp"),
+        filename    = os.path.join(
+            source_dir,
+            "__constants.c"
+        ),
         source_code = ConstantCodes.getConstantsDefinitionCode(
             context = global_context
         )
@@ -403,37 +420,41 @@ def makeSourceDirectory(main_module):
     )
 
     writeSourceCode(
-        filename    = Utils.joinpath(source_dir, "__helpers.hpp"),
+        filename    = os.path.join(
+            source_dir,
+            "__helpers.h"
+        ),
         source_code = helper_decl_code
     )
 
     writeSourceCode(
-        filename    = Utils.joinpath(source_dir, "__helpers.cpp"),
+        filename    = os.path.join(
+            source_dir,
+            "__helpers.c"
+        ),
         source_code = helper_impl_code
     )
 
 
 def runScons(main_module, quiet):
     # Scons gets transported many details, that we express as variables, and
-    # have checks for them, leading to many branches, pylint: disable=R0912
-
-    python_version_str = "%d.%d" % (sys.version_info[0], sys.version_info[1])
+    # have checks for them, leading to many branches, pylint: disable=too-many-branches
 
     if hasattr(sys, "abiflags"):
+        abiflags = sys.abiflags
+
         if Options.isPythonDebug() or \
            hasattr(sys, "getobjects"):
-            if sys.abiflags.startswith('d'):
-                python_version_str += sys.abiflags
-            else:
-                python_version_str += 'd' + sys.abiflags
-        else:
-            python_version_str += sys.abiflags
+            if not abiflags.startswith('d'):
+                abiflags = 'd' + abiflags
+    else:
+        abiflags = None
 
     def asBoolStr(value):
         return "true" if value else "false"
 
     options = {
-        "name"            : Utils.basename(
+        "name"            : os.path.basename(
             getTreeFilenameWithSuffix(main_module, "")
         ),
         "result_name"     : getResultBasepath(main_module),
@@ -442,14 +463,14 @@ def runScons(main_module, quiet):
         "python_debug"    : asBoolStr(Options.isPythonDebug()),
         "unstripped_mode" : asBoolStr(Options.isUnstripped()),
         "module_mode"     : asBoolStr(Options.shallMakeModule()),
-        "optimize_mode"   : asBoolStr(Options.isOptimize()),
         "full_compat"     : asBoolStr(Options.isFullCompat()),
-        "experimental"    : asBoolStr(Options.isExperimental()),
+        "experimental"    : ','.join(Options.getExperimentalIndications()),
         "trace_mode"      : asBoolStr(Options.shallTraceExecution()),
         "python_version"  : python_version_str,
         "target_arch"     : Utils.getArchitecture(),
         "python_prefix"   : sys.prefix,
         "nuitka_src"      : SconsInterface.getSconsDataPath(),
+        "nuitka_cache"    : getCacheDir(),
         "module_count"    : "%d" % (
             1 + \
             len(ModuleRegistry.getDoneUserModules()) + \
@@ -526,6 +547,8 @@ def runScons(main_module, quiet):
     if python_version < 300 and sys.flags.unicode:
         options["python_sysflag_unicode"] = "true"
 
+    if abiflags:
+        options["abiflags"] = abiflags
 
     return SconsInterface.runScons(options, quiet), options
 
@@ -533,7 +556,7 @@ def runScons(main_module, quiet):
 def writeSourceCode(filename, source_code):
     # Prevent accidental overwriting. When this happens the collision detection
     # or something else has failed.
-    assert not Utils.isFile(filename), filename
+    assert not os.path.isfile(filename), filename
 
     if python_version >= 300:
         with open(filename, "wb") as output_file:
@@ -546,7 +569,7 @@ def writeSourceCode(filename, source_code):
 def writeBinaryData(filename, binary_data):
     # Prevent accidental overwriting. When this happens the collision detection
     # or something else has failed.
-    assert not Utils.isFile(filename), filename
+    assert not os.path.isfile(filename), filename
 
     assert type(binary_data) is bytes
 
@@ -554,7 +577,7 @@ def writeBinaryData(filename, binary_data):
         output_file.write(binary_data)
 
 
-def callExec(args, clean_path, add_path):
+def callExecPython(args, clean_path, add_path):
     old_python_path = os.environ.get("PYTHONPATH", None)
 
     if clean_path and old_python_path is not None:
@@ -573,16 +596,21 @@ def callExec(args, clean_path, add_path):
     # Add the main arguments, previous separated.
     args += Options.getPositionalArgs()[1:] + Options.getMainArgs()
 
-    Utils.callExec(args)
+    Execution.callExec(args)
 
 
 def executeMain(binary_filename, clean_path):
     args = (binary_filename, binary_filename)
 
     if Options.shallRunInDebugger():
-        args = ("/usr/bin/gdb", "gdb", "-ex=run", "-ex=where", "--args", binary_filename)
+        gdb_path = Execution.getExecutablePath("gdb")
 
-    callExec(
+        if gdb_path is None:
+            sys.exit("Error, no 'gdb' binary found in path.")
+
+        args = (gdb_path, "gdb", "-ex=run", "-ex=where", "--args", binary_filename)
+
+    callExecPython(
         clean_path = clean_path,
         add_path   = False,
         args       = args
@@ -599,7 +627,7 @@ def executeModule(tree, clean_path):
         python_command,
     )
 
-    callExec(
+    callExecPython(
         clean_path = clean_path,
         add_path   = True,
         args       = args
@@ -609,7 +637,7 @@ def executeModule(tree, clean_path):
 def compileTree(main_module):
     source_dir = getSourceDirectoryPath(main_module)
 
-    if not Options.shallOnlyExecCppCall():
+    if not Options.shallOnlyExecCCompilerCall():
         # Now build the target language code for the whole tree.
         makeSourceDirectory(
             main_module = main_module
@@ -619,25 +647,28 @@ def compileTree(main_module):
 
         if frozen_code is not None:
             writeSourceCode(
-                filename    = Utils.joinpath(
+                filename    = os.path.join(
                     source_dir,
-                    "__frozen.cpp"
+                    "__frozen.c"
                 ),
                 source_code = frozen_code
             )
 
         writeBinaryData(
-            filename    = Utils.joinpath(source_dir, "__constants.bin"),
+            filename    = os.path.join(
+                source_dir,
+                "__constants.bin"
+            ),
             binary_data = ConstantCodes.stream_data.getBytes()
         )
     else:
         source_dir = getSourceDirectoryPath(main_module)
 
-        if not Utils.isFile(Utils.joinpath(source_dir, "__helpers.hpp")):
+        if not os.path.isfile(os.path.join(source_dir, "__helpers.h")):
             sys.exit("Error, no previous build directory exists.")
 
     if Options.isShowProgress() or Options.isShowMemory():
-        Tracing.printLine(
+        info(
             "Total memory usage before running scons: {memory}:".format(
                 memory = MemoryUsage.getHumanReadableProcessMemoryUsage()
             )
@@ -646,7 +677,7 @@ def compileTree(main_module):
     if Options.isShowMemory():
         InstanceCounters.printStats()
 
-    if Options.shallNotDoExecCppCall():
+    if Options.shallNotDoExecCCompilerCall():
         return True, {}
 
     # Run the Scons to build things.
@@ -658,6 +689,29 @@ def compileTree(main_module):
     return result, options
 
 
+def handleSyntaxError(e):
+    # Syntax or indentation errors, output them to the user and abort. If
+    # we are not in full compat, and user has not specified the Python
+    # versions he wants, tell him about the potential version problem.
+    error_message = SyntaxErrors.formatOutput(e)
+
+    if not Options.isFullCompat() and \
+       Options.getIntendedPythonVersion() is None:
+        if python_version < 300:
+            suggested_python_version_str = getSupportedPythonVersions()[-1]
+        else:
+            suggested_python_version_str = "2.7"
+
+        error_message += """
+
+Nuitka is very syntax compatible with standard Python. It is currently running
+with Python version '%s', you might want to specify more clearly with the use
+of e.g. '--python-version=%s' option, if that's not the one expected.
+""" % (python_version_str, suggested_python_version_str)
+
+    sys.exit(error_message)
+
+
 data_files = []
 
 def main():
@@ -667,22 +721,18 @@ def main():
         in the desired version of Python with desired flags, and we just get
         to execute the task assigned.
 
-        We might be asked to only re-compile generated C++, dump only an XML
+        We might be asked to only re-compile generated C, dump only an XML
         representation of the internal node tree after optimization, etc.
     """
 
-    # Main has to fullfil many options, leading to many branches and statements
-    # to deal with them.  pylint: disable=R0912
-
-    positional_args = Options.getPositionalArgs()
-    assert len(positional_args) > 0
-
+    # Main has to fulfill many options, leading to many branches and statements
+    # to deal with them.  pylint: disable=too-many-branches
     filename = Options.getPositionalArgs()[0]
 
     # Inform the importing layer about the main script directory, so it can use
     # it when attempting to follow imports.
     Importing.setMainScriptDirectory(
-        main_dir = Utils.dirname(Utils.abspath(filename))
+        main_dir = os.path.dirname(os.path.abspath(filename))
     )
 
     # Detect to be frozen modules if any, so we can consider to not recurse
@@ -692,12 +742,12 @@ def main():
             ModuleRegistry.addUncompiledModule(module)
 
             if module.getName() == "site":
-                origin_prefix_filename = Utils.joinpath(
-                    Utils.dirname(module.getCompileTimeFilename()),
+                origin_prefix_filename = os.path.join(
+                    os.path.dirname(module.getCompileTimeFilename()),
                     "orig-prefix.txt"
                 )
 
-                if Utils.isFile(origin_prefix_filename):
+                if os.path.isfile(origin_prefix_filename):
                     data_files.append(
                         (filename, "orig-prefix.txt")
                     )
@@ -708,17 +758,17 @@ def main():
             filename = filename
         )
     except (SyntaxError, IndentationError) as e:
-        # Syntax or indentation errors, output them to the user and abort.
-        sys.exit(
-            SyntaxErrors.formatOutput(e)
-        )
+        handleSyntaxError(e)
 
     if Options.shallDumpBuiltTreeXML():
+        # XML only.
         for module in ModuleRegistry.getDoneModules():
             dumpTreeXML(module)
     elif Options.shallDisplayBuiltTree():
+        # GUI only.
         displayTree(main_module)
     else:
+        # Make the actual compilation.
         result, options = compileTree(
             main_module = main_module
         )
@@ -727,13 +777,17 @@ def main():
         if not result:
             sys.exit(1)
 
-        if Options.shallNotDoExecCppCall():
+        if Options.shallNotDoExecCCompilerCall():
+            if Options.isShowMemory():
+                MemoryUsage.showMemoryTrace()
+
             sys.exit(0)
 
         # Remove the source directory (now build directory too) if asked to.
         if Options.isRemoveBuildDir():
-            shutil.rmtree(
-                getSourceDirectoryPath(main_module)
+            removeDirectory(
+                path          = getSourceDirectoryPath(main_module),
+                ignore_errors = False
             )
 
         if Options.isStandaloneMode():
@@ -741,7 +795,7 @@ def main():
 
             standalone_entry_points.insert(
                 0,
-                (binary_filename, None)
+                (binary_filename, binary_filename, None)
             )
 
             dist_dir = getStandaloneDirectoryPath(main_module)
@@ -761,19 +815,18 @@ def main():
                 standalone_entry_points = standalone_entry_points
             )
 
-
             for module in ModuleRegistry.getDoneModules():
                 data_files.extend(
                     Plugins.considerDataFiles(module)
                 )
 
             for source_filename, target_filename in data_files:
-                target_filename = Utils.joinpath(
+                target_filename = os.path.join(
                     getStandaloneDirectoryPath(main_module),
                     target_filename
                 )
 
-                Utils.makePath(Utils.dirname(target_filename))
+                makePath(os.path.dirname(target_filename))
 
                 shutil.copy2(
                     source_filename,
@@ -791,6 +844,41 @@ def main():
                 )
             )
 
+        if Options.shallMakeModule() and Options.shallCreatePyiFile():
+            pyi_filename = getResultBasepath(main_module) + ".pyi"
+
+            with open(pyi_filename, 'w') as pyi_file:
+                pyi_file.write(
+                    """\
+# This file was generated by Nuitka and describes the types of the
+# created shared library.
+
+# At this time it lists only the imports made and can be used by the
+# tools that bundle libraries, including Nuitka itself. For instance
+# standalone mode usage of the created library will need it.
+
+# In the future, this will also contain type information for values
+# in the module, so IDEs will use this. Therfore please include it
+# when you make software releases of the extension module that it
+# describes.
+
+%(imports)s
+
+# This is not Python source even if it looks so. Make it clear for
+# now. This was decided by PEP 484 designers.
+__name__ = ...
+
+""" % {
+                        "imports" : '\n'.join(
+                            "import %s" % module_name
+                            for module_name in
+                            getImportedNames()
+                        )
+
+                    }
+                )
+
+
         # Execute the module immediately if option was given.
         if Options.shallExecuteImmediately():
             if Options.shallMakeModule():
