@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,9 +25,11 @@
     the existing set of modules.
 """
 
+import os
+
 from nuitka.containers.oset import OrderedSet
 from nuitka.PythonVersions import python_version
-from nuitka.utils import Utils
+from nuitka.utils.FileOperations import areSamePaths
 
 # One or more root modules, i.e. entry points that must be there.
 root_modules = OrderedSet()
@@ -58,6 +60,18 @@ def hasRootModule(module_name):
     return False
 
 
+def replaceRootModule(old, new):
+    # Using global here, as this is really a singleton, in the form of a module,
+    # pylint: disable=global-statement
+    global root_modules
+    new_root_modules = OrderedSet()
+
+    for module in root_modules:
+        new_root_modules.add(module if module is not old else new)
+
+    root_modules = new_root_modules
+
+
 def addUncompiledModule(module):
     uncompiled_modules.add(module)
 
@@ -69,6 +83,34 @@ def getUncompiledModules():
     )
 
 
+def getUncompiledTechnicalModules():
+    result = [
+        module
+        for module in
+        uncompiled_modules
+        if module.isTechnical()
+    ]
+
+    return sorted(
+        result,
+        key = lambda module : module.getFullName()
+    )
+
+
+def getUncompiledNonTechnicalModules():
+    result = [
+        module
+        for module in
+        uncompiled_modules
+        if not module.isTechnical()
+    ]
+
+    return sorted(
+        result,
+        key = lambda module : module.getFullName()
+    )
+
+
 def _normalizeModuleFilename(filename):
     if python_version >= 300:
         filename = filename.replace("__pycache__", "")
@@ -81,8 +123,8 @@ def _normalizeModuleFilename(filename):
         if filename.endswith(".pyc"):
             filename = filename[:-3] + ".py"
 
-    if Utils.basename(filename) == "__init__.py":
-        filename = Utils.dirname(filename)
+    if os.path.basename(filename) == "__init__.py":
+        filename = os.path.dirname(filename)
 
     return filename
 
@@ -90,7 +132,7 @@ def _normalizeModuleFilename(filename):
 def getUncompiledModule(module_name, module_filename):
     for uncompiled_module in uncompiled_modules:
         if module_name == uncompiled_module.getFullName():
-            if Utils.areSamePaths(
+            if areSamePaths(
                 _normalizeModuleFilename(module_filename),
                 _normalizeModuleFilename(uncompiled_module.filename)
             ):
@@ -105,7 +147,7 @@ def removeUncompiledModule(module):
 
 def startTraversal():
     # Using global here, as this is really a singleton, in the form of a module,
-    # pylint: disable=W0603
+    # pylint: disable=global-statement
     global active_modules, done_modules
 
     active_modules = OrderedSet(root_modules)
@@ -150,7 +192,6 @@ def getDoneUserModules():
             for module in
             done_modules
             if not module.isInternalModule()
-            if not module.isMainModule()
         ],
         key = lambda module : module.getFullName()
     )
@@ -158,3 +199,38 @@ def getDoneUserModules():
 
 def removeDoneModule(module):
     done_modules.remove(module)
+
+
+def getModuleFromCodeName(code_name):
+    # TODO: We need something to just load modules.
+    for module in root_modules:
+        if module.getCodeName() == code_name:
+            return module
+
+    assert False, code_name
+
+
+def getOwnerFromCodeName(code_name):
+    if "$$$" in code_name:
+        module_code_name, _function_code_name = code_name.split("$$$", 1)
+
+        module = getModuleFromCodeName(module_code_name)
+        return module.getFunctionFromCodeName(code_name)
+    else:
+        return getModuleFromCodeName(code_name)
+
+
+def getModuleByName(module_name):
+    for module in active_modules:
+        if module.getFullName() == module_name:
+            return module
+
+    for module in done_modules:
+        if module.getFullName() == module_name:
+            return module
+
+    for module in uncompiled_modules:
+        if module.getFullName() == module_name:
+            return module
+
+    return None
