@@ -16,6 +16,7 @@
 #     limitations under the License.
 #
 """ Options module """
+
 import logging
 import os
 import sys
@@ -69,9 +70,9 @@ parser.add_option(
     help    = """\
 Enable standalone mode in build. This allows you to transfer the created binary
 to other machines without it relying on an existing Python installation. It
-implies these options: "--recurse-all --recurse-stdlib". You may also want
-to use "--python-flag=no_site" to avoid the "site.py" module. Defaults to
-off.""",
+implies these option: "--recurse-all". You may also want to use
+"--python-flag=no_site" to avoid the "site.py" module, which can save a lot
+of code dependencies. Defaults to off.""",
 )
 
 parser.add_option(
@@ -139,14 +140,14 @@ Python runtime warnings), "-O" (alias "noasserts"). Default empty."""
 )
 
 parser.add_option(
-    "--python2-for-scons",
+    "--python-for-scons", "--python2-for-scons",
     action  = "store",
-    dest    = "python2_scons",
+    dest    = "python_scons",
     default = None,
     help    = """\
-If using Python3, provide the path of a Python2 binary to use. For Python2
-defaults to what you run Nuitka with, otherwise searches on Windows in the
-registry or non-Windows in PATH."""
+If using Python3.2 to Python3.4, provide the path of a Python binary to use
+for Scons. Otherwise Nuitka can use what you run Nuitka with or a "scons"
+binary that is found in PATH, or a Python installation from Windows registry."""
 )
 
 parser.add_option(
@@ -373,7 +374,7 @@ parser.add_option_group(codegen_group)
 
 outputdir_group = OptionGroup(
     parser,
-    "Output directory choices"
+    "Output choices"
 )
 
 outputdir_group.add_option(
@@ -383,8 +384,9 @@ outputdir_group.add_option(
     metavar = "DIRECTORY",
     default = "",
     help    = """\
-Specify where intermediate and final output files should be put. DIRECTORY will
-be populated with C files, object files, etc. Defaults to current directory.
+Specify where intermediate and final output files should be put. The DIRECTORY
+will be populated with C files, object files, etc.
+Defaults to current directory.
 """
 )
 
@@ -398,6 +400,18 @@ Removes the build directory after producing the module or exe file.
 Defaults to off."""
 )
 
+outputdir_group.add_option(
+    "--no-pyi-file",
+    action  = "store_false",
+    dest    = "pyi_file",
+    default = True,
+    help    = """\
+Do not create a ".pyi" file for extension modules created by Nuitka.
+Defaults to off."""
+)
+
+
+
 parser.add_option_group(outputdir_group)
 
 
@@ -689,6 +703,16 @@ this detection code is run in vain once you are certain of which plug-ins to
 use. Defaults to off."""
 )
 
+plugin_group.add_option(
+    "--plugin-list",
+    action  = "store_true",
+    dest    = "list_plugins",
+    default = False,
+    help    = """\
+Show list of all available plugins and exit. Defaults to off."""
+)
+
+
 parser.add_option_group(plugin_group)
 
 
@@ -697,7 +721,7 @@ positional_args = None
 extra_args = []
 
 def parseArgs():
-    # many cases, pylint: disable=global-statement,too-many-branches
+    # singleton with many cases, pylint: disable=global-statement,too-many-branches,too-many-statements
     global options, positional_args, extra_args
 
     # First, isolate the first non-option arguments.
@@ -722,6 +746,10 @@ def parseArgs():
 
     options, positional_args = parser.parse_args()
 
+    if shallListPlugins():
+        from nuitka.plugins.Plugins import listPlugins
+        listPlugins()
+
     if not positional_args:
         parser.print_help()
 
@@ -789,35 +817,44 @@ Error, '--recurse-not-to' takes only module names, not directory path '%s'.""" %
                 no_case_module
             )
 
-    scons_python = getPython2PathForScons()
+    scons_python = getPythonPathForScons()
 
     if scons_python is not None and not os.path.exists(scons_python):
         sys.exit("Error, no such Python2 binary '%s'." % scons_python)
 
+
 def shallTraceExecution():
     return options.trace_execution
 
+
 def shallExecuteImmediately():
     return options.immediate_execution
 
+
 def shallRunInDebugger():
     return options.debugger
 
+
 def shallDumpBuiltTreeXML():
     return options.dump_xml
 
+
 def shallDisplayBuiltTree():
     return options.display_tree
 
+
 def shallOnlyExecCCompilerCall():
     return options.recompile_c_only
 
+
 def shallNotDoExecCCompilerCall():
     return options.generate_c_only
 
+
 def shallHaveStatementLines():
     return options.statement_lines
 
+
 def getFileReferenceMode():
     if options.file_reference_mode is None:
         value = ("runtime"
@@ -828,102 +865,142 @@ def getFileReferenceMode():
 
     return value
 
+
 def shallMakeModule():
     return not options.executable
 
-def shallFollowStandardLibrary():
-    return options.recurse_stdlib
 
-def shallFollowNoImports():
-    return options.recurse_none
+def shallCreatePyiFile():
+    return options.pyi_file
 
-def shallFollowAllImports():
-    return options.recurse_all
-
-def getShallFollowModules():
-    return sum([ x.split(',') for x in options.recurse_modules ], [])
 
 def isAllowedToReexecute():
     return options.allow_reexecute
 
+
+def shallFollowStandardLibrary():
+    return options.recurse_stdlib
+
+
+def shallFollowNoImports():
+    return options.recurse_none
+
+
+def shallFollowAllImports():
+    return options.recurse_all
+
+
+def _splitShellPattern(value):
+    return value.split(',') if '{' not in value else [value]
+
+
 def getShallFollowInNoCase():
-    return sum([ x.split(',') for x in options.recurse_not_modules ], [])
+    return sum([ _splitShellPattern(x) for x in options.recurse_not_modules ], [])
+
+
+def getShallFollowModules():
+    return sum([ _splitShellPattern(x) for x in options.recurse_modules ], [])
+
 
 def getShallFollowExtra():
     return sum([ x.split(',') for x in options.recurse_extra ], [])
 
+
 def getShallFollowExtraFilePatterns():
     return sum([ x.split(',') for x in options.recurse_extra_files ], [])
 
+
 def shallWarnImplicitRaises():
     return options.warn_implicit_exceptions
 
+
 def shallWarnUnusualCode():
     return options.warn_unusual_code
 
+
 def isDebug():
     return options is not None and (options.debug or options.debugger)
 
+
 def isPythonDebug():
     return options.python_debug or sys.flags.debug
 
+
 def isUnstripped():
     return options.unstripped or options.profile
 
+
 def isProfile():
     return options.profile
 
+
 def shouldCreateGraph():
     return options.graph
 
+
 def getOutputPath(path):
     if options.output_dir:
         return os.path.normpath(os.path.join(options.output_dir, path))
     else:
         return path
 
+
 def getOutputDir():
     return options.output_dir if options.output_dir else '.'
 
+
 def getPositionalArgs():
     return tuple(positional_args)
 
+
 def getMainArgs():
     return tuple(extra_args)
 
+
 def shallOptimizeStringExec():
     return False
 
+
 def shallClearPythonPathEnvironment():
     return not options.keep_pythonpath
 
+
 def isShowScons():
     return options.show_scons
 
+
 def getJobLimit():
     return int(options.jobs)
 
+
 def isLto():
     return options.lto
 
+
 def isClang():
     return options.clang
 
+
 def isMingw():
     return options.mingw
 
+
 def getMsvcVersion():
     return options.msvc
 
+
 def shallDisableConsoleWindow():
     return options.win_disable_console
 
+
 def isFullCompat():
     return not options.improved
 
+
 def isShowProgress():
     return options.show_progress
 
+
 def isShowMemory():
     return options is not None and options.show_memory
 
@@ -968,32 +1045,44 @@ def isStandaloneMode():
 def getIconPath():
     return options.icon_path
 
+_python_flags = None
 
 def getPythonFlags():
-    result = set()
+    # singleton, pylint: disable=global-statement
+    global _python_flags
 
-    for parts in options.python_flags:
-        for part in parts.split(','):
-            if part in ("-S", "nosite", "no_site"):
-                result.add("no_site")
-            elif part in ("static_hashes", "norandomization", "no_randomization"):
-                result.add("no_randomization")
-            elif part in ("-v", "trace_imports", "trace_import"):
-                result.add("trace_imports")
-            elif part in ("no_warnings", "nowarnings"):
-                result.add("no_warnings")
-            elif part in ("-O", "no_asserts", "noasserts"):
-                result.add("no_asserts")
-            else:
-                logging.warning("Unsupported flag '%s'.", part)
+    if _python_flags is None:
+        _python_flags = set()
 
-    return result
+        for parts in options.python_flags:
+            for part in parts.split(','):
+                if part in ("-S", "nosite", "no_site"):
+                    _python_flags.add("no_site")
+                elif part in ("static_hashes", "norandomization",
+                              "no_randomization"):
+                    _python_flags.add("no_randomization")
+                elif part in ("-v", "trace_imports", "trace_import"):
+                    _python_flags.add("trace_imports")
+                elif part in ("no_warnings", "nowarnings"):
+                    _python_flags.add("no_warnings")
+                elif part in ("-O", "no_asserts", "noasserts"):
+                    _python_flags.add("no_asserts")
+                else:
+                    # Do not warn before executing in final context.
+                    if "PYTHONHASHSEED" in os.environ:
+                        logging.warning("Unsupported flag '%s'.", part)
+
+    return _python_flags
 
 
 def shallFreezeAllStdlib():
     return options.freeze_stdlib
 
 
+def shallListPlugins():
+    return options is not None and options.list_plugins
+
+
 def getPluginsEnabled():
     if not options:
         return ()
@@ -1017,5 +1106,5 @@ def getPluginOptions(plugin_name):
     return {}
 
 
-def getPython2PathForScons():
-    return options.python2_scons
+def getPythonPathForScons():
+    return options.python_scons
