@@ -67,6 +67,9 @@ class Variable(object):
     def getOwner(self):
         return self.owner
 
+    def getEntryPoint(self):
+        return self.owner.getEntryPoint()
+
     def getCodeName(self):
         var_name = self.variable_name
         var_name = var_name.replace('.', '$')
@@ -120,15 +123,19 @@ class Variable(object):
         if not self.users:
             return False
 
+        owner = self.owner.getEntryPoint()
+
         for user in self.users:
-            while user is not self.owner and \
+            user = user.getEntryPoint()
+
+            while user is not owner and \
                   (
                    (user.isExpressionFunctionBody() and not user.needsCreation()) or \
                    user.isExpressionClassBody()
                   ):
                 user = user.getParentVariableProvider()
 
-            if user is not self.owner:
+            if user is not owner:
                 return True
 
         return False
@@ -250,7 +257,7 @@ class ParameterVariable(LocalVariable):
 
 
 class ModuleVariable(Variable):
-    __slots__ = "module",
+    __slots__ = ("module",)
 
     def __init__(self, module, variable_name):
         assert type(variable_name) is str, repr(variable_name)
@@ -303,7 +310,7 @@ class TempVariable(Variable):
         return True
 
 
-def updateFromCollection(old_collection, new_collection):
+def updateVariablesFromCollection(old_collection, new_collection):
     # After removing/adding traces, we need to pre-compute the users state
     # information.
     touched_variables = set()
