@@ -75,6 +75,11 @@ from .ClassCodes import (
     generateBuiltinSuperCode,
     generateSelectMetaclassCode
 )
+from .CodeHelpers import (
+    generateStatementSequenceCode,
+    setExpressionDispatchDict,
+    setStatementDispatchDict
+)
 from .ComparisonCodes import generateComparisonExpressionCode
 from .ConditionalCodes import (
     generateConditionalAndOrCode,
@@ -147,15 +152,11 @@ from .GlobalsLocalsCodes import (
     generateBuiltinVarsCode,
     generateSetLocalsCode
 )
-from .Helpers import (
-    generateStatementSequenceCode,
-    setExpressionDispatchDict,
-    setStatementDispatchDict
-)
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
     generateImportModuleHardCode,
+    generateImportModuleNameHardCode,
     generateImportNameCode,
     generateImportStarCode
 )
@@ -184,6 +185,7 @@ from .LoopCodes import (
 )
 from .ModuleCodes import (
     generateModuleFileAttributeCode,
+    generateModuleLoaderRefCode,
     getModuleCode,
     getModuleValues
 )
@@ -290,6 +292,7 @@ def generateFunctionBodyCode(function_body, context):
             function_identifier    = function_identifier,
             closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
+            outline_variables      = function_body.getOutlineLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
@@ -300,6 +303,7 @@ def generateFunctionBodyCode(function_body, context):
             function_identifier    = function_identifier,
             closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
+            outline_variables      = function_body.getOutlineLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
@@ -310,6 +314,7 @@ def generateFunctionBodyCode(function_body, context):
             function_identifier    = function_identifier,
             closure_variables      = function_body.getClosureVariables(),
             user_variables         = function_body.getUserLocalVariables(),
+            outline_variables      = function_body.getOutlineLocalVariables(),
             temp_variables         = function_body.getTempVariables(),
             needs_exception_exit   = needs_exception_exit,
             needs_generator_return = function_body.needsGeneratorReturnExit()
@@ -321,6 +326,7 @@ def generateFunctionBodyCode(function_body, context):
             parameters           = None,
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
+            outline_variables    = function_body.getOutlineLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
@@ -337,6 +343,7 @@ def generateFunctionBodyCode(function_body, context):
             parameters           = parameters,
             closure_variables    = function_body.getClosureVariables(),
             user_variables       = function_body.getUserLocalVariables(),
+            outline_variables    = function_body.getOutlineLocalVariables(),
             temp_variables       = function_body.getTempVariables(),
             function_doc         = function_body.getDoc(),
             needs_exception_exit = needs_exception_exit,
@@ -468,6 +475,7 @@ def prepareModuleCode(global_context, module, module_name):
         function_decl_codes = function_decl_codes,
         function_body_codes = function_body_codes,
         temp_variables      = module.getTempVariables(),
+        outline_variables   = module.getOutlineLocalVariables(),
         is_main_module      = module.isMainModule(),
         is_internal_module  = module.isInternalModule(),
         context             = context
@@ -608,10 +616,12 @@ setExpressionDispatchDict(
         "EXPRESSION_FUNCTION_CREATION"              : generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL"                  : generateFunctionCallCode,
         "EXPRESSION_IMPORT_MODULE_HARD"             : generateImportModuleHardCode,
+        "EXPRESSION_IMPORT_MODULE_NAME_HARD"        : generateImportModuleNameHardCode,
         "EXPRESSION_IMPORT_NAME"                    : generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND"          : generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_POP"             : generateListOperationPopCode,
         "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"      : generateModuleFileAttributeCode,
+        "EXPRESSION_MODULE_LOADER_REF"              : generateModuleLoaderRefCode,
         "EXPRESSION_MAKE_GENERATOR_OBJECT"          : generateMakeGeneratorObjectCode,
         "EXPRESSION_MAKE_COROUTINE_OBJECT"          : generateMakeCoroutineObjectCode,
         "EXPRESSION_MAKE_ASYNCGEN_OBJECT"           : generateMakeAsyncgenObjectCode,
@@ -628,6 +638,9 @@ setExpressionDispatchDict(
         "EXPRESSION_OPERATION_UNARY"                : generateOperationUnaryCode,
         "EXPRESSION_OPERATION_NOT"                  : generateOperationUnaryCode,
         "EXPRESSION_OUTLINE_BODY"                   : generateFunctionOutlineCode,
+        "EXPRESSION_OUTLINE_FUNCTION"               : generateFunctionOutlineCode,
+        # TODO: Rename to make more clear it is an outline
+        "EXPRESSION_CLASS_BODY"                     : generateFunctionOutlineCode,
         "EXPRESSION_RETURNED_VALUE_REF"             : generateReturnedValueRefCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP"               : generateSubscriptLookupCode,
         "EXPRESSION_SLICE_LOOKUP"                   : generateSliceLookupCode,
