@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,25 +28,25 @@ for a single module only.
 """
 
 import ctypes
+import marshal
 import re
 import struct
 import sys
 from logging import warning
 
-import marshal
 from nuitka import Options
-from nuitka.__past__ import iterItems, long, unicode  # pylint: disable=W0622
-from nuitka.codegen import Emission
-from nuitka.Constants import (
-    constant_builtin_types,
-    getConstantWeight,
-    isMutable
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    iterItems,
+    long,
+    unicode,
+    xrange
 )
+from nuitka.Constants import compareConstants, getConstantWeight, isMutable
+from nuitka.PythonVersions import python_version
 
 from .BlobCodes import StreamData
 from .Emission import SourceCodeCollector
 from .Indentation import indented
-from .Pickling import getStreamedConstant
 from .templates.CodeTemplatesConstants import template_constants_reading
 
 
@@ -60,18 +60,52 @@ def generateConstantReferenceCode(to_name, expression, emit, context):
         context  = context
     )
 
+
+def generateConstantNoneReferenceCode(to_name, expression, emit, context):
+    """ Assign 'None' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=unused-argument
+
+    emit(
+        "%s = Py_None;" % to_name
+    )
+
+
+def generateConstantTrueReferenceCode(to_name, expression, emit, context):
+    """ Assign 'True' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=unused-argument
+
+    emit(
+        "%s = Py_True;" % to_name
+    )
+
+
+def generateConstantFalseReferenceCode(to_name, expression, emit, context):
+    """ Assign 'False' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=unused-argument
+
+    emit(
+        "%s = Py_False;" % to_name
+    )
+
+
+def generateConstantEllipsisReferenceCode(to_name, expression, emit, context):
+    """ Assign 'Ellipsis' to to_name."""
+
+    # No context or other knowledge needed, pylint: disable=unused-argument
+
+    emit(
+        "%s = Py_Ellipsis;" % to_name
+    )
+
+
 # One global stream of constant information. In the future it might make
 # sense to have per module ones, for better locality of indexes within it,
 # but we don't do this yet.
 stream_data = StreamData()
 
-# TODO: This is deprecated, and should be removed.
-def getConstantCode(context, constant):
-    return context.getConstantCode(constant)
-
-def getConstantCodeName(context, constant):
-    return context.getConstantCode(constant)
-
 # TODO: The determination of this should already happen in Building or in a
 # helper not during code generation.
 _match_attribute_names = re.compile(r"[a-zA-Z_][a-zA-Z0-9_]*$")
@@ -81,29 +115,6 @@ def _isAttributeName(value):
     # used for generator expressions, iterator value.
     return _match_attribute_names.match(value) or value == ".0"
 
-
-
-def _getUnstreamCode2(constant_value):
-    saved = getStreamedConstant(
-        constant_value = constant_value
-    )
-    assert type(saved) is bytes
-
-    return stream_data.getStreamDataCode(saved)
-
-
-def _getUnstreamCode(constant_value, constant_identifier):
-    """ Get code to assign given constant value to an identifier from a stream.
-
-        This uses pickle, and usage should be minimized.
-    """
-
-    return "%s = UNSTREAM_CONSTANT( %s );" % (
-        constant_identifier,
-        _getUnstreamCode2(constant_value)
-    )
-
-
 sizeof_long = ctypes.sizeof(ctypes.c_long)
 
 max_unsigned_long = 2**(sizeof_long*8)-1
@@ -122,13 +133,16 @@ def _getConstantInitValueCode(constant_value, constant_type):
         error.
     """
 
-    # This function is a case driven by returns, pylint: disable=R0911
+    # This function is a case driven by returns, pylint: disable=too-many-return-statements
 
     if constant_type is unicode:
+        # Python3: Strings that can be encoded as UTF-8 are done more or less
+        # directly. When they cannot be expressed as UTF-8, that is rare not we
+        # can indeed use pickling.
         try:
             encoded = constant_value.encode("utf-8")
 
-            if str is not unicode:
+            if str is bytes:
                 return "UNSTREAM_UNICODE( %s )" % (
                     stream_data.getStreamDataCode(encoded)
                 )
@@ -142,10 +156,7 @@ def _getConstantInitValueCode(constant_value, constant_type):
             # TODO: try and use "surrogateescape" for this
             return None
     elif constant_type is str:
-        # Python3: Strings that can be encoded as UTF-8 are done more or less
-        # directly. When they cannot be expressed as UTF-8, that is rare not we
-        # can indeed use pickling.
-        assert str is not unicode
+        assert str is bytes
 
         if len(constant_value) == 1:
             return "UNSTREAM_CHAR( %d, %d )" % (
@@ -158,7 +169,7 @@ def _getConstantInitValueCode(constant_value, constant_type):
                 1 if _isAttributeName(constant_value) else 0
             )
     elif constant_type is bytes:
-        assert str is unicode
+        assert str is not bytes
 
         return "UNSTREAM_BYTES( %s )" % (
             stream_data.getStreamDataCode(constant_value)
@@ -175,6 +186,8 @@ def decideMarshal(constant_value):
         them.
     """
 
+    # Many cases to deal with, pylint: disable=too-many-return-statements
+
     constant_type = type(constant_value)
 
     if constant_type is type:
@@ -192,6 +205,10 @@ def decideMarshal(constant_value):
         for element_value in constant_value:
             if not decideMarshal(element_value):
                 return False
+    elif constant_type is xrange:
+        return False
+    elif constant_type is slice:
+        return False
 
     return True
 
@@ -221,8 +238,29 @@ def isMarshalConstant(constant_value):
     marshal_value = marshal.dumps(constant_value)
     restored = marshal.loads(marshal_value)
 
-    # TODO: Potentially warn about these.
-    return constant_value == restored
+    r = compareConstants(constant_value, restored)
+    if not r:
+        pass
+        # TODO: Potentially warn about these, where that is not the case.
+
+    return r
+
+
+def getMarshalCode(constant_identifier, constant_value, emit):
+    """ Force the marshal of a value.
+
+    """
+    marshal_value = marshal.dumps(constant_value)
+    restored = marshal.loads(marshal_value)
+
+    assert compareConstants(constant_value, restored)
+
+    emit(
+        "%s = PyMarshal_ReadObjectFromString( (char *)%s );" % (
+            constant_identifier,
+            stream_data.getStreamDataCode(marshal_value)
+        )
+    )
 
 
 def attemptToMarshal(constant_identifier, constant_value, emit):
@@ -239,7 +277,7 @@ def attemptToMarshal(constant_identifier, constant_value, emit):
 
     # TODO: The check in isMarshalConstant is currently preventing this from
     # happening.
-    if constant_value != restored:
+    if not compareConstants(constant_value, restored):
         warning("Problem with marshal of constant %r", constant_value)
 
         return False
@@ -261,10 +299,6 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
         constants belong into the same scope.
     """
 
-    # This is just a wrapper to make sure that hash values become initialized
-    # for every constant too.
-    if constant_value in constant_builtin_types:
-        return
     if constant_value is None:
         return
     if constant_value is False:
@@ -273,13 +307,15 @@ def _addConstantInitCode(context, emit, check, constant_type, constant_value,
         return
     if constant_value is Ellipsis:
         return
+    if type(constant_value) is type:
+        return
 
     # Do not repeat ourselves.
     if constant_identifier in done:
         return
 
     if Options.shallTraceExecution():
-        emit("""puts("Creating constant: %s");""" % constant_identifier)
+        emit("""NUITKA_PRINT_TRACE("Creating constant: %s");""" % constant_identifier)
 
     # Then it's a real named constant not yet created.
     __addConstantInitCode(context, emit, check, constant_type, constant_value,
@@ -301,14 +337,6 @@ assert( hash_%(constant_identifier)s == DEEP_HASH( %(constant_identifier)s ) );"
              }
         )
 
-        if Options.isExperimental():
-            check(
-                """\
-if ( hash_%(constant_identifier)s == -1 ) puts("Note: Weak hash for: %(constant_identifier)s.");""" % {
-                "constant_identifier" : constant_identifier
-                }
-            )
-
 
 def __addConstantInitCode(context, emit, check, constant_type, constant_value,
                           constant_identifier, module_level):
@@ -318,7 +346,7 @@ def __addConstantInitCode(context, emit, check, constant_type, constant_value,
         constants belong into the same scope.
     """
     # This has many cases, that all return, and do a lot.
-    # pylint: disable=R0911,R0912,R0914,R0915
+    # pylint: disable=too-many-branches,too-many-locals,too-many-return-statements,too-many-statements
 
     # For the module level, we only mean to create constants that are used only
     # inside of it. For the global level, it must must be single use.
@@ -372,10 +400,13 @@ CHECK_OBJECT( const_int_pos_1 );
 
             return
         else:
-            # Note, other longs cannot be handled like that yet. We might create
-            # code that does it better in the future, abusing e.g. internal
-            # representation of "long" integer values.
-            pass
+            getMarshalCode(
+                constant_identifier = constant_identifier,
+                constant_value      = constant_value,
+                emit                = emit
+            )
+
+            return
     elif constant_type is int:
         if constant_value >= min_signed_long:
             emit(
@@ -405,16 +436,10 @@ CHECK_OBJECT( const_int_pos_1 );
             return
 
     if constant_type is unicode:
-        # Attempting to marshal is OK, but esp. Python2 cannot do it for all
-        # "unicode" values.
-
-        if attemptToMarshal(constant_identifier, constant_value, emit):
-            return
-
         try:
             encoded = constant_value.encode("utf-8")
 
-            if str is not unicode:
+            if str is bytes:
                 emit(
                     "%s = UNSTREAM_UNICODE( %s );" % (
                         constant_identifier,
@@ -432,13 +457,19 @@ CHECK_OBJECT( const_int_pos_1 );
 
             return
         except UnicodeEncodeError:
-            # So fall back to below code, which will unstream it then.
-            pass
+            getMarshalCode(
+                constant_identifier = constant_identifier,
+                constant_value      = constant_value,
+                emit                = emit
+            )
+
+            return
+
     elif constant_type is str:
         # Python3: Strings that can be encoded as UTF-8 are done more or less
         # directly. When they cannot be expressed as UTF-8, that is rare not we
         # can indeed use pickling.
-        assert str is not unicode
+        assert str is bytes
 
         if len(constant_value) == 1:
             emit(
@@ -460,7 +491,7 @@ CHECK_OBJECT( const_int_pos_1 );
         return
     elif constant_type is bytes:
         # Python3 only, for Python2, bytes do not happen.
-        assert str is unicode
+        assert str is not bytes
 
         emit(
             "%s = UNSTREAM_BYTES( %s );" % (
@@ -499,7 +530,7 @@ CHECK_OBJECT( const_int_pos_1 );
         )
 
         for key, value in iterItems(constant_value):
-            key_name = getConstantCodeName(context, key)
+            key_name = context.getConstantCode(key)
             _addConstantInitCode(
                 emit                = emit,
                 check               = check,
@@ -510,7 +541,7 @@ CHECK_OBJECT( const_int_pos_1 );
                 context             = context
             )
 
-            value_name = getConstantCodeName(context, value)
+            value_name = context.getConstantCode(value)
             _addConstantInitCode(
                 emit                = emit,
                 check               = check,
@@ -554,8 +585,7 @@ CHECK_OBJECT( const_int_pos_1 );
         )
 
         for count, element_value in enumerate(constant_value):
-            element_name = getConstantCodeName(
-                context  = context,
+            element_name = context.getConstantCode(
                 constant = element_value
             )
 
@@ -564,8 +594,7 @@ CHECK_OBJECT( const_int_pos_1 );
                 check               = check,
                 constant_type       = type(element_value),
                 constant_value      = element_value,
-                constant_identifier = getConstantCodeName(
-                    context  = context,
+                constant_identifier = context.getConstantCode(
                     constant = element_value
                 ),
                 module_level        = module_level,
@@ -599,8 +628,7 @@ CHECK_OBJECT( const_int_pos_1 );
         )
 
         for count, element_value in enumerate(constant_value):
-            element_name = getConstantCodeName(
-                context  = context,
+            element_name = context.getConstantCode(
                 constant = element_value
             )
 
@@ -626,7 +654,7 @@ CHECK_OBJECT( const_int_pos_1 );
 
         return
 
-    if constant_type is set:
+    if constant_type is set or constant_type is frozenset:
         # Not all sets can or should be marshaled. For small ones,
         # or ones with strange values, like "{type}", we have to do it.
         if attemptToMarshal(constant_identifier, constant_value, emit):
@@ -634,14 +662,14 @@ CHECK_OBJECT( const_int_pos_1 );
 
         # TODO: Hinting size is really not possible?
         emit(
-            "%s = PySet_New( NULL );" % constant_identifier
+            "%s = %s( NULL );" % (
+                constant_identifier,
+                "PySet_New" if constant_type is set else "PyFrozenSet_New"
+            )
         )
 
         for element_value in constant_value:
-            element_name = getConstantCodeName(
-                context  = context,
-                constant = element_value
-            )
+            element_name = context.getConstantCode(element_value)
 
             _addConstantInitCode(
                 emit                = emit,
@@ -670,7 +698,7 @@ CHECK_OBJECT( const_int_pos_1 );
         return
 
     if constant_type is slice:
-        slice1_name = getConstantCodeName(context, constant_value.start)
+        slice1_name = context.getConstantCode(constant_value.start)
         _addConstantInitCode(
             emit                = emit,
             check               = check,
@@ -680,7 +708,7 @@ CHECK_OBJECT( const_int_pos_1 );
             module_level        = module_level,
             context             = context
         )
-        slice2_name = getConstantCodeName(context, constant_value.stop)
+        slice2_name = context.getConstantCode(constant_value.stop)
         _addConstantInitCode(
             emit                = emit,
             check               = check,
@@ -690,7 +718,7 @@ CHECK_OBJECT( const_int_pos_1 );
             module_level        = module_level,
             context             = context
         )
-        slice3_name = getConstantCodeName(context, constant_value.step)
+        slice3_name = context.getConstantCode(constant_value.step)
         _addConstantInitCode(
             emit                = emit,
             check               = check,
@@ -712,16 +740,96 @@ CHECK_OBJECT( const_int_pos_1 );
 
         return
 
-    # TODO: Ranges could very well be created for Python3. And "frozenset" and
-    # set, are to be examined.
+    if constant_type is xrange:
+        # Strip const_xrange.
+        assert constant_identifier.startswith("const_xrange_")
 
-    if constant_type in (frozenset, complex, unicode, long, range):
-        # Lets attempt marshal these.
-        if attemptToMarshal(constant_identifier, constant_value, emit):
-            return
+        # For Python2, xrange needs only long values to be created, so avoid objects.
+        range_args =  constant_identifier[13:].split('_')
 
+        # Default start.
+        if len(range_args) == 1:
+            range_args.insert(0, '0')
+
+        # Default step
+        if len(range_args) < 3:
+            range_args.append('1')
+
+
+        # Negative values are encoded with "neg" prefix.
+        range_args = [
+            int(range_arg.replace("neg", '-'))
+            for range_arg in
+            range_args
+        ]
+
+        if xrange is not range:
+            emit(
+                 "%s = MAKE_XRANGE( %s, %s, %s );" % (
+                    constant_identifier,
+                    range_args[0],
+                    range_args[1],
+                    range_args[2]
+                )
+            )
+        else:
+            range1_name = context.getConstantCode(range_args[0])
+            _addConstantInitCode(
+                emit                = emit,
+                check               = check,
+                constant_type       = type(range_args[0]),
+                constant_value      = range_args[0],
+                constant_identifier = range1_name,
+                module_level        = module_level,
+                context             = context
+            )
+            range2_name = context.getConstantCode(range_args[1])
+            _addConstantInitCode(
+                emit                = emit,
+                check               = check,
+                constant_type       = type(range_args[1]),
+                constant_value      = range_args[1],
+                constant_identifier = range2_name,
+                module_level        = module_level,
+                context             = context
+            )
+            range3_name = context.getConstantCode(range_args[2])
+            _addConstantInitCode(
+                emit                = emit,
+                check               = check,
+                constant_type       = type(range_args[2]),
+                constant_value      = range_args[2],
+                constant_identifier = range3_name,
+                module_level        = module_level,
+                context             = context
+            )
+
+            emit(
+                 "%s = BUILTIN_XRANGE3( %s, %s, %s );" % (
+                    constant_identifier,
+                    range1_name,
+                    range2_name,
+                    range3_name
+                )
+            )
+
+        return
+
+    if constant_type is bytearray:
         emit(
-            _getUnstreamCode(constant_value, constant_identifier)
+            "%s = UNSTREAM_BYTEARRAY( %s );" % (
+                constant_identifier,
+                stream_data.getStreamDataCode(bytes(constant_value))
+            )
+        )
+
+        return
+
+    if constant_type is complex:
+        getMarshalCode(
+            constant_identifier = constant_identifier,
+            constant_value      = constant_value,
+            emit                = emit
         )
 
         return
@@ -766,8 +874,6 @@ def getConstantsDeclCode(context):
 
     for constant_identifier, constant_value in sorted_constants:
         # Need not declare built-in types.
-        if constant_value in constant_builtin_types:
-            continue
         if constant_value is None:
             continue
         if constant_value is False:
@@ -776,6 +882,8 @@ def getConstantsDeclCode(context):
             continue
         if constant_value is Ellipsis:
             continue
+        if type(constant_value) is type:
+            continue
 
         if context.getConstantUseCount(constant_identifier) != 1:
             statements.append("PyObject *%s;" % constant_identifier)
@@ -788,7 +896,7 @@ def getConstantsDeclCode(context):
 
 def getConstantAccess(to_name, constant, emit, context):
     # Many cases, because for each type, we may copy or optimize by creating
-    # empty.  pylint: disable=R0912,R0915
+    # empty.  pylint: disable=too-many-branches,too-many-statements
 
     if type(constant) is dict:
         if constant:
@@ -802,25 +910,16 @@ def getConstantAccess(to_name, constant, emit, context):
                 needs_deep = False
 
             if needs_deep:
-                code = "DEEP_COPY( %s )" % getConstantCode(
-                    constant = constant,
-                    context  = context
-                )
+                code = "DEEP_COPY( %s )" % context.getConstantCode(constant)
             else:
-                code = "PyDict_Copy( %s )" % getConstantCode(
-                    constant = constant,
-                    context  = context
-                )
+                code = "PyDict_Copy( %s )" % context.getConstantCode(constant)
         else:
             code = "PyDict_New()"
 
         ref_count = 1
     elif type(constant) is set:
         if constant:
-            code = "PySet_New( %s )" % getConstantCode(
-                constant = constant,
-                context  = context
-            )
+            code = "PySet_New( %s )" % context.getConstantCode(constant)
         else:
             code = "PySet_New( NULL )"
 
@@ -835,15 +934,9 @@ def getConstantAccess(to_name, constant, emit, context):
                 needs_deep = False
 
             if needs_deep:
-                code = "DEEP_COPY( %s )" % getConstantCode(
-                    constant = constant,
-                    context  = context
-                )
+                code = "DEEP_COPY( %s )" % context.getConstantCode(constant)
             else:
-                code = "LIST_COPY( %s )" % getConstantCode(
-                    constant = constant,
-                    context  = context
-                )
+                code = "LIST_COPY( %s )" % context.getConstantCode(constant)
         else:
             code = "PyList_New( 0 )"
 
@@ -857,22 +950,18 @@ def getConstantAccess(to_name, constant, emit, context):
             needs_deep = False
 
         if needs_deep:
-            code = "DEEP_COPY( %s )" % getConstantCode(
-                 constant = constant,
-                 context  = context
-            )
+            code = "DEEP_COPY( %s )" % context.getConstantCode(constant)
 
             ref_count = 1
         else:
-            code = getConstantCode(
-                context  = context,
-                constant = constant
-            )
+            code = context.getConstantCode(constant)
 
             ref_count = 0
+    elif type(constant) is bytearray:
+        code = "BYTEARRAY_COPY( %s )"  % context.getConstantCode(constant)
+        ref_count = 1
     else:
-        code = getConstantCode(
-            context  = context,
+        code = context.getConstantCode(
             constant = constant
         )
 
@@ -906,8 +995,8 @@ constant_counts = {}
 
 def getConstantInitCodes(module_context):
     decls = []
-    inits = Emission.SourceCodeCollector()
-    checks = Emission.SourceCodeCollector()
+    inits = SourceCodeCollector()
+    checks = SourceCodeCollector()
 
     sorted_constants = sorted(
         module_context.getConstants(),
@@ -956,12 +1045,14 @@ def getConstantInitCodes(module_context):
 
 
 def allocateNestedConstants(module_context):
+    # Lots of types to deal with, pylint: disable=too-many-branches
+
     def considerForDeferral(constant_value):
+        module_context.getConstantCode(constant_value)
+
         if isMarshalConstant(constant_value):
             return
 
-        module_context.getConstantCode(constant_value)
-
         constant_type = type(constant_value)
 
         if constant_type in (tuple, list, set, frozenset):
@@ -975,6 +1066,27 @@ def allocateNestedConstants(module_context):
             considerForDeferral(constant_value.start)
             considerForDeferral(constant_value.step)
             considerForDeferral(constant_value.stop)
+        elif constant_type is xrange:
+            if xrange is range:
+                if python_version >= 330:
+                    # For Python2 ranges, we use C long values directly.
+                    considerForDeferral(constant_value.start)
+                    considerForDeferral(constant_value.step)
+                    considerForDeferral(constant_value.stop)
+                else:
+                    parts = [
+                        int(value)
+                        for value in
+                        str(constant_value)[6:-1].split(',')
+                    ]
+
+                    if len(parts) <= 1:
+                        parts.append(0)
+                    if len(parts) <= 2:
+                        parts.append(1)
+
+                    for value in parts:
+                        considerForDeferral(value)
 
     for constant_identifier in set(module_context.getConstants()):
         constant_value = module_context.global_context.constants[
@@ -983,12 +1095,12 @@ def allocateNestedConstants(module_context):
 
         constant_type = type(constant_value)
 
-        if constant_type in (tuple, dict, list, set, frozenset, slice):
+        if constant_type in (tuple, dict, list, set, frozenset, slice, xrange):
             considerForDeferral(constant_value)
 
 
 def getConstantsDefinitionCode(context):
-    """ Create the code code "__constants.cpp" file.
+    """ Create the code code "__constants.c" file.
 
         This needs to create code to make all global constants (used in more
         than one module) and create them.
