@@ -19,6 +19,10 @@
 
 """
 
+from .CodeHelpers import (
+    generateChildExpressionsCode,
+    generateStatementSequenceCode
+)
 from .Emission import SourceCodeCollector
 from .ErrorCodes import getErrorExitCode, getReleaseCode
 from .FunctionCodes import (
@@ -26,10 +30,6 @@ from .FunctionCodes import (
     setupFunctionLocalVariables
 )
 from .GeneratorCodes import getClosureCopyCode
-from .Helpers import (
-    generateChildExpressionsCode,
-    generateStatementSequenceCode
-)
 from .Indentation import indented
 from .LineNumberCodes import emitLineNumberUpdateCode
 from .PythonAPICodes import getReferenceExportCode
@@ -50,13 +50,14 @@ def getCoroutineObjectDeclCode(function_identifier):
 
 
 def getCoroutineObjectCode(context, function_identifier, closure_variables,
-                           user_variables, temp_variables, needs_exception_exit,
+                           user_variables, outline_variables,
+                           temp_variables, needs_exception_exit,
                            needs_generator_return):
     function_locals, function_cleanup = setupFunctionLocalVariables(
         context           = context,
         parameters        = None,
         closure_variables = closure_variables,
-        user_variables    = user_variables,
+        user_variables    = user_variables + outline_variables,
         temp_variables    = temp_variables
     )
 
@@ -69,17 +70,17 @@ def getCoroutineObjectCode(context, function_identifier, closure_variables,
         context            = context
     )
 
-    function_locals += finalizeFunctionLocalVariables(context)
+    finalizeFunctionLocalVariables(context, function_locals, function_cleanup)
 
     if needs_exception_exit:
         generator_exit = template_coroutine_exception_exit % {
             "function_identifier" : function_identifier,
-            "function_cleanup"    : function_cleanup
+            "function_cleanup"    : indented(function_cleanup)
         }
     else:
         generator_exit = template_coroutine_noexception_exit % {
             "function_identifier" : function_identifier,
-            "function_cleanup"    : function_cleanup
+            "function_cleanup"    : indented(function_cleanup)
         }
 
     if needs_generator_return:
@@ -175,8 +176,10 @@ def generateAsyncIterCode(to_name, expression, emit, context):
     )
 
     emit(
-        "%s = MAKE_ASYNC_ITERATOR( coroutine, %s );" % (
+        "%s = %s_ASYNC_MAKE_ITERATOR( %s, %s );" % (
             to_name,
+            context.getContextObjectName().upper(),
+            context.getContextObjectName(),
             value_name
         )
     )
@@ -204,8 +207,10 @@ def generateAsyncNextCode(to_name, expression, emit, context):
     )
 
     emit(
-        "%s = ASYNC_ITERATOR_NEXT( coroutine, %s );" % (
+        "%s = %s_ASYNC_ITERATOR_NEXT( %s, %s );" % (
             to_name,
+            context.getContextObjectName().upper(),
+            context.getContextObjectName(),
             value_name,
         )
     )
