@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,8 +22,8 @@
 from nuitka import Options
 from nuitka.PythonVersions import python_version
 
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
 from .ErrorCodes import getErrorExitBoolCode, getErrorExitCode, getReleaseCodes
-from .Helpers import generateChildExpressionsCode, generateExpressionCode
 
 
 def generateBuiltinDictCode(to_name, expression, emit, context):
@@ -139,31 +139,18 @@ def getDictionaryCreationCode(to_name, pairs, emit, context):
             generateKeyCode(dict_key_name, pair)
             generateValueCode(dict_value_name, pair)
 
-        if pair.getKey().isKnownToBeHashable():
-            emit(
-                "PyDict_SetItem( %s, %s, %s );" % (
-                    to_name,
-                    dict_key_name,
-                    dict_value_name
-                )
-            )
-        else:
-            res_name = context.getIntResName()
+        needs_check = not pair.getKey().isKnownToBeHashable()
 
-            emit(
-                "%s = PyDict_SetItem( %s, %s, %s );" % (
-                    res_name,
-                    to_name,
-                    dict_key_name,
-                    dict_value_name
-                )
-            )
+        res_name = context.getIntResName()
 
-            getErrorExitBoolCode(
-                condition = "%s != 0" % res_name,
-                emit      = emit,
-                context   = context
+        emit(
+            "%s = PyDict_SetItem( %s, %s, %s );" % (
+                res_name,
+                to_name,
+                dict_key_name,
+                dict_value_name
             )
+        )
 
         if context.needsCleanup(dict_value_name):
             emit("Py_DECREF( %s );" % dict_value_name)
@@ -173,6 +160,14 @@ def getDictionaryCreationCode(to_name, pairs, emit, context):
             emit("Py_DECREF( %s );" % dict_key_name)
             context.removeCleanupTempName(dict_key_name)
 
+        getErrorExitBoolCode(
+            condition   = "%s != 0" % res_name,
+            needs_check = needs_check,
+            emit        = emit,
+            context     = context
+        )
+
+
 
 def generateDictOperationUpdateCode(statement, emit, context):
     value_arg_name = context.allocateTempName("dictupdate_value", unique = True)
@@ -209,7 +204,7 @@ def generateDictOperationUpdateCode(statement, emit, context):
     )
 
     getErrorExitBoolCode(
-        condition   = "%s == -1" % res_name,
+        condition   = "%s != 0" % res_name,
         needs_check = statement.mayRaiseException(BaseException),
         emit        = emit,
         context     = context
