@@ -58,6 +58,19 @@ def getErrorExitReleaseCode(context):
     return temp_release
 
 
+def getFrameVariableTypeDescriptionCode(context):
+
+    type_description = context.getFrameVariableTypeDescription()
+
+    if type_description:
+        return '%s = "%s";' % (
+            context.getFrameVariableTypeDescriptionName(),
+            type_description,
+        )
+    else:
+        return ""
+
+
 def getErrorExitBoolCode(condition, emit, context,
                          needs_check = True, quick_exception = None):
     assert not condition.endswith(';')
@@ -72,13 +85,16 @@ def getErrorExitBoolCode(condition, emit, context,
         emit(
             indented(
                 template_error_catch_quick_exception % {
-                    "condition"        : condition,
-                    "exception_exit"   : context.getExceptionEscape(),
-                    "quick_exception"  : getExceptionIdentifier(quick_exception),
-                    "release_temps"    : indented(
+                    "condition"            : condition,
+                    "exception_exit"       : context.getExceptionEscape(),
+                    "quick_exception"      : getExceptionIdentifier(quick_exception),
+                    "release_temps"        : indented(
                         getErrorExitReleaseCode(context)
                     ),
-                    "line_number_code" : indented(
+                    "var_description_code" : indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
+                    "line_number_code"     : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
                 },
@@ -94,6 +110,9 @@ def getErrorExitBoolCode(condition, emit, context,
                     "release_temps"    : indented(
                         getErrorExitReleaseCode(context)
                     ),
+                    "var_description_code": indented(
+                        getFrameVariableTypeDescriptionCode(context),
+                    ),
                     "line_number_code" : indented(
                         getErrorLineNumberUpdateCode(context)
                     )
@@ -169,17 +188,22 @@ def getErrorFormatExitBoolCode(condition, exception, args, emit, context):
             "release_temps"    : indented(
                 getErrorExitReleaseCode(context)
             ),
+            "var_description_code" : indented(
+                getFrameVariableTypeDescriptionCode(context),
+            ),
             "line_number_code" : indented(
                 getErrorLineNumberUpdateCode(context)
             )
         }
     )
 
+
 def getErrorVariableDeclarations():
     return (
-        "PyObject *exception_type = NULL, *exception_value = NULL;",
+        "PyObject *exception_type = NULL;",
+        "PyObject *exception_value = NULL;",
         "PyTracebackObject *exception_tb = NULL;",
-        "NUITKA_MAY_BE_UNUSED int exception_lineno = -1;"
+        "NUITKA_MAY_BE_UNUSED int exception_lineno = 0;"
     )
 
 def getExceptionKeeperVariableNames(keeper_index):
@@ -207,7 +231,7 @@ def getExceptionKeeperVariableNames(keeper_index):
         ),
         "NUITKA_MAY_BE_UNUSED int exception_keeper_lineno_%d%s;" % (
             keeper_index,
-            " = -1" if debug else ""
+            " = 0" if debug else ""
         )
     )
 
@@ -268,7 +292,7 @@ def getReleaseCodes(release_names, emit, context):
 def getMustNotGetHereCode(reason, context, emit):
     getCommentCode(reason, emit)
 
-    provider = context.getOwner()
+    provider = context.getEntryPoint()
 
     emit(
         "NUITKA_CANNOT_GET_HERE( %(function_identifier)s );" % {
@@ -280,6 +304,8 @@ def getMustNotGetHereCode(reason, context, emit):
         emit("return;")
     elif provider.isExpressionCoroutineObjectBody():
         emit("return;")
+    elif provider.isExpressionAsyncgenObjectBody():
+        emit("return;")
     elif provider.isCompiledPythonModule():
         emit("return MOD_RETURN_VALUE( NULL );")
     else:
@@ -288,3 +314,58 @@ def getMustNotGetHereCode(reason, context, emit):
 
 def getAssertionCode(check, emit):
     emit("assert( %s );" % check)
+
+
+def getCheckObjectCode(check_name, emit):
+    emit("CHECK_OBJECT( %s );" % check_name)
+
+
+def getLocalVariableReferenceErrorCode(variable, condition, emit, context):
+    if variable.getOwner() is not context.getOwner():
+        getErrorFormatExitBoolCode(
+            condition = condition,
+            exception = "PyExc_NameError",
+            args      = (
+                """\
+free variable '%s' referenced before assignment in enclosing scope""",
+                variable.getName()
+            ),
+            emit      = emit,
+            context   = context
+        )
+    else:
+        getErrorFormatExitBoolCode(
+            condition = condition,
+            exception = "PyExc_UnboundLocalError",
+            args      = (
+                """\
+local variable '%s' referenced before assignment""",
+                variable.getName()
+            ),
+            emit      = emit,
+            context   = context
+        )
+
+
+def getNameReferenceErrorCode(variable_name, condition, emit, context):
+    if python_version < 340:
+        owner = context.getOwner()
+
+        if not owner.isCompiledPythonModule() and \
+           not owner.isExpressionClassBody():
+            error_message = "global name '%s' is not defined"
+        else:
+            error_message = "name '%s' is not defined"
+    else:
+        error_message = "name '%s' is not defined"
+
+    getErrorFormatExitBoolCode(
+        condition = condition,
+        exception = "PyExc_NameError",
+        args      = (
+            error_message,
+            variable_name
+        ),
+        emit      = emit,
+        context   = context
+    )
