@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,8 +21,9 @@
 
 from nuitka.PythonVersions import python_version
 
-from .ConstantCodes import getConstantCode
-from .CoroutineCodes import getCoroutineObjectDeclCode
+from .c_types.CTypePyObjectPtrs import CTypeCellObject, CTypePyObjectPtrPtr
+from .CodeHelpers import generateExpressionCode, generateStatementSequenceCode
+from .Contexts import PythonFunctionOutlineContext
 from .Emission import SourceCodeCollector
 from .ErrorCodes import (
     getErrorExitCode,
@@ -32,26 +33,24 @@ from .ErrorCodes import (
     getMustNotGetHereCode,
     getReleaseCode
 )
-from .GeneratorCodes import getGeneratorObjectDeclCode
-from .Helpers import generateExpressionCode
 from .Indentation import indented
-from .LabelCodes import getLabelCode
+from .LabelCodes import getGotoCode, getLabelCode
+from .LineNumberCodes import emitErrorLineNumberUpdateCode
 from .ModuleCodes import getModuleAccessCode
 from .PythonAPICodes import getReferenceExportCode
 from .templates.CodeTemplatesFunction import (
     function_dict_setup,
     function_direct_body_template,
     template_function_body,
-    template_function_closure_making,
     template_function_direct_declaration,
     template_function_exception_exit,
     template_function_make_declaration,
     template_function_return_exit,
-    template_make_function_with_context_template,
-    template_make_function_without_context_template
+    template_make_function_template
 )
 from .TupleCodes import getTupleCreationCode
 from .VariableCodes import (
+    getLocalVariableCodeType,
     getLocalVariableInitCode,
     getVariableCode,
     getVariableCodeName
@@ -61,11 +60,12 @@ from .VariableCodes import (
 def getClosureVariableProvisionCode(context, closure_variables):
     result = []
 
-    for variable in closure_variables:
+    for variable, version in closure_variables:
         result.append(
             getVariableCode(
                 context  = context,
-                variable = variable
+                variable = variable,
+                version  = version
             )
         )
 
@@ -87,7 +87,7 @@ def _getFunctionCreationArgs(defaults_name, kw_defaults_name,
 
     for closure_variable in closure_variables:
         result.append(
-            "PyCellObject *%s" % (
+            "struct Nuitka_CellObject *%s" % (
                 getVariableCodeName(
                     variable   = closure_variable,
                     in_context = True
@@ -125,7 +125,7 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
                          kw_defaults_name, annotations_name, function_doc,
                          context):
     # We really need this many parameters here and functions have many details,
-    # that we express as variables, pylint: disable=R0914
+    # that we express as variables, pylint: disable=too-many-locals
     function_creation_args = _getFunctionCreationArgs(
         defaults_name     = defaults_name,
         kw_defaults_name  = kw_defaults_name,
@@ -136,105 +136,65 @@ def getFunctionMakerCode(function_name, function_qualname, function_identifier,
     if python_version < 330 or function_qualname == function_name:
         function_qualname_obj = "NULL"
     else:
-        function_qualname_obj = getConstantCode(
-            constant = function_qualname,
-            context  = context
+        function_qualname_obj = context.getConstantCode(
+            constant = function_qualname
         )
 
-    if closure_variables:
-        closure_copy = []
+    closure_copy = []
 
-        for count, closure_variable in enumerate(closure_variables):
-            closure_copy.append(
-                "closure[%d] = %s;" % (
-                    count,
-                    getVariableCodeName(
-                        True,
-                        closure_variable
-                    )
+    for count, closure_variable in enumerate(closure_variables):
+        closure_copy.append(
+            "result->m_closure[%d] = %s;" % (
+
+                count,
+                getVariableCodeName(
+                    True,
+                    closure_variable
                 )
             )
-            closure_copy.append(
-                "Py_INCREF( closure[%d] );" %count
-            )
+        )
+        closure_copy.append(
+            "Py_INCREF( result->m_closure[%d] );" %count
+        )
 
-        closure_making = template_function_closure_making % {
-            "closure_copy"  : indented(closure_copy),
-            "closure_count" : len(closure_variables)
-        }
-
-        result = template_make_function_with_context_template % {
-            "function_name_obj"          : getConstantCode(
-                constant = function_name,
-                context  = context
-            ),
-            "function_qualname_obj"      : function_qualname_obj,
-            "function_identifier"        : function_identifier,
-            "function_impl_identifier" : getFunctionEntryPointIdentifier(
-                function_identifier = function_identifier,
-            ),
-            "function_creation_args"     : ", ".join(
-                function_creation_args
-            ),
-            "code_identifier"            : code_identifier,
-            "closure_making"             : closure_making,
-            "function_doc"               : getConstantCode(
-                constant = function_doc,
-                context  = context
-            ),
-            "defaults"                   : "defaults"
-                                             if defaults_name else
-                                           "NULL",
-            "kw_defaults"                : "kw_defaults"
-                                             if kw_defaults_name else
-                                           "NULL",
-            "annotations"                : "annotations"
-                                             if annotations_name else
-                                           context.getConstantCode({}),
-            "closure_count"              : len(closure_variables),
-            "module_identifier"          : getModuleAccessCode(
-                context = context
-            ),
-        }
-    else:
-        result = template_make_function_without_context_template % {
-            "function_name_obj"          : getConstantCode(
-                constant = function_name,
-                context  = context
-            ),
-            "function_qualname_obj"      : function_qualname_obj,
-            "function_identifier"        : function_identifier,
-            "function_impl_identifier" : getFunctionEntryPointIdentifier(
-                function_identifier = function_identifier,
-            ),
-            "function_creation_args"     : ", ".join(
-                function_creation_args
-            ),
-            "code_identifier"            : code_identifier,
-            "function_doc"               : getConstantCode(
-                constant = function_doc,
-                context  = context
-            ),
-            "defaults"                   : "defaults"
-                                             if defaults_name else
-                                           "NULL",
-            "kw_defaults"                : "kw_defaults"
-                                             if kw_defaults_name else
-                                           "NULL",
-            "annotations"                : "annotations"
-                                             if annotations_name else
-                                           context.getConstantCode({}),
-            "module_identifier"          : getModuleAccessCode(
-                context = context
-            ),
-        }
+    result = template_make_function_template % {
+        "function_name_obj"          : context.getConstantCode(
+            constant = function_name
+        ),
+        "function_qualname_obj"      : function_qualname_obj,
+        "function_identifier"        : function_identifier,
+        "function_impl_identifier"   : getFunctionEntryPointIdentifier(
+            function_identifier = function_identifier,
+        ),
+        "function_creation_args"     : ", ".join(
+            function_creation_args
+        ),
+        "code_identifier"            : code_identifier,
+        "closure_copy"               : indented(closure_copy, 0, True),
+        "function_doc"               : context.getConstantCode(
+            constant = function_doc
+        ),
+        "defaults"                   : "defaults"
+                                         if defaults_name else
+                                       "NULL",
+        "kw_defaults"                : "kw_defaults"
+                                         if kw_defaults_name else
+                                       "NULL",
+        "annotations"                : "annotations"
+                                         if annotations_name else
+                                       context.getConstantCode({}),
+        "closure_count"              : len(closure_variables),
+        "module_identifier"          : getModuleAccessCode(
+            context = context
+        ),
+    }
 
     return result
 
 
 def generateFunctionCreationCode(to_name, expression, emit, context):
     # This is about creating functions, which is detail ridden stuff,
-    # pylint: disable=R0914
+    # pylint: disable=too-many-locals
 
     function_body  = expression.getFunctionRef().getFunctionBody()
     code_object    = expression.getCodeObject()
@@ -301,21 +261,13 @@ def generateFunctionCreationCode(to_name, expression, emit, context):
 
     # Creation code needs to be done only once.
     if not context.hasHelperCode(function_identifier):
-        code_identifier = context.getCodeObjectHandle(
-            code_object  = code_object,
-            filename     = function_body.getParentModule().getRunTimeFilename(),
-            line_number  = function_body.getSourceReference().getLineNumber(),
-            is_optimized = not function_body.needsLocalsDict(),
-            new_locals   = True,
-            has_closure  = function_body.getClosureVariables() != (),
-            future_flags = function_body.getSourceReference().getFutureSpec().asFlags()
-        )
-
         maker_code = getFunctionMakerCode(
             function_name       = function_body.getFunctionName(),
             function_qualname   = function_body.getFunctionQualname(),
             function_identifier = function_identifier,
-            code_identifier     = code_identifier,
+            code_identifier     = context.getCodeObjectHandle(
+                code_object = code_object,
+            ),
             closure_variables   = function_body.getClosureVariables(),
             defaults_name       = defaults_name,
             kw_defaults_name    = kw_defaults_name,
@@ -342,7 +294,7 @@ def generateFunctionCreationCode(to_name, expression, emit, context):
         defaults_name       = defaults_name,
         kw_defaults_name    = kw_defaults_name,
         annotations_name    = annotations_name,
-        closure_variables   = function_body.getClosureVariables(),
+        closure_variables   = expression.getClosureVariableVersions(),
         emit                = emit,
         context             = context
     )
@@ -360,7 +312,8 @@ def getFunctionCreationCode(to_name, function_identifier, defaults_name,
     args = []
 
     if defaults_name is not None:
-        args.append(getReferenceExportCode(defaults_name, context))
+        getReferenceExportCode(defaults_name, emit, context)
+        args.append(defaults_name)
 
     if kw_defaults_name is not None:
         args.append(kw_defaults_name)
@@ -396,10 +349,18 @@ def getDirectFunctionCallCode(to_name, function_identifier, arg_names,
         function_identifier = function_identifier
     )
 
-    suffix_args = getClosureVariableProvisionCode(
-        context           = context,
-        closure_variables = closure_variables
-    )
+    suffix_args = []
+
+    for closure_variable, closure_variable_version in closure_variables:
+        variable_code_name, variable_c_type = getLocalVariableCodeType(
+            context  = context,
+            variable = closure_variable,
+            version  = closure_variable_version
+        )
+
+        suffix_args.append(
+            variable_c_type.getVariableArgReferencePassingCode(variable_code_name)
+        )
 
     # TODO: We ought to not assume references for direct calls, or make a
     # profile if an argument needs a reference at all. Most functions don't
@@ -454,41 +415,21 @@ def getDirectFunctionCallCode(to_name, function_identifier, arg_names,
     context.addCleanupTempName(to_name)
 
 
-def getFunctionDirectClosureArgs(closure_variables):
-    result = []
-
-    for closure_variable in closure_variables:
-        if closure_variable.isSharedTechnically():
-            result.append(
-                "PyCellObject *%s" % (
-                    getVariableCodeName(
-                        in_context = True,
-                        variable   = closure_variable
-                    )
-                )
-            )
-        else:
-            # TODO: The reference is only needed for Python3, could make it
-            # version dependent.
-            result.append(
-                "PyObject *&%s" % (
-                    getVariableCodeName(
-                        in_context = True,
-                        variable   = closure_variable
-                    )
-                )
-            )
-
-    return result
-
-
-def getFunctionDirectDecl(function_identifier, closure_variables, file_scope):
-
+def getFunctionDirectDecl(function_identifier, closure_variables, file_scope, context):
     parameter_objects_decl = [
         "PyObject **python_pars"
     ]
 
-    parameter_objects_decl += getFunctionDirectClosureArgs(closure_variables)
+    for closure_variable in closure_variables:
+        variable_code_name, variable_c_type = getLocalVariableCodeType(
+            context  = context,
+            variable = closure_variable,
+            version  = 0
+        )
+
+        parameter_objects_decl.append(
+            variable_c_type.getVariableArgDeclarationCode(variable_code_name)
+        )
 
     result = template_function_direct_declaration % {
         "file_scope"           : file_scope,
@@ -499,26 +440,22 @@ def getFunctionDirectDecl(function_identifier, closure_variables, file_scope):
     return result
 
 
-def getFunctionCode(context, function_identifier, parameters, closure_variables,
-                    user_variables, temp_variables, function_codes, function_doc,
-                    file_scope, needs_exception_exit):
-
-    # Functions have many details, that we express as variables, with many
-    # branches to decide, pylint: disable=R0912,R0914
+def setupFunctionLocalVariables(context, parameters, closure_variables,
+                                user_variables, temp_variables):
 
     function_locals = []
+    function_cleanup = []
 
     if context.hasLocalsDict():
-        function_locals += function_dict_setup.split('\n')
-        function_cleanup = "Py_DECREF( locals_dict );\n"
-    else:
-        function_cleanup = ""
+        context.allocateLocalsDictName()
 
     if parameters is not None:
         for count, variable in enumerate(parameters.getAllVariables()):
             function_locals.append(
                 getLocalVariableInitCode(
+                    context   = context,
                     variable  = variable,
+                    version   = 0,
                     init_from = "python_pars[ %d ]" % count
                 )
             )
@@ -526,16 +463,42 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
     # User local variable initializations
     function_locals += [
         getLocalVariableInitCode(
-            variable = variable,
+            context   = context,
+            variable  = variable,
+            version   = 0,
+            init_from = None
         )
         for variable in
         user_variables + tuple(
             variable
             for variable in
-            temp_variables
+            sorted(
+                temp_variables,
+                key = lambda variable: variable.getName()
+            )
         )
     ]
 
+    for closure_variable in closure_variables:
+        # Temporary variable closures are to be ignored.
+        if closure_variable.isTempVariable():
+            continue
+
+        variable_code_name, variable_c_type = getLocalVariableCodeType(
+            context  = context,
+            variable = closure_variable,
+            version  = 0
+        )
+
+        if variable_c_type in (CTypeCellObject, CTypePyObjectPtrPtr):
+            context.setVariableType(closure_variable, variable_code_name, variable_c_type)
+        else:
+            assert False, (variable_code_name, variable_c_type)
+
+    return function_locals, function_cleanup
+
+
+def finalizeFunctionLocalVariables(context, function_locals, function_cleanup):
     if context.needsExceptionVariables():
         function_locals.extend(getErrorVariableDeclarations())
 
@@ -545,6 +508,8 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
     for preserver_id in context.getExceptionPreserverCounts():
         function_locals.extend(getExceptionPreserverVariableNames(preserver_id))
 
+    tmp_infos = context.getTempNameInfos()
+
     function_locals += [
         "%s%s%s;" % (
             tmp_type,
@@ -552,7 +517,7 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
             tmp_name
         )
         for tmp_name, tmp_type in
-        context.getTempNameInfos()
+        tmp_infos
     ]
 
     function_locals += context.getFrameDeclarations()
@@ -561,12 +526,53 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
     # occur.
     if context.hasTempName("return_value"):
         function_locals.append("tmp_return_value = NULL;")
-    for tmp_name, tmp_type in context.getTempNameInfos():
+    if context.hasTempName("generator_return"):
+        function_locals.append("tmp_generator_return = false;")
+    for tmp_name, _tmp_type in tmp_infos:
         if tmp_name.startswith("tmp_outline_return_value_"):
             function_locals.append("%s = NULL;" % tmp_name)
 
-    function_doc = getConstantCode(
-        context  = context,
+    for locals_dict_name in context.getLocalsDictNames():
+        function_locals.append(
+            function_dict_setup % {
+                "locals_dict" : locals_dict_name
+            }
+        )
+
+        function_cleanup.append(
+            "Py_DECREF( %(locals_dict)s );\n" % {
+                "locals_dict" : locals_dict_name
+            }
+        )
+
+
+def getFunctionCode(context, function_identifier, parameters, closure_variables,
+                    user_variables, outline_variables,
+                    temp_variables, function_doc, file_scope, needs_exception_exit):
+
+    # Functions have many details, that we express as variables, with many
+    # branches to decide, pylint: disable=too-many-locals
+
+    function_locals, function_cleanup = setupFunctionLocalVariables(
+        context           = context,
+        parameters        = parameters,
+        closure_variables = closure_variables,
+        user_variables    = user_variables + outline_variables,
+        temp_variables    = temp_variables
+    )
+
+    function_codes = SourceCodeCollector()
+
+    generateStatementSequenceCode(
+        statement_sequence = context.getOwner().getBody(),
+        allow_none         = True,
+        emit               = function_codes,
+        context            = context
+    )
+
+    finalizeFunctionLocalVariables(context, function_locals, function_cleanup)
+
+    function_doc = context.getConstantCode(
         constant = function_doc
     )
 
@@ -585,7 +591,7 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
 
     if needs_exception_exit:
         function_exit += template_function_exception_exit % {
-            "function_cleanup"    : function_cleanup,
+            "function_cleanup" : indented(function_cleanup),
         }
 
     if context.hasTempName("return_value"):
@@ -596,7 +602,7 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
         )
 
     if context.isForCreatedFunction():
-        parameter_objects_decl = ["Nuitka_FunctionObject const *self"]
+        parameter_objects_decl = ["struct Nuitka_FunctionObject const *self"]
     else:
         parameter_objects_decl = []
 
@@ -605,25 +611,34 @@ def getFunctionCode(context, function_identifier, parameters, closure_variables,
     ]
 
     if context.isForDirectCall():
-        parameter_objects_decl += getFunctionDirectClosureArgs(closure_variables)
+        for closure_variable in closure_variables:
+            variable_code_name, variable_c_type = getLocalVariableCodeType(
+                context  = context,
+                variable = closure_variable,
+                version  = 0
+            )
+
+            parameter_objects_decl.append(
+                variable_c_type.getVariableArgDeclarationCode(variable_code_name)
+            )
 
         result += function_direct_body_template % {
-            "file_scope"                   : file_scope,
-            "function_identifier"          : function_identifier,
-            "direct_call_arg_spec"         : ", ".join(
+            "file_scope"           : file_scope,
+            "function_identifier"  : function_identifier,
+            "direct_call_arg_spec" : ", ".join(
                 parameter_objects_decl
             ),
-            "function_locals"              : indented(function_locals),
-            "function_body"                : indented(function_codes),
-            "function_exit"                : function_exit
+            "function_locals"      : indented(function_locals),
+            "function_body"        : indented(function_codes.codes),
+            "function_exit"        : function_exit
         }
     else:
         result += template_function_body % {
-            "function_identifier"          : function_identifier,
-            "parameter_objects_decl"       : ", ".join(parameter_objects_decl),
-            "function_locals"              : indented(function_locals),
-            "function_body"                : indented(function_codes),
-            "function_exit"                : function_exit
+            "function_identifier"    : function_identifier,
+            "parameter_objects_decl" : ", ".join(parameter_objects_decl),
+            "function_locals"        : indented(function_locals),
+            "function_body"          : indented(function_codes.codes),
+            "function_exit"          : function_exit
         }
 
     return result
@@ -636,36 +651,6 @@ def getExportScopeCode(cross_module):
         return "NUITKA_LOCAL_MODULE"
 
 
-
-def generateFunctionDeclCode(function_body):
-    if function_body.isExpressionGeneratorObjectBody():
-        return getGeneratorObjectDeclCode(
-            function_identifier = function_body.getCodeName(),
-        )
-    elif function_body.isExpressionCoroutineObjectBody():
-        return getCoroutineObjectDeclCode(
-            function_identifier = function_body.getCodeName(),
-        )
-    elif function_body.isExpressionClassBody():
-        return getFunctionDirectDecl(
-            function_identifier = function_body.getCodeName(),
-            closure_variables   = function_body.getClosureVariables(),
-            file_scope          = getExportScopeCode(
-                cross_module = False
-            )
-        )
-    elif function_body.needsDirectCall():
-        return getFunctionDirectDecl(
-            function_identifier = function_body.getCodeName(),
-            closure_variables   = function_body.getClosureVariables(),
-            file_scope          = getExportScopeCode(
-                cross_module = function_body.isCrossModuleUsed()
-            )
-        )
-    else:
-        return None
-
-
 def generateFunctionCallCode(to_name, expression, emit, context):
     assert expression.getFunction().isExpressionFunctionCreation()
 
@@ -695,7 +680,7 @@ def generateFunctionCallCode(to_name, expression, emit, context):
         to_name             = to_name,
         function_identifier = function_identifier,
         arg_names           = arg_names,
-        closure_variables   = function_body.getClosureVariables(),
+        closure_variables   = expression.getClosureVariableVersions(),
         needs_check         = expression.getFunction().getFunctionRef().\
                                 getFunctionBody().mayRaiseException(BaseException),
         emit                = emit,
@@ -704,7 +689,23 @@ def generateFunctionCallCode(to_name, expression, emit, context):
 
 
 def generateFunctionOutlineCode(to_name, expression, emit, context):
-    assert expression.isExpressionOutlineBody()
+    assert expression.isExpressionOutlineBody() or \
+           expression.isExpressionOutlineFunction() or \
+           expression.isExpressionClassBody()
+
+    if expression.isExpressionOutlineFunctionBodyBase():
+        context = PythonFunctionOutlineContext(
+            parent  = context,
+            outline = expression
+        )
+
+        locals_dict_handling = True
+    else:
+        locals_dict_handling = False
+
+    if locals_dict_handling:
+        if expression.hasLocalsDict():
+            context.allocateLocalsDictName()
 
     # Need to set return target, to assign to_name from.
     old_return_release_mode = context.getReturnReleaseMode()
@@ -715,7 +716,13 @@ def generateFunctionOutlineCode(to_name, expression, emit, context):
     return_value_name = context.allocateTempName("outline_return_value")
     old_return_value_name = context.setReturnValueName(return_value_name)
 
-    from .CodeGeneration import generateStatementSequenceCode
+    if expression.isExpressionOutlineFunctionBodyBase() and \
+       expression.getBody().mayRaiseException(BaseException):
+        exception_target = context.allocateLabel("outline_exception")
+        old_exception_target = context.setExceptionEscape(exception_target)
+    else:
+        exception_target = None
+
     generateStatementSequenceCode(
         statement_sequence = expression.getBody(),
         emit               = emit,
@@ -729,6 +736,16 @@ def generateFunctionOutlineCode(to_name, expression, emit, context):
         emit    = emit
     )
 
+    if exception_target is not None:
+        getLabelCode(exception_target, emit)
+
+        context.setCurrentSourceCodeReference(expression.getSourceReference())
+
+        emitErrorLineNumberUpdateCode(emit, context)
+        getGotoCode(old_exception_target, emit)
+
+        context.setExceptionEscape(old_exception_target)
+
     getLabelCode(return_target, emit)
     emit(
         "%s = %s;" % (
@@ -743,3 +760,7 @@ def generateFunctionOutlineCode(to_name, expression, emit, context):
     context.setReturnTarget(old_return_target)
     context.setReturnReleaseMode(old_return_release_mode)
     context.setReturnValueName(old_return_value_name)
+
+    if locals_dict_handling:
+        if expression.hasLocalsDict():
+            context.endLocalsDictName()
