@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -20,16 +20,13 @@
 That is import as expression, and star import.
 """
 
-
-from .ConstantCodes import getConstantCode
+from .CodeHelpers import generateChildExpressionsCode, generateExpressionCode
 from .ErrorCodes import (
     getErrorExitBoolCode,
     getErrorExitCode,
     getReleaseCode,
     getReleaseCodes
 )
-from .GlobalsLocalsCodes import getLoadGlobalsCode, getLoadLocalsCode
-from .Helpers import generateExpressionCode, generateExpressionsCode
 from .LineNumberCodes import emitLineNumberUpdateCode
 from .ModuleCodes import getModuleAccessCode
 
@@ -37,50 +34,12 @@ from .ModuleCodes import getModuleAccessCode
 def generateBuiltinImportCode(to_name, expression, emit, context):
     # We know that 5 expressions are created, pylint: disable=W0632
     module_name, globals_name, locals_name, import_list_name, level_name = \
-      generateExpressionsCode(
-        expressions = (
-            expression.getImportName(),
-            expression.getGlobals(),
-            expression.getLocals(),
-            expression.getFromList(),
-            expression.getLevel()
-        ),
-        names       = (
-            "import_modulename",
-            "import_globals",
-            "import_locals",
-            "import_fromlist",
-            "import_level"
-        ),
-        emit        = emit,
-        context     = context
+      generateChildExpressionsCode(
+        expression = expression,
+        emit       = emit,
+        context    = context
     )
 
-    if expression.getGlobals() is None:
-        globals_name = context.allocateTempName("import_globals")
-
-        getLoadGlobalsCode(
-            to_name = globals_name,
-            emit    = emit,
-            context = context
-        )
-
-    if expression.getLocals() is None:
-        provider = expression.getParentVariableProvider()
-
-        if provider.isCompiledPythonModule():
-            locals_name = globals_name
-        else:
-            locals_name = context.allocateTempName("import_locals")
-
-            getLoadLocalsCode(
-                to_name  = locals_name,
-                provider = provider,
-                mode     = provider.getLocalsMode(),
-                emit     = emit,
-                context  = context
-            )
-
     getBuiltinImportCode(
         to_name          = to_name,
         module_name      = module_name,
@@ -88,61 +47,109 @@ def generateBuiltinImportCode(to_name, expression, emit, context):
         locals_name      = locals_name,
         import_list_name = import_list_name,
         level_name       = level_name,
+        needs_check      = expression.mayRaiseException(BaseException),
         emit             = emit,
         context          = context
     )
 
 
+def getCountedArgumentsHelperCallCode(helper_prefix, to_name, args, min_args,
+                                      needs_check, emit, context):
+    orig_args = args
+    args = list(args)
+    while args[-1] is None:
+        del args[-1]
+
+    if None in args:
+        emit(
+            "%s = %s_KW( %s );" % (
+                to_name,
+                helper_prefix,
+                ", ".join(
+                    "NULL" if arg is None else arg
+                    for arg in
+                    orig_args
+                )
+            )
+        )
+    else:
+        # Check that no following arguments are not None.
+        assert len(args) >= min_args
+
+        emit(
+            "%s = %s%d( %s );" % (
+                to_name,
+                helper_prefix,
+                len(args),
+                ", ".join(args)
+            )
+        )
+
+    getReleaseCodes(
+        release_names = args,
+        emit          = emit,
+        context       = context
+    )
+
+    getErrorExitCode(
+        check_name  = to_name,
+        needs_check = needs_check,
+        emit        = emit,
+        context     = context
+    )
+
+    context.addCleanupTempName(to_name)
+
+
 def getBuiltinImportCode(to_name, module_name, globals_name, locals_name,
-                         import_list_name, level_name, emit, context):
+                         import_list_name, level_name, needs_check, emit,
+                         context):
 
     emitLineNumberUpdateCode(emit, context)
 
-    emit(
-        "%s = IMPORT_MODULE( %s, %s, %s, %s, %s );" % (
-            to_name,
-            module_name,
-            globals_name,
-            locals_name,
-            import_list_name,
-            level_name
-        )
-    )
-
-    getReleaseCodes(
-        release_names = (
+    getCountedArgumentsHelperCallCode(
+        helper_prefix = "IMPORT_MODULE",
+        to_name       = to_name,
+        args          = (
             module_name,
             globals_name,
             locals_name,
             import_list_name,
             level_name
         ),
+        min_args      = 1,
+        needs_check   = needs_check,
         emit          = emit,
         context       = context
     )
 
-    getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
-    )
-
-    context.addCleanupTempName(to_name)
-
 
 def generateImportModuleHardCode(to_name, expression, emit, context):
-    getImportModuleHardCode(
-        to_name     = to_name,
-        module_name = expression.getModuleName(),
-        import_name = expression.getImportName(),
-        needs_check = expression.mayRaiseException(BaseException),
+    module_name = expression.getModuleName()
+    needs_check = expression.mayRaiseException(BaseException)
+
+    emitLineNumberUpdateCode(emit, context)
+
+    emit(
+        """%s = PyImport_ImportModule("%s");""" % (
+            to_name,
+            module_name
+        )
+    )
+
+    getErrorExitCode(
+        check_name  = to_name,
+        needs_check = needs_check,
         emit        = emit,
         context     = context
     )
 
 
-def getImportModuleHardCode(to_name, module_name, import_name, needs_check,
-                            emit, context):
+def generateImportModuleNameHardCode(to_name, expression, emit, context):
+    module_name = expression.getModuleName()
+    import_name = expression.getImportName()
+    needs_check = expression.mayRaiseException(BaseException)
+
     if module_name == "sys":
         emit(
             """%s = PySys_GetObject( (char *)"%s" );""" % (
@@ -150,80 +157,45 @@ def getImportModuleHardCode(to_name, module_name, import_name, needs_check,
                 import_name
             )
         )
-    elif module_name in ("os", "__future__"):
+    elif module_name in ("os", "__future__", "importlib._bootstrap"):
+        emitLineNumberUpdateCode(emit, context)
+
         emit(
-             """%s = PyObject_GetAttrString(PyImport_ImportModule("%s"), "%s");""" % (
-                to_name,
-                module_name,
-                import_name
-            )
+             """\
+{
+    PyObject *module = PyImport_ImportModule("%(module_name)s");
+    if (likely( module != NULL ))
+    {
+        %(to_name)s = PyObject_GetAttr( module, %(import_name)s );
+    }
+    else
+    {
+        %(to_name)s = NULL;
+    }
+}
+""" % {
+                "to_name"     : to_name,
+                "module_name" : module_name,
+                "import_name" : context.getConstantCode(import_name)
+            }
         )
     else:
         assert False, module_name
 
-    if needs_check:
-        getErrorExitCode(
-            check_name = to_name,
-            emit       = emit,
-            context    = context
-        )
-
-
-def generateImportModuleCode(to_name, expression, emit, context):
-    provider = expression.getParentVariableProvider()
-
-    globals_name = context.allocateTempName("import_globals")
-
-    getLoadGlobalsCode(
-        to_name = globals_name,
-        emit    = emit,
-        context = context
+    getErrorExitCode(
+        check_name  = to_name,
+        needs_check = needs_check,
+        emit        = emit,
+        context     = context
     )
 
-    if provider.isCompiledPythonModule():
-        locals_name = globals_name
-    else:
-        locals_name = context.allocateTempName("import_locals")
-
-        getLoadLocalsCode(
-            to_name  = locals_name,
-            provider = expression.getParentVariableProvider(),
-            mode     = "updated",
-            emit     = emit,
-            context  = context
-        )
-
-    old_source_ref = context.setCurrentSourceCodeReference(expression.getSourceReference())
-
-    getBuiltinImportCode(
-        to_name          = to_name,
-        module_name      = getConstantCode(
-            constant = expression.getModuleName(),
-            context  = context
-        ),
-        globals_name     = globals_name,
-        locals_name      = locals_name,
-        import_list_name = getConstantCode(
-            constant = expression.getImportList(),
-            context  = context
-        ),
-        level_name       = getConstantCode(
-            constant = expression.getLevel(),
-            context  = context
-        ),
-        emit             = emit,
-        context          = context
-    )
-
-    context.setCurrentSourceCodeReference(old_source_ref)
-
 
 def generateImportStarCode(statement, emit, context):
     module_name = context.allocateTempName("star_imported")
 
-    generateImportModuleCode(
+    generateExpressionCode(
         to_name    = module_name,
-        expression = statement.getModule(),
+        expression = statement.getSourceModule(),
         emit       = emit,
         context    = context
     )
@@ -244,8 +216,9 @@ def generateImportStarCode(statement, emit, context):
         )
     else:
         emit(
-            "%s = IMPORT_MODULE_STAR( locals_dict, false, %s );" % (
+            "%s = IMPORT_MODULE_STAR( %s, false, %s );" % (
                 res_name,
+                context.getLocalsDictName(),
                 module_name
             )
         )
@@ -279,9 +252,8 @@ def generateImportNameCode(to_name, expression, emit, context):
         "%s = IMPORT_NAME( %s, %s );" % (
             to_name,
             from_arg_name,
-            getConstantCode(
-                constant = expression.getImportName(),
-                context  = context
+            context.getConstantCode(
+                constant = expression.getImportName()
             )
         )
     )
@@ -293,9 +265,10 @@ def generateImportNameCode(to_name, expression, emit, context):
     )
 
     getErrorExitCode(
-        check_name = to_name,
-        emit       = emit,
-        context    = context
+        check_name  = to_name,
+        needs_check = expression.mayRaiseException(BaseException),
+        emit        = emit,
+        context     = context
     )
 
     context.addCleanupTempName(to_name)
