@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -18,22 +18,28 @@
 
 """ Indentation of code.
 
-Language independent, the amount of the spaces is not configurable, as it needs to
-be the same as in templates.
-
+Language independent, the amount of the spaces is not configurable, as it needs
+to be the same as in templates.
 """
-def _indentedCode( codes, count ):
-    return "\n".join(
-        " " * count + line if (line and not line.startswith( "#" )) else line for line in codes
+
+def _indentedCode(codes, count):
+    return '\n'.join(
+        ' ' * count + line
+          if (line and not line.startswith('#')) else
+        line for line in codes
     )
 
-def indented( codes, level = 1, vert_block = False ):
-    if type( codes ) is str:
-        codes = codes.split( "\n" )
 
-    if vert_block and codes != [ "" ]:
+def indented(codes, level = 1, vert_block = False):
+    if type(codes) is str:
+        codes = codes.split('\n')
 
-        codes.insert( 0, "" )
-        codes.append( "" )
+    if vert_block and codes != [""]:
+        codes.insert(0, "")
+        codes.append("")
 
-    return _indentedCode( codes, level * 4 )
+    return _indentedCode(codes, level * 4)
+
+
+def getCommentCode(comment, emit):
+    emit("// " + comment)
