@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,7 +24,7 @@ if ( %(keeper_tb)s == NULL )
 {
     %(keeper_tb)s = %(tb_making)s;
 }
-else if ( %(keeper_lineno)s != -1 )
+else if ( %(keeper_lineno)s != 0 )
 {
     %(keeper_tb)s = ADD_TRACEBACK( %(keeper_tb)s, %(frame_identifier)s, %(keeper_lineno)s );
 }
@@ -46,6 +46,7 @@ if ( %(condition)s )
     }
 %(release_temps)s
 
+%(var_description_code)s
 %(line_number_code)s
     goto %(exception_exit)s;
 }"""
@@ -59,6 +60,7 @@ if ( %(condition)s )
 %(release_temps)s
 
 %(line_number_code)s
+%(var_description_code)s
     goto %(exception_exit)s;
 }"""
 
@@ -69,6 +71,7 @@ if ( %(condition)s )
 %(set_exception)s
 
 %(line_number_code)s
+%(var_description_code)s
     goto %(exception_exit)s;
 }
 """
