@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -40,6 +40,8 @@ if (likely( %(attempt_name)s == NULL ))
         {
             FETCH_ERROR_OCCURRED( &exception_type, &exception_value, &exception_tb );
 %(release_temps_1)s
+%(var_description_code_1)s
+%(line_number_code_1)s
             goto %(exception_exit)s;
         }
     }
@@ -56,6 +58,8 @@ else
 #endif
     FETCH_ERROR_OCCURRED( &exception_type, &exception_value, &exception_tb );
 %(release_temps_2)s
+%(var_description_code_2)s
+%(line_number_code_2)s
     goto %(exception_exit)s;
 }"""
 
@@ -71,8 +75,12 @@ if ( %(to_name)s == NULL )
     {
 %(release_temps)s
         FETCH_ERROR_OCCURRED( &exception_type, &exception_value, &exception_tb );
+%(var_description_code)s
 %(line_number_code)s
         goto %(exception_target)s;
     }
 }
 """
+
+from . import TemplateDebugWrapper # isort:skip
+TemplateDebugWrapper.checkDebug(globals())
