@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,7 +28,7 @@ from nuitka.Options import isDebug
 def enableDebug(globals_dict):
     templates = dict(globals_dict)
 
-    class TemplateWrapper:
+    class TemplateWrapper(object):
         """ Wrapper around templates.
 
             To better trace and control template usage.
