@@ -26,8 +26,13 @@ import sys
 from logging import info
 
 from nuitka import Tracing
-from nuitka.__past__ import raw_input, urlretrieve  # pylint: disable=W0622
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    raw_input,
+    urlretrieve
+)
 from nuitka.utils import Utils
+from nuitka.utils.AppDirs import getAppDir
+from nuitka.utils.FileOperations import deleteFile, makePath
 
 
 def getDependsExePath():
@@ -41,50 +46,58 @@ def getDependsExePath():
     else:
         depends_url = "http://dependencywalker.com/depends22_x64.zip"
 
-    if "APPDATA" not in os.environ:
-        sys.exit("Error, standalone mode cannot find 'APPDATA' environment.")
+    nuitka_app_dir = getAppDir()
 
-    nuitka_app_dir = Utils.joinpath(os.environ["APPDATA"], "nuitka")
-    if not Utils.isDir(nuitka_app_dir):
-        Utils.makePath(nuitka_app_dir)
-
-    nuitka_depends_zip = Utils.joinpath(
+    nuitka_depends_dir = os.path.join(
         nuitka_app_dir,
-        Utils.basename(depends_url)
+        Utils.getArchitecture()
     )
+    nuitka_depends_zip = os.path.join(
+        nuitka_depends_dir,
+        os.path.basename(depends_url)
+    )
+    depends_exe = os.path.join(
+        nuitka_depends_dir,
+        "depends.exe"
+    )
+    makePath(nuitka_depends_dir)
 
-    if not Utils.isFile(nuitka_depends_zip):
-        Tracing.printLine("""\
+    if not os.path.isfile(nuitka_depends_zip) and not os.path.isfile(depends_exe):
+        Tracing.printLine(
+            """\
 Nuitka will make use of Dependency Walker (http://dependencywalker.com) tool
 to analyze the dependencies of Python extension modules. Is it OK to download
-and put it in APPDATA (no installer needed, cached, one time question).""")
+and put it in "%s".
+No installer needed, cached, one time question.
 
-        reply = raw_input("Proceed and download? [Yes]/No ")
+Proceed and download? [Yes]/No """ % (
+                nuitka_app_dir
+            )
+        )
+        Tracing.flushStdout()
+
+        reply = raw_input()
 
         if reply.lower() in ("no", 'n'):
             sys.exit("Nuitka does not work in --standalone on Windows without.")
 
         info("Downloading '%s'" % depends_url)
 
-        urlretrieve(
-            depends_url,
-            nuitka_depends_zip
-        )
+        try:
+            urlretrieve(
+                depends_url,
+                nuitka_depends_zip
+            )
+        except Exception: # Any kind of error, pylint: disable=broad-except
+            sys.exit(
+                """Failed to download '%s'.\
+Contents should manually be extracted to '%s'.""" % (
+                    depends_url,
+                    nuitka_depends_dir
+                )
+            )
 
-    nuitka_depends_dir = Utils.joinpath(
-        nuitka_app_dir,
-        Utils.getArchitecture()
-    )
-
-    if not Utils.isDir(nuitka_depends_dir):
-        os.makedirs(nuitka_depends_dir)
-
-    depends_exe = os.path.join(
-        nuitka_depends_dir,
-        "depends.exe"
-    )
-
-    if not Utils.isFile(depends_exe):
+    if not os.path.isfile(depends_exe):
         info("Extracting to '%s'" % depends_exe)
 
         import zipfile
@@ -95,8 +108,8 @@ and put it in APPDATA (no installer needed, cached, one time question).""")
         except Exception: # Catching anything zip throws, pylint:disable=W0703
             info("Problem with the downloaded zip file, deleting it.")
 
-            Utils.deleteFile(depends_exe, must_exist = False)
-            Utils.deleteFile(nuitka_depends_zip, must_exist = True)
+            deleteFile(depends_exe, must_exist = False)
+            deleteFile(nuitka_depends_zip, must_exist = True)
 
             sys.exit(
                 "Error, need '%s' as extracted from '%s'." % (
@@ -105,6 +118,6 @@ and put it in APPDATA (no installer needed, cached, one time question).""")
                 )
             )
 
-    assert Utils.isFile(depends_exe)
+    assert os.path.isfile(depends_exe)
 
     return depends_exe
