@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,16 +22,16 @@ MacOS, Windows, and Linux. Patches for other platforms are
 very welcome.
 """
 
+import marshal
 import os
 import shutil
 import subprocess
 import sys
 from logging import debug, info, warning
 
-import marshal
 from nuitka import Options, SourceCodeReferences, Tracing
 from nuitka.__past__ import iterItems
-from nuitka.codegen.ConstantCodes import needsPickleInit
+from nuitka.containers.odict import OrderedDict
 from nuitka.importing import ImportCache
 from nuitka.importing.StandardLibrary import (
     getStandardLibraryPaths,
@@ -41,9 +41,18 @@ from nuitka.nodes.ModuleNodes import (
     PythonShlibModule,
     makeUncompiledPythonModule
 )
+from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils import Utils
+from nuitka.utils.Execution import withEnvironmentPathAdded
+from nuitka.utils.FileOperations import (
+    areSamePaths,
+    deleteFile,
+    getSubDirectories,
+    listDir
+)
+from nuitka.utils.Timing import TimerReport
 
 from .DependsExe import getDependsExePath
 
@@ -56,14 +65,16 @@ def loadCodeObjectData(precompiled_filename):
 
 module_names = set()
 
-def _detectedPrecompiledFile(filename, module_name, result, user_provided):
+def _detectedPrecompiledFile(filename, module_name, result, user_provided,
+                             technical):
     if filename.endswith(".pyc"):
-        if Utils.isFile(filename[:-1]):
+        if os.path.isfile(filename[:-1]):
             return _detectedSourceFile(
                 filename      = filename[:-1],
                 module_name   = module_name,
                 result        = result,
-                user_provided = user_provided
+                user_provided = user_provided,
+                technical     = technical
             )
 
     if module_name in module_names:
@@ -75,22 +86,24 @@ def _detectedPrecompiledFile(filename, module_name, result, user_provided):
         filename
     )
 
-    result.append(
-        makeUncompiledPythonModule(
-            module_name   = module_name,
-            bytecode      = loadCodeObjectData(
-                precompiled_filename = filename
-            ),
-            is_package    = "__init__" in filename,
-            filename      = filename,
-            user_provided = user_provided
-        )
+    uncompiled_module = makeUncompiledPythonModule(
+        module_name   = module_name,
+        bytecode      = loadCodeObjectData(
+            precompiled_filename = filename
+        ),
+        is_package    = "__init__" in filename,
+        filename      = filename,
+        user_provided = user_provided,
+        technical     = technical
     )
 
+    ImportCache.addImportedModule(uncompiled_module)
+
+    result.append(uncompiled_module)
     module_names.add(module_name)
 
 
-def _detectedSourceFile(filename, module_name, result, user_provided):
+def _detectedSourceFile(filename, module_name, result, user_provided, technical):
     if module_name in module_names:
         return
 
@@ -99,7 +112,8 @@ def _detectedSourceFile(filename, module_name, result, user_provided):
             filename      = filename,
             module_name   = "_collections_abc",
             result        = result,
-            user_provided = user_provided
+            user_provided = user_provided,
+            technical     = technical
         )
 
     source_code = readSourceCodeFromFilename(module_name, filename)
@@ -111,28 +125,57 @@ def _detectedSourceFile(filename, module_name, result, user_provided):
         source_code = """\
 __file__ = (__nuitka_binary_dir + '%s%s') if '__nuitka_binary_dir' in dict(__builtins__ ) else '<frozen>';%s""" % (
             os.path.sep,
-            Utils.basename(filename),
+            os.path.basename(filename),
             source_code
         )
 
+        # Debian stretch site.py
+        source_code = source_code.replace(
+            "PREFIXES = [sys.prefix, sys.exec_prefix]",
+            "PREFIXES = []"
+        )
+
+        # Anaconda3 4.1.2 site.py
+        source_code = source_code.replace(
+            "def main():",
+            "def main():return\n\nif 0:\n def _unused():",
+        )
+
     debug(
         "Freezing module '%s' (from '%s').",
         module_name,
         filename
     )
 
-    result.append(
-        makeUncompiledPythonModule(
-            module_name   = module_name,
-            bytecode      = marshal.dumps(
-                compile(source_code, filename, "exec")
-            ),
-            is_package    = Utils.basename(filename) == "__init__.py",
-            filename      = filename,
-            user_provided = user_provided
-        )
+    is_package = os.path.basename(filename) == "__init__.py"
+    source_code = Plugins.onFrozenModuleSourceCode(
+        module_name = module_name,
+        is_package  = is_package,
+        source_code = source_code
     )
 
+    bytecode = compile(source_code, filename, "exec", dont_inherit = True)
+
+    bytecode = Plugins.onFrozenModuleBytecode(
+        module_name = module_name,
+        is_package  = is_package,
+        bytecode    = bytecode
+    )
+
+    uncompiled_module = makeUncompiledPythonModule(
+        module_name   = module_name,
+        bytecode      = marshal.dumps(
+            bytecode
+        ),
+        is_package    = is_package,
+        filename      = filename,
+        user_provided = user_provided,
+        technical     = technical
+    )
+
+    ImportCache.addImportedModule(uncompiled_module)
+
+    result.append(uncompiled_module)
     module_names.add(module_name)
 
 
@@ -169,13 +212,13 @@ def _detectedShlibFile(filename, module_name):
     module_names.add(module_name)
 
 
-def _detectImports(command, user_provided):
+def _detectImports(command, user_provided, technical):
     # This is pretty complicated stuff, with variants to deal with.
-    # pylint: disable=R0912,R0914,R0915
+    # pylint: disable=too-many-branches,too-many-locals,too-many-statements
 
     # Print statements for stuff to show, the modules loaded.
     if python_version >= 300:
-        command += '\nimport sys\nprint("\\n".join(sorted("import " + module.__name__ + " # sourcefile " + ' \
+        command += '\nprint("\\n".join(sorted("import " + module.__name__ + " # sourcefile " + ' \
                    'module.__file__ for module in sys.modules.values() if hasattr(module, "__file__") and ' \
                    'module.__file__ != "<frozen>")), file = sys.stderr)'  # do not read it
 
@@ -183,26 +226,26 @@ def _detectImports(command, user_provided):
         path_element
         for path_element in
         sys.path
-        if not Utils.areSamePaths(
+        if not areSamePaths(
             path_element,
             '.'
         )
-        if not Utils.areSamePaths(
+        if not areSamePaths(
             path_element,
-            Utils.dirname(sys.modules["__main__"].__file__)
+            os.path.dirname(sys.modules["__main__"].__file__)
         )
     ]
 
     # Make sure the right import path (the one Nuitka binary is running with)
     # is used.
-    command = ("import sys; sys.path = %s;" % repr(reduced_path)) + command
+    command = ("import sys; sys.path = %s; sys.real_prefix = sys.prefix;" % repr(reduced_path)) + command
 
     import tempfile
     tmp_file, tmp_filename = tempfile.mkstemp()
 
     try:
         if python_version >= 300:
-            command = command.encode("ascii")
+            command = command.encode("utf8")
         os.write(tmp_file, command)
         os.close(tmp_file)
 
@@ -219,13 +262,15 @@ def _detectImports(command, user_provided):
     if process.returncode != 0:
         warning("There is a problem with detecting imports, CPython said:")
         for line in stderr.split(b"\n"):
-            Tracing.printLine(line)
+            Tracing.printError(line)
         sys.exit("Error, please report the issue with above output.")
 
     result = []
 
     debug("Detecting imports:")
 
+    detections = []
+
     for line in stderr.replace(b"\r", b"").split(b"\n"):
         if line.startswith(b"import "):
             # print(line)
@@ -249,11 +294,8 @@ def _detectImports(command, user_provided):
                 if not isStandardLibraryPath(filename):
                     continue
 
-                _detectedPrecompiledFile(
-                    filename      = filename,
-                    module_name   = module_name,
-                    result        = result,
-                    user_provided = user_provided
+                detections.append(
+                    (module_name, 3, "precompiled", filename)
                 )
             elif origin == b"sourcefile":
                 filename = parts[1][len(b"sourcefile "):]
@@ -265,16 +307,19 @@ def _detectImports(command, user_provided):
                     continue
 
                 if filename.endswith(".py"):
-                    _detectedSourceFile(
-                        filename      = filename,
-                        module_name   = module_name,
-                        result        = result,
-                        user_provided = user_provided
+                    detections.append(
+                        (module_name, 2, "sourcefile", filename)
                     )
                 elif not filename.endswith("<frozen>"):
-                    _detectedShlibFile(
-                        filename    = filename,
-                        module_name = module_name
+                    # Python3 started lying in "__name__" for the "_decimal"
+                    # calls itself "decimal", which then is wrong and also
+                    # clashes with "decimal" proper
+                    if python_version >= 300:
+                        if module_name == "decimal":
+                            module_name = "_decimal"
+
+                    detections.append(
+                        (module_name, 2, "shlib", filename)
                     )
             elif origin == b"dynamically":
                 # Shared library in early load, happens on RPM based systems and
@@ -287,36 +332,38 @@ def _detectImports(command, user_provided):
                 if not isStandardLibraryPath(filename):
                     continue
 
-                _detectedShlibFile(
-                    filename    = filename,
-                    module_name = module_name
+                detections.append(
+                    (module_name, 1, "shlib", filename)
                 )
 
+    for module_name, _prio, kind, filename in sorted(detections):
+        if kind == "precompiled":
+            _detectedPrecompiledFile(
+                filename      = filename,
+                module_name   = module_name,
+                result        = result,
+                user_provided = user_provided,
+                technical     = technical
+            )
+        elif kind == "sourcefile":
+            _detectedSourceFile(
+                filename      = filename,
+                module_name   = module_name,
+                result        = result,
+                user_provided = user_provided,
+                technical     = technical
+            )
+        elif kind == "shlib":
+            _detectedShlibFile(
+                filename    = filename,
+                module_name = module_name
+            )
+        else:
+            assert False, kind
+
     return result
 
 
-def detectLateImports():
-    command = ""
-    # When we are using pickle internally (for some hard constant cases we do),
-    # we need to make sure it will be available as well.
-    if needsPickleInit():
-        command += "import {pickle};".format(
-            pickle = "pickle" if python_version >= 300 else "cPickle"
-        )
-
-    # For Python3 we patch inspect without knowing if it is used.
-    if python_version >= 300:
-        command += "import inspect;"
-
-    if command:
-        result = _detectImports(command, True)
-
-        debug("Finished detecting late imports.")
-
-        return result
-    else:
-        return ()
-
 # Some modules we want to blacklist.
 ignore_modules = [
     "__main__.py",
@@ -327,9 +374,10 @@ if Utils.getOS() != "Windows":
     ignore_modules.append("wintypes.py")
     ignore_modules.append("cp65001.py")
 
+
 def scanStandardLibraryPath(stdlib_dir):
     # There is a lot of black-listing here, done in branches, so there
-    # is many of them, but that's acceptable, pylint: disable=R0912
+    # is many of them, but that's acceptable, pylint: disable=too-many-branches
 
     for root, dirs, filenames in os.walk(stdlib_dir):
         import_path = root[len(stdlib_dir):].strip("/\\")
@@ -352,12 +400,21 @@ def scanStandardLibraryPath(stdlib_dir):
             if "ensurepip" in dirs:
                 dirs.remove("ensurepip")
 
+            # Ignore "lib-dynload" and "lib-tk" and alikes.
+            dirs[:] = [
+                dirname
+                for dirname in
+                dirs
+                if not dirname.startswith("lib-")
+                if dirname != "Tools"
+            ]
+
         if import_path in ("tkinter", "importlib", "ctypes", "unittest",
-                           "sqlite3", "distutils"):
+                           "sqlite3", "distutils", "email", "bsddb"):
             if "test" in dirs:
                 dirs.remove("test")
 
-        if import_path == "lib2to3":
+        if import_path in ("lib2to3", "json", "distutils"):
             if "tests" in dirs:
                 dirs.remove("tests")
 
@@ -391,19 +448,36 @@ def scanStandardLibraryPath(stdlib_dir):
 
 
 def detectEarlyImports():
-    # TODO: Should recursively include all of encodings module.
-    import_code = "import encodings.utf_8;import encodings.ascii;import encodings.idna;"
+    encoding_names = [
+        filename[:-3]
+        for _path, filename in
+        listDir(os.path.dirname(sys.modules["encodings"].__file__))
+        if filename.endswith(".py")
+        if "__init__" not in filename
+    ]
 
-    if Utils.getOS() == "Windows":
-        import_code += "import encodings.mbcs;import encodings.cp437;"
+    if Utils.getOS() != "Windows":
+        for encoding_name in ("mbcs", "cp65001", "oem"):
+            if encoding_name in encoding_names:
+                encoding_names.remove(encoding_name)
 
-    # String method hex depends on it.
-    if python_version < 300:
-        import_code += "import encodings.hex_codec;"
+    import_code = ';'.join(
+        "import encodings.%s" % encoding_name
+        for encoding_name in
+        encoding_names
+    )
 
-    import_code += "import locale;"
+    import_code += ";import locale;"
 
-    result = _detectImports(import_code, False)
+    # For Python3 we patch inspect without knowing if it is used.
+    if python_version >= 300:
+        import_code += "import inspect;"
+
+    result = _detectImports(
+        command       = import_code,
+        user_provided = False,
+        technical     = True
+    )
 
     if Options.shallFreezeAllStdlib():
         stdlib_modules = set()
@@ -413,12 +487,52 @@ def detectEarlyImports():
             for module_name in scanStandardLibraryPath(stdlib_dir):
                 stdlib_modules.add(module_name)
 
-        import_code = "imports = " + repr(sorted(stdlib_modules)) + '\n'\
-                      "for imp in imports:\n" \
-                      "    try:\n" \
-                      "        __import__(imp)\n" \
-                      "    except (ImportError, SyntaxError):\n" \
-                      "        pass\n"
+        # Put here ones that should be imported first.
+        first_ones = (
+            "Tkinter",
+        )
+
+        # We have to fight zombie modules in this, some things, e.g. Tkinter
+        # on newer Python 2.7, comes back after failure without a second error
+        # being raised, leading to other issues. So we fight it after each
+        # module that was tried, and prevent re-try by adding a meta path
+        # based loader that will never load it again, and remove it from the
+        # "sys.modules" over and over, once it sneaks back. The root cause is
+        # that extension modules sometimes only raise an error when first
+        # imported, not the second time around.
+        # Otherwise this just makes imports of everything so we can see where
+        # it comes from and what it requires.
+
+        import_code = """
+imports = %r
+
+failed = set()
+
+class ImportBlocker(object):
+    def find_module(self, fullname, path = None):
+        if fullname in failed:
+            return self
+
+        return None
+
+    def load_module(self, name):
+        raise ImportError("%%s has failed before" %% name)
+
+sys.meta_path.insert(0, ImportBlocker())
+
+for imp in imports:
+    try:
+        __import__(imp)
+    except (ImportError, SyntaxError):
+        failed.add(imp)
+
+    for fail in failed:
+        if fail in sys.modules:
+            del sys.modules[fail]
+""" % sorted(
+    stdlib_modules,
+    key = lambda name: (name not in first_ones, name)
+)
 
         early_names = [
             module.getFullName()
@@ -427,7 +541,11 @@ def detectEarlyImports():
 
         result += [
             module
-            for module in _detectImports(import_code, False)
+            for module in _detectImports(
+                command       = import_code,
+                user_provided = False,
+                technical     = False
+            )
             if module.getFullName() not in early_names
         ]
 
@@ -435,52 +553,100 @@ def detectEarlyImports():
 
     return result
 
+_detected_python_rpath = None
 
-def _detectBinaryPathDLLsLinuxBSD(binary_filename):
+ldd_result_cache = {}
+
+def _detectBinaryPathDLLsLinuxBSD(dll_filename):
     # Ask "ldd" about the libraries being used by the created binary, these
     # are the ones that interest us.
     result = set()
 
-    process = subprocess.Popen(
-        args   = [
-            "ldd",
-            binary_filename
-        ],
-        stdout = subprocess.PIPE,
-        stderr = subprocess.PIPE
-    )
+    # This is the rpath of the Python binary, which will be effective when
+    # loading the other DLLs too. This happens at least for Python installs
+    # on Travis. pylint: disable=global-statement
+    global _detected_python_rpath
+    if _detected_python_rpath is None:
+        _detected_python_rpath = getSharedLibraryRPATH(sys.executable) or False
 
-    stdout, _stderr = process.communicate()
+        if _detected_python_rpath:
+            _detected_python_rpath = _detected_python_rpath.replace(
+                b"$ORIGIN",
+                os.path.dirname(sys.executable).encode("utf-8")
+            )
 
-    for line in stdout.split(b"\n"):
-        if not line:
-            continue
+    if dll_filename not in ldd_result_cache:
+        with withEnvironmentPathAdded("LD_LIBRARY_PATH", _detected_python_rpath):
+            process = subprocess.Popen(
+                args   = [
+                    "ldd",
+                    dll_filename
+                ],
+                stdout = subprocess.PIPE,
+                stderr = subprocess.PIPE
+            )
 
-        if b"=>" not in line:
-            continue
+            stdout, _stderr = process.communicate()
 
-        part = line.split(b" => ", 2)[1]
+            for line in stdout.split(b"\n"):
+                if not line:
+                    continue
 
-        if b"(" in part:
-            filename = part[:part.rfind(b"(")-1]
-        else:
-            filename = part
+                if b"=>" not in line:
+                    continue
 
-        if not filename:
-            continue
+                part = line.split(b" => ", 2)[1]
 
-        if python_version >= 300:
-            filename = filename.decode("utf-8")
+                if b"(" in part:
+                    filename = part[:part.rfind(b"(")-1]
+                else:
+                    filename = part
 
-        # Sometimes might use stuff not found.
-        if filename == "not found":
-            continue
+                if not filename:
+                    continue
 
-        result.add(filename)
+                if python_version >= 300:
+                    filename = filename.decode("utf-8")
 
-    return result
+                # Sometimes might use stuff not found.
+                if filename == "not found":
+                    continue
 
-def _detectBinaryPathDLLsMacOS(binary_filename):
+                # Do not include kernel specific libraries.
+                if os.path.basename(filename).startswith(
+                        (
+                            "libc.so.",
+                            "libpthread.so.",
+                            "libm.so.",
+                            "libdl.so."
+                        )
+                    ):
+                    continue
+
+                result.add(filename)
+
+        # Allow plugins to prevent inclusion.
+        blocked = Plugins.removeDllDependencies(
+            dll_filename = dll_filename,
+            dll_filenames = result
+        )
+
+        for to_remove in blocked:
+            result.discard(to_remove)
+
+        ldd_result_cache[dll_filename] = result
+    else:
+        result = ldd_result_cache[dll_filename]
+
+    sub_result = set(result)
+
+    for sub_dll_filename in result:
+        sub_result = sub_result.union(_detectBinaryPathDLLsLinuxBSD(sub_dll_filename))
+
+    return sub_result
+
+
+def _detectBinaryPathDLLsMacOS(original_dir, binary_filename):
     result = set()
 
     process = subprocess.Popen(
@@ -510,6 +676,11 @@ def _detectBinaryPathDLLsMacOS(binary_filename):
             if python_version >= 300:
                 filename = filename.decode("utf-8")
 
+            if filename.startswith("@rpath/"):
+                filename = os.path.join(original_dir, filename[7:])
+            elif filename.startswith("@loader_path/"):
+                filename = os.path.join(original_dir, filename[13:])
+
             # print "adding", filename
             result.add(filename)
 
@@ -528,9 +699,9 @@ def _makeBinaryPathPathDLLSearchEnv(package_name):
 
     if package_name is not None:
         for element in sys.path:
-            candidate = Utils.joinpath(element, package_name)
+            candidate = os.path.join(element, package_name)
 
-            if Utils.isDir(candidate):
+            if os.path.isdir(candidate):
                 path.append(candidate)
 
 
@@ -539,7 +710,7 @@ def _makeBinaryPathPathDLLSearchEnv(package_name):
     return env
 
 
-def _detectBinaryPathDLLsWindows(binary_filename, package_name):
+def _detectBinaryPathDLLsWindows(original_dir, binary_filename, package_name):
     result = set()
 
     depends_exe = getDependsExePath()
@@ -551,12 +722,26 @@ def _detectBinaryPathDLLsWindows(binary_filename, package_name):
 KnownDLLs
 SysPath
 AppDir
+{original_dirs}
 32BitSysDir
 16BitSysDir
 OSDir
 AppPath
 SxS
-""")
+""".format(
+            original_dirs = (
+                '\n'.join(
+                    "UserDir %s" % dirname
+                    for dirname in
+                    [original_dir] + getSubDirectories(original_dir)
+                    if not os.path.basename(dirname) == "__pycache__"
+                    if any(entry[1].lower().endswith(".dll") for entry in listDir(dirname))
+                )
+                if original_dir is not None
+                else ""
+            )
+        )
+    )
 
     subprocess.call(
         (
@@ -605,9 +790,9 @@ SxS
             first = False
             continue
 
-        assert Utils.isFile(dll_filename), dll_filename
+        assert os.path.isfile(dll_filename), dll_filename
 
-        dll_name = Utils.basename(dll_filename).upper()
+        dll_name = os.path.basename(dll_filename).upper()
 
         # Win API can be assumed.
         if dll_name.startswith("API-MS-WIN-") or \
@@ -666,15 +851,16 @@ SxS
             continue
 
         result.add(
-            Utils.normcase(Utils.abspath(dll_filename))
+            os.path.normcase(os.path.abspath(dll_filename))
         )
 
-    Utils.deleteFile(binary_filename + ".depends", must_exist = True)
-    Utils.deleteFile(binary_filename + ".dwp", must_exist = True)
+    deleteFile(binary_filename + ".depends", must_exist = True)
+    deleteFile(binary_filename + ".dwp", must_exist = True)
 
     return result
 
-def detectBinaryDLLs(binary_filename, package_name):
+
+def detectBinaryDLLs(original_filename, binary_filename, package_name):
     """ Detect the DLLs used by a binary.
 
         Using "ldd" (Linux), "depends.exe" (Windows), or "otool" (MacOS) the list
@@ -682,20 +868,21 @@ def detectBinaryDLLs(binary_filename, package_name):
     """
 
 
-    if Utils.getOS() in ("Linux", "NetBSD"):
-        # TODO: FreeBSD may work the same way, not tested.
-
+    if Utils.getOS() in ("Linux", "NetBSD", "FreeBSD"):
         return _detectBinaryPathDLLsLinuxBSD(
-            binary_filename = binary_filename
+            dll_filename = original_filename
         )
     elif Utils.getOS() == "Windows":
-        return _detectBinaryPathDLLsWindows(
-            binary_filename = binary_filename,
-            package_name    = package_name
-        )
+        with TimerReport("Running depends.exe for %s took %%.2f seconds" % binary_filename):
+            return _detectBinaryPathDLLsWindows(
+                original_dir    = os.path.dirname(original_filename),
+                binary_filename = binary_filename,
+                package_name    = package_name
+            )
     elif Utils.getOS() == "Darwin":
         return _detectBinaryPathDLLsMacOS(
-            binary_filename = binary_filename
+            original_dir    = os.path.dirname(original_filename),
+            binary_filename = original_filename
         )
     else:
         # Support your platform above.
@@ -703,17 +890,18 @@ def detectBinaryDLLs(binary_filename, package_name):
 
 
 def detectUsedDLLs(standalone_entry_points):
-    result = {}
+    result = OrderedDict()
 
-    for binary_filename, package_name in standalone_entry_points:
+    for original_filename, binary_filename, package_name in standalone_entry_points:
         used_dlls = detectBinaryDLLs(
-            binary_filename = binary_filename,
-            package_name    = package_name
+            original_filename = original_filename,
+            binary_filename   = binary_filename,
+            package_name      = package_name
         )
 
         for dll_filename in used_dlls:
             # We want these to be absolute paths.
-            assert Utils.isAbsolutePath(dll_filename), dll_filename
+            assert os.path.isabs(dll_filename), dll_filename
 
             if dll_filename not in result:
                 result[dll_filename] = []
@@ -755,7 +943,7 @@ def fixupBinaryDLLPaths(binary_filename, is_exe, dll_map):
     assert process.returncode == 0, stderr
 
 
-def removeSharedLibraryRPATH(filename):
+def getSharedLibraryRPATH(filename):
     process = subprocess.Popen(
         ["readelf", "-d", filename],
         stdout = subprocess.PIPE,
@@ -763,54 +951,70 @@ def removeSharedLibraryRPATH(filename):
         shell  = False
     )
 
-    stdout, _stderr = process.communicate()
+    stdout, stderr = process.communicate()
     retcode = process.poll()
 
-    assert retcode == 0, filename
+    if retcode != 0:
+        sys.exit(
+            "Error reading shared library path for %s, tool said %r" % (
+                filename,
+                stderr
+            )
+        )
 
     for line in stdout.split(b"\n"):
-        if b"RPATH" in line:
-            if Options.isShowInclusion():
-                info("Removing 'RPATH' setting from '%s'.", filename)
+        if b"RPATH" in line or b"RUNPATH" in line:
+            return line[line.find(b'[')+1:line.rfind(b']')]
 
-            if not Utils.isExecutableCommand("chrpath"):
-                sys.exit(
-                    """\
+    return None
+
+
+def removeSharedLibraryRPATH(filename):
+    rpath = getSharedLibraryRPATH(filename)
+
+    if rpath is not None:
+        if Options.isShowInclusion():
+            info("Removing 'RPATH' setting from '%s'.", filename)
+
+        if not Utils.isExecutableCommand("chrpath"):
+            sys.exit(
+                """\
 Error, needs 'chrpath' on your system, due to 'RPATH' settings in used shared
 libraries that need to be removed."""
-                )
-
-            os.chmod(filename, int("644", 8))
-            process = subprocess.Popen(
-                ["chrpath", "-d", filename],
-                stdout = subprocess.PIPE,
-                stderr = subprocess.PIPE,
-                shell  = False
             )
-            process.communicate()
-            retcode = process.poll()
-            os.chmod(filename, int("444", 8))
 
-            assert retcode == 0, filename
+        os.chmod(filename, int("644", 8))
+        process = subprocess.Popen(
+            ["chrpath", "-d", filename],
+            stdout = subprocess.PIPE,
+            stderr = subprocess.PIPE,
+            shell  = False
+        )
+        process.communicate()
+        retcode = process.poll()
+        os.chmod(filename, int("444", 8))
+
+        assert retcode == 0, filename
 
 
 def copyUsedDLLs(dist_dir, standalone_entry_points):
     # This is terribly complex, because we check the list of used DLLs
     # trying to avoid duplicates, and detecting errors with them not
     # being binary identical, so we can report them. And then of course
-    # we also need to handle OS specifics, pylint: disable=R0912
+    # we also need to handle OS specifics, pylint: disable=too-many-branches
 
-    dll_map = []
 
     used_dlls = detectUsedDLLs(standalone_entry_points)
 
+    # Fist make checks and remove some.
     for dll_filename1, sources1 in tuple(iterItems(used_dlls)):
         for dll_filename2, sources2 in tuple(iterItems(used_dlls)):
             if dll_filename1 == dll_filename2:
                 continue
 
             # Colliding basenames are an issue to us.
-            if Utils.basename(dll_filename1) != Utils.basename(dll_filename2):
+            if os.path.basename(dll_filename1) != \
+               os.path.basename(dll_filename2):
                 continue
 
             # May already have been removed earlier
@@ -820,7 +1024,7 @@ def copyUsedDLLs(dist_dir, standalone_entry_points):
             if dll_filename2 not in used_dlls:
                 continue
 
-            dll_name = Utils.basename(dll_filename1)
+            dll_name = os.path.basename(dll_filename1)
 
             if Options.isShowInclusion():
                 info(
@@ -855,10 +1059,12 @@ different from
                 )
             )
 
-    for dll_filename, sources in iterItems(used_dlls):
-        dll_name = Utils.basename(dll_filename)
+    dll_map = []
 
-        target_path = Utils.joinpath(
+    for dll_filename, sources in iterItems(used_dlls):
+        dll_name = os.path.basename(dll_filename)
+
+        target_path = os.path.join(
             dist_dir,
             dll_name
         )
@@ -885,14 +1091,14 @@ different from
         # the relative DLL location in the ".dist" folder.
         for standalone_entry_point in standalone_entry_points:
             fixupBinaryDLLPaths(
-                binary_filename = standalone_entry_point[0],
+                binary_filename = standalone_entry_point[1],
                 is_exe          = standalone_entry_point is standalone_entry_points[0],
                 dll_map         = dll_map
             )
 
         for _original_path, dll_filename in dll_map:
             fixupBinaryDLLPaths(
-                binary_filename = Utils.joinpath(
+                binary_filename = os.path.join(
                     dist_dir,
                     dll_filename
                 ),
@@ -903,12 +1109,12 @@ different from
     if Utils.getOS() == "Linux":
         # For Linux, the "rpath" of libraries may be an issue and must be
         # removed.
-        for _original_path, dll_filename in dll_map:
-            removeSharedLibraryRPATH(
-                Utils.joinpath(dist_dir, dll_filename)
-            )
-
         for standalone_entry_point in standalone_entry_points[1:]:
             removeSharedLibraryRPATH(
-                standalone_entry_point[0]
+                standalone_entry_point[1]
+            )
+
+        for _original_path, dll_filename in dll_map:
+            removeSharedLibraryRPATH(
+                os.path.join(dist_dir, dll_filename)
             )
