@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -17,19 +17,22 @@
 #
 """ Locating modules and package source on disk.
 
-The actual import of a module would already execute code that changes things.
-Consider a module that does "os.system()", it would be executed. People often
-connect to databases, and these kind of things, at import time. Not a good
-style, but it's being done.
+The actual import of a module would already execute code that changes
+things. Imagine a module that does ``os.system()``, it would be done during
+compilation. People often connect to databases, and these kind of things,
+at import time.
 
-Therefore CPython exhibits the interfaces in an "imp" module or more modern
-"importlib" in standard library, which one can use those to know ahead of time,
-to tell what filename an import would load from.
+Therefore CPython exhibits the interfaces in an ``imp`` module in standard
+library, which one can use those to know ahead of time, what file import would
+load. For us unfortunately there is nothing in CPython that is easily
+accessible and gives us this functionality for packages and search paths
+exactly like CPython does, so we implement here a multi step search process
+that is compatible.
+
+This approach is much safer of course and there is no loss. To determine if
+it's from the standard library, one can abuse the attribute ``__file__`` of
+the ``os`` module like it's done in ``isStandardLibraryPath`` of this module.
 
-For us unfortunately there is nothing in CPython that gives the fully
-compatible functionality we need for packages and search paths exactly like
-CPython really does, so we implement here a multiple step search process that is
-compatible.
 """
 
 from __future__ import print_function
@@ -40,10 +43,11 @@ import sys
 from logging import warning
 
 from nuitka import Options
-from nuitka.containers import oset
+from nuitka.containers.oset import OrderedSet
+from nuitka.importing import StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.utils import Utils
+from nuitka.utils.FileOperations import listDir
 
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
 from .Whitelisting import isWhiteListedNotExistingModule
@@ -60,7 +64,7 @@ def setMainScriptDirectory(main_dir):
 
         We use this as part of the search path for modules.
     """
-    # We need to set this from the outside, pylint: disable=W0603
+    # We need to set this from the outside, pylint: disable=global-statement
 
     global main_path
     main_path = main_dir
@@ -74,10 +78,10 @@ def isPackageDir(dirname):
         extra packages provided via "*.pth" file tricks by "site.py" loading.
     """
 
-    return Utils.isDir(dirname) and \
+    return os.path.isdir(dirname) and \
            (
                python_version >= 330 or
-               Utils.isFile(Utils.joinpath(dirname, "__init__.py")) or
+               os.path.isfile(os.path.join(dirname, "__init__.py")) or
                isPreloadedPackagePath(dirname)
            )
 
@@ -88,6 +92,14 @@ def getPackageNameFromFullName(full_name):
     else:
         return None
 
+
+def getModuleFullNameFromPackageAndName(package, name):
+    if not package:
+        return name
+    else:
+        return package + '.' + name
+
+
 def getExtensionModuleSuffixes():
     for suffix, _mode, kind in imp.get_suffixes():
         if kind == imp.C_EXTENSION:
@@ -95,11 +107,11 @@ def getExtensionModuleSuffixes():
 
 
 def getModuleNameAndKindFromFilename(module_filename):
-    if Utils.isDir(module_filename):
-        module_name = Utils.basename(module_filename)
+    if os.path.isdir(module_filename):
+        module_name = os.path.basename(module_filename)
         module_kind = "py"
     elif module_filename.endswith(".py"):
-        module_name = Utils.basename(module_filename)[:-3]
+        module_name = os.path.basename(module_filename)[:-3]
         module_kind = "py"
     else:
         for suffix, _mode, kind in imp.get_suffixes():
@@ -107,7 +119,7 @@ def getModuleNameAndKindFromFilename(module_filename):
                 continue
 
             if module_filename.endswith(suffix):
-                module_name = Utils.basename(module_filename)[:-len(suffix)]
+                module_name = os.path.basename(module_filename)[:-len(suffix)]
                 module_kind = "shlib"
 
                 break
@@ -118,12 +130,19 @@ def getModuleNameAndKindFromFilename(module_filename):
     return module_name, module_kind
 
 
+def isWhiteListedImport(node):
+    module = node.getParentModule()
+
+    return StandardLibrary.isStandardLibraryPath(module.getFilename())
+
+
 def warnAbout(importing, module_name, parent_package, level):
     # This probably should not be dealt with here.
     if module_name == "":
         return
 
-    if not isWhiteListedNotExistingModule(module_name):
+    if not isWhiteListedImport(importing) and \
+       not isWhiteListedNotExistingModule(module_name):
         key = module_name, parent_package, level
 
         if key not in warned_about:
@@ -162,12 +181,13 @@ def warnAbout(importing, module_name, parent_package, level):
                     level_desc
                 )
 
+
 def normalizePackageName(module_name):
     # The "os.path" is strangely hacked into the "os" module, dispatching per
     # platform, we either cannot look into it, or we require that we resolve it
     # here correctly.
     if module_name == "os.path":
-        module_name = Utils.basename(os.path.__name__)
+        module_name = os.path.basename(os.path.__name__)
 
     return module_name
 
@@ -178,12 +198,13 @@ def findModule(importing, module_name, parent_package, level, warn):
         The package name can be None of course. Level is the same
         as with "__import__" built-in. Warnings are optional.
 
-        Returns a triple of package name the module is in, module name
-        and filename of it, which can be a directory.
+        Returns a triple of package name the module is in, filename of
+        it, which can be a directory for packages, and the location
+        method.
     """
 
     # We have many branches here, because there are a lot of cases to try.
-    # pylint: disable=R0912
+    # pylint: disable=too-many-branches
     if _debug_module_finding:
         print(
             "findModule: Enter to search %r in package %r level %s." % (
@@ -277,7 +298,7 @@ def findModule(importing, module_name, parent_package, level, warn):
             importing      = importing,
             module_name    = module_name,
             parent_package = parent_package,
-            level          = level,
+            level          = level
         )
 
     return None, None, "not-found"
@@ -294,32 +315,35 @@ def _findModuleInPath2(module_name, search_path):
         None, if it is a built-in.
     """
     # We have many branches here, because there are a lot of cases to try.
-    # pylint: disable=R0912
+    # pylint: disable=too-many-branches
 
     # We may have to decide between package and module, therefore build
     # a list of candidates.
-    candidates = oset.OrderedSet()
+    candidates = OrderedSet()
 
     considered = set()
 
     for entry in search_path:
         # Don't try again, just with an entry of different casing or complete
         # duplicate.
-        if Utils.normcase(entry) in considered:
+        if os.path.normcase(entry) in considered:
             continue
-        considered.add(Utils.normcase(entry))
+        considered.add(os.path.normcase(entry))
 
         package_directory = os.path.join(entry, module_name)
 
         # First, check for a package with an init file, that would be the
         # first choice.
-        if Utils.isDir(package_directory):
-            for suffix in (".py", ".pyc"):
+        if os.path.isdir(package_directory):
+            for suffix, _mode, mtype in imp.get_suffixes():
+                if mtype == imp.C_EXTENSION:
+                    continue
+
                 package_file_name = "__init__" + suffix
 
                 file_path = os.path.join(package_directory, package_file_name)
 
-                if Utils.isFile(file_path):
+                if os.path.isfile(file_path):
                     candidates.add(
                         (entry, 1, package_directory)
                     )
@@ -332,8 +356,8 @@ def _findModuleInPath2(module_name, search_path):
 
         # Then, check out suffixes of all kinds.
         for suffix, _mode, _type in imp.get_suffixes():
-            file_path = Utils.joinpath(entry, module_name + suffix)
-            if Utils.isFile(file_path):
+            file_path = os.path.join(entry, module_name + suffix)
+            if os.path.isfile(file_path):
                 candidates.add(
                     (entry, 1, file_path)
                 )
@@ -358,10 +382,8 @@ def _findModuleInPath2(module_name, search_path):
             return candidates[0][2]
         else:
             for candidate in candidates:
-                dir_listing = os.listdir(candidate[0])
-
-                for filename in dir_listing:
-                    if Utils.joinpath(candidate[0], filename) == candidate[2]:
+                for fullname, _filename in listDir(candidate[0]):
+                    if fullname == candidate[2]:
                         return candidate[2]
 
             # Only exact case matches matter, all candidates were ignored,
@@ -381,7 +403,7 @@ def getPackageSearchPath(package_name):
 
         result = []
         for element in getPackageSearchPath(parent_package_name):
-            package_dir = Utils.joinpath(
+            package_dir = os.path.join(
                 element,
                 child_package_name
             )
@@ -402,12 +424,12 @@ def getPackageSearchPath(package_name):
             return preloaded_path
 
         def getPackageDirCandidates(element):
-            yield Utils.joinpath(element, package_name), False
+            yield os.path.join(element, package_name), False
 
             # Hack for PyWin32. TODO: Move this "__path__" extensions to be
             # plug-in decisions.
             if package_name == "win32com":
-                yield Utils.joinpath(element, "win32comext"), True
+                yield os.path.join(element, "win32comext"), True
 
         result = []
         for element in getPackageSearchPath(None):
@@ -438,7 +460,7 @@ def _findModuleInPath(module_name, package_name):
     search_path = getPackageSearchPath(package_name)
 
     if _debug_module_finding:
-        print("_findModuleInPath: Using search path", search_path)
+        print("_findModuleInPath: Using search path", search_path, "for", package_name)
 
     try:
         module_filename = _findModuleInPath2(
