@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,16 +19,13 @@
 
 """
 
-from nuitka.nodes.DictionaryNodes import (
-    ExpressionKeyValuePair,
-    ExpressionMakeDict
-)
-from nuitka.nodes.NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 from nuitka.optimizations.BuiltinOptimization import builtin_dict_spec
 
 from .BuiltinIteratorNodes import ExpressionBuiltinIter1
-from .ConstantRefNodes import ExpressionConstantRef
-from .NodeBases import ExpressionChildrenHavingBase
+from .ConstantRefNodes import makeConstantRefNode
+from .DictionaryNodes import ExpressionKeyValuePair, ExpressionMakeDict
+from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 
 
 class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
@@ -49,7 +46,7 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
                 "pos_arg" : pos_arg,
                 "pairs"   : tuple(
                     ExpressionKeyValuePair(
-                        ExpressionConstantRef(key, source_ref),
+                        makeConstantRefNode(key, source_ref),
                         value,
                         value.getSourceReference()
                     )
@@ -77,7 +74,7 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
 
         return True
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         pos_arg = self.getPositionalArgument()
         pairs = self.getNamedArgumentPairs()
 
@@ -114,8 +111,8 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
             )
 
             # Just in case, the iteration may do that.
-            if pos_arg.mayRaiseExceptionIter(BaseException):
-                constraint_collection.onExceptionRaiseExit(BaseException)
+            if not pos_arg.hasShapeSlotIter():
+                trace_collection.onExceptionRaiseExit(BaseException)
 
             return (
                 new_node,
@@ -133,7 +130,7 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
             else:
                 pos_args = None
 
-            return constraint_collection.getCompileTimeComputationResult(
+            return trace_collection.getCompileTimeComputationResult(
                 node        = self,
                 computation = lambda : builtin_dict_spec.simulateCall(
                     (pos_args, self.getNamedArgumentPairs())
@@ -141,7 +138,7 @@ class ExpressionBuiltinDict(ExpressionChildrenHavingBase):
                 description = "Replace 'dict' call with constant arguments."
             )
         else:
-            constraint_collection.onExceptionRaiseExit(BaseException)
+            trace_collection.onExceptionRaiseExit(BaseException)
 
             return self, None, None
 
