@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,12 +19,8 @@
 
 """
 
-from .NodeBases import (
-    ExpressionChildrenHavingBase,
-    ExpressionMixin,
-    NodeBase,
-    StatementChildrenHavingBase
-)
+from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
+from .NodeBases import StatementChildrenHavingBase
 from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
 
@@ -82,15 +78,15 @@ class StatementRaiseException(StatementChildrenHavingBase):
     def isImplicit():
         return False
 
-    def computeStatement(self, constraint_collection):
-        constraint_collection.onExpression(
+    def computeStatement(self, trace_collection):
+        trace_collection.onExpression(
             expression = self.getExceptionType(),
             allow_none = True
         )
         exception_type = self.getExceptionType()
 
         # TODO: Limit by type.
-        constraint_collection.onExceptionRaiseExit(BaseException)
+        trace_collection.onExceptionRaiseExit(BaseException)
 
         if exception_type is not None and \
            exception_type.willRaiseException(BaseException):
@@ -104,7 +100,7 @@ class StatementRaiseException(StatementChildrenHavingBase):
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception type."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionValue(),
             allow_none = True
         )
@@ -121,7 +117,7 @@ Explicit raise already raises implicitly building exception type."""
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception value."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionTrace(),
             allow_none = True
         )
@@ -140,7 +136,7 @@ Explicit raise already raises implicitly building exception value."""
             return result, "new_raise", """\
 Explicit raise already raises implicitly building exception traceback."""
 
-        constraint_collection.onExpression(
+        trace_collection.onExpression(
             expression = self.getExceptionCause(),
             allow_none = True
         )
@@ -213,10 +209,10 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
         "exception_value"
     )
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         return self, None, None
 
-    def computeExpressionDrop(self, statement, constraint_collection):
+    def computeExpressionDrop(self, statement, trace_collection):
         result = StatementRaiseExceptionImplicit(
             exception_type  = self.getExceptionType(),
             exception_value = self.getExceptionValue(),
@@ -257,7 +253,7 @@ class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
 
     getArgs = ExpressionChildrenHavingBase.childGetter("args")
 
-    def computeExpression(self, constraint_collection):
+    def computeExpression(self, trace_collection):
         return self, None, None
 
     def mayRaiseException(self, exception_type):
@@ -268,16 +264,16 @@ class ExpressionBuiltinMakeException(ExpressionChildrenHavingBase):
         return False
 
 
-class ExpressionCaughtExceptionTypeRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionTypeRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TYPE_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
@@ -286,16 +282,16 @@ class ExpressionCaughtExceptionTypeRef(NodeBase, ExpressionMixin):
         return False
 
 
-class ExpressionCaughtExceptionValueRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionValueRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_VALUE_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
@@ -304,16 +300,16 @@ class ExpressionCaughtExceptionValueRef(NodeBase, ExpressionMixin):
         return False
 
 
-class ExpressionCaughtExceptionTracebackRef(NodeBase, ExpressionMixin):
+class ExpressionCaughtExceptionTracebackRef(ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TRACEBACK_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         return self, None, None
 
     def mayHaveSideEffects(self):
