@@ -47,12 +47,55 @@ from .shapes.BuiltinTypeShapes import ShapeTypeBuiltinModule, ShapeTypeModule
 
 
 class ExpressionImportModuleHard(ExpressionBase):
-    """ Hard code import, e.g. of "sys" module as done in Python mechanics.
+    """ Hard coded import names, e.g. of "__future__"
 
+        These are directly created for some Python mechanics, but also due to
+        compile time optimization for imports of statically known modules.
     """
 
     kind = "EXPRESSION_IMPORT_MODULE_HARD"
 
+    __slots__ = "module_name",
+
+    def __init__(self, module_name, source_ref):
+        ExpressionBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+        self.module_name = module_name
+
+    def getDetails(self):
+        return {
+            "module_name" : self.module_name
+        }
+
+    def getModuleName(self):
+        return self.module_name
+
+    def computeExpressionRaw(self, trace_collection):
+        return self, None, None
+
+    def mayHaveSideEffects(self):
+        if self.module_name == "sys":
+            return False
+        elif self.module_name == "__future__":
+            return False
+        else:
+            return True
+
+    def mayRaiseException(self, exception_type):
+        return self.mayHaveSideEffects()
+
+
+class ExpressionImportModuleNameHard(ExpressionBase):
+    """ Hard coded import names, e.g. of "os.path.dirname"
+
+        These are directly created for some Python mechanics.
+    """
+
+    kind = "EXPRESSION_IMPORT_MODULE_NAME_HARD"
+
     __slots__ = "module_name", "import_name"
 
     def __init__(self, module_name, import_name, source_ref):
@@ -108,18 +151,28 @@ class ExpressionBuiltinImport(ExpressionChildrenHavingBase):
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
-                "name"        : name,
-                "globals"     : globals_arg,
-                "locals"      : locals_arg,
-                "fromlist"    : fromlist,
-                "level"       : level
+                "name"     : name,
+                "globals"  : globals_arg,
+                "locals"   : locals_arg,
+                "fromlist" : fromlist,
+                "level"    : level
             },
             source_ref = source_ref
         )
 
         self.recurse_attempted = False
+
+        # The module actually referenced in that import.
         self.imported_module = None
+
+        # The fromlist imported modules if any.
         self.import_list_modules = []
+
+        # For "package.sub_package.module" we also need to import the package,
+        # because the imported_module not be found, as it's not a module, e.g.
+        # in the case of "os.path" or "six.moves".
+        self.package_modules = None
+
         self.finding = None
 
         self.type_shape = ShapeTypeModule
@@ -262,6 +315,31 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
                                 self.import_list_modules.append(
                                     sub_imported_module.getFullName()
                                 )
+        else:
+            while "." in module_name:
+                module_name = ".".join(module_name.split(".")[:-1])
+
+                module_package, module_filename, _finding = findModule(
+                    importing      = self,
+                    module_name    = module_name,
+                    parent_package = parent_package,
+                    level          = level,
+                    warn           = True
+                )
+
+                if module_filename is not None:
+                    package_module = self._consider(
+                        trace_collection = trace_collection,
+                        module_filename  = module_filename,
+                        module_package   = module_package
+                    )
+
+                    if package_module is not None:
+                        if self.package_modules is None:
+                            self.package_modules = []
+
+                        self.package_modules.append(package_module)
+
 
     def _addUsedModules(self, trace_collection):
         if self.finding != "not-found":
@@ -271,6 +349,13 @@ Not recursing to '%(full_path)s' (%(filename)s), please specify \
             for import_list_module in self.import_list_modules:
                 trace_collection.onUsedModule(import_list_module)
 
+        # These are added in any case.
+        if self.package_modules is not None:
+            for package_module in self.package_modules:
+                trace_collection.onUsedModule(package_module.getFullName())
+
+
+
     def computeExpression(self, trace_collection):
         # TODO: In fact, if the module is not a package, we don't have to insist
         # on the "fromlist" that much, but normally it's not used for anything
