@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,7 @@ The top of the tree. Packages are also modules. Modules are what hold a program
 together and cross-module optimizations are the most difficult to tackle.
 """
 
-import re
+import os
 
 from nuitka import Options, Variables
 from nuitka.containers.oset import OrderedSet
@@ -30,30 +30,41 @@ from nuitka.importing.Importing import (
     getModuleNameAndKindFromFilename
 )
 from nuitka.importing.Recursion import decideRecursion, recurseTo
-from nuitka.optimizations.TraceCollections import ConstraintCollectionModule
+from nuitka.ModuleRegistry import getModuleByName, getOwnerFromCodeName
+from nuitka.optimizations.TraceCollections import TraceCollectionModule
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference, fromFilename
-from nuitka.utils import Utils
+from nuitka.utils.CStrings import encodePythonIdentifierToC
+from nuitka.utils.FileOperations import relpath
 
 from .Checkers import checkStatementsSequenceOrNone
-from .CodeObjectSpecs import CodeObjectSpec
-from .ConstantRefNodes import ExpressionConstantRef
-from .FutureSpecs import FutureSpec
+from .ConstantRefNodes import makeConstantRefNode
+from .ExpressionBases import ExpressionBase
+from .FutureSpecs import FutureSpec, fromFlags
+from .IndicatorMixins import EntryPointMixin, MarkNeedsAnnotationsMixin
 from .NodeBases import (
     ChildrenHavingMixin,
-    ClosureGiverNodeBase,
-    ExpressionMixin,
-    NodeBase
+    ClosureGiverNodeMixin,
+    NodeBase,
+    extractKindAndArgsFromXML,
+    fromXML
 )
 
 
-class PythonModuleMixin:
-    def __init__(self, name, package_name):
+class PythonModuleBase(NodeBase):
+    __slots__ = "name", "package_name", "package"
+
+    def __init__(self, name, package_name, source_ref):
         assert type(name) is str, type(name)
         assert '.' not in name, name
         assert package_name is None or \
                (type(package_name) is str and package_name != "")
 
+        NodeBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
         self.name = name
         self.package_name = package_name
         self.package = None
@@ -85,7 +96,10 @@ class PythonModuleMixin:
         result = []
 
         if self.package_name is not None and self.package is None:
-            package_package, package_filename, _finding = findModule(
+            self.package = getModuleByName(self.package_name)
+
+        if self.package_name is not None and self.package is None:
+            package_package, package_filename, finding = findModule(
                 importing      = self,
                 module_name    = self.package_name,
                 parent_package = None,
@@ -101,7 +115,7 @@ class PythonModuleMixin:
             if self.package_name == "uniconvertor.app.modules":
                 return []
 
-            assert package_filename is not None, self.package_name
+            assert package_filename is not None, (self.package_name, finding)
 
             _package_name, package_kind = getModuleNameAndKindFromFilename(package_filename)
             # assert _package_name == self.package_name, (package_filename, _package_name, self.package_name)
@@ -114,18 +128,16 @@ class PythonModuleMixin:
             )
 
             if decision is not None:
-                imported_module, is_added = recurseTo(
+                self.package, is_added = recurseTo(
                     module_package  = package_package,
                     module_filename = package_filename,
-                    module_relpath  = Utils.relpath(package_filename),
+                    module_relpath  = relpath(package_filename),
                     module_kind     = "py",
                     reason          = "Containing package of recursed module '%s'." % self.getFullName(),
                 )
 
-                self.package = imported_module
-
                 if is_added:
-                    result.append(imported_module)
+                    result.append(self.package)
 
         if self.package:
             from nuitka.ModuleRegistry import addUsedModule
@@ -138,13 +150,14 @@ class PythonModuleMixin:
         return result
 
     def getCodeName(self):
-        # Abstract method, pylint: disable=R0201
+        # Abstract method, pylint: disable=no-self-use
         return None
 
     def getCompileTimeFilename(self):
-        return Utils.abspath(self.getSourceReference().getFilename())
+        return os.path.abspath(self.getSourceReference().getFilename())
 
     def getRunTimeFilename(self):
+        # TODO: Don't look at such things this late, push this into building.
         reference_mode = Options.getFileReferenceMode()
 
         if reference_mode == "original":
@@ -156,7 +169,7 @@ class PythonModuleMixin:
 
             full_name = self.getFullName()
 
-            result = Utils.basename(filename)
+            result = os.path.basename(filename)
             current = filename
 
             levels = full_name.count('.')
@@ -164,14 +177,19 @@ class PythonModuleMixin:
                 levels += 1
 
             for _i in range(levels):
-                current = Utils.dirname(current)
-                result = Utils.joinpath(Utils.basename(current), result)
+                current = os.path.dirname(current)
+
+                result = os.path.join(
+                    os.path.basename(current),
+                    result
+                )
 
             return result
 
 
-class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
-                           ClosureGiverNodeBase):
+class CompiledPythonModule(ChildrenHavingMixin, ClosureGiverNodeMixin,
+                           MarkNeedsAnnotationsMixin, EntryPointMixin,
+                           PythonModuleBase):
     """ Compiled Python Module
 
     """
@@ -180,57 +198,48 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
 
     named_children = (
         "body",
+        "functions"
     )
 
     checkers = {
         "body": checkStatementsSequenceOrNone
     }
 
-    def __init__(self, name, package_name, mode, source_ref):
-        ClosureGiverNodeBase.__init__(
-            self,
-            name        = name,
-            code_prefix = "module",
-            source_ref  = source_ref
-        )
-
-        PythonModuleMixin.__init__(
+    def __init__(self, name, package_name, mode, future_spec, source_ref):
+        PythonModuleBase.__init__(
             self,
             name         = name,
-            package_name = package_name
+            package_name = package_name,
+            source_ref   = source_ref
+        )
+
+        ClosureGiverNodeMixin.__init__(
+            self,
+            name        = name,
+            code_prefix = "module"
         )
 
         ChildrenHavingMixin.__init__(
             self,
             values = {
-                "body" : None # delayed
+                "body" : None, # delayed
+                "functions" : (),
             },
         )
 
+        MarkNeedsAnnotationsMixin.__init__(self)
+
+        EntryPointMixin.__init__(self)
+
         self.mode = mode
 
         self.variables = {}
 
-        # The list functions contained in that module.
-        self.functions = OrderedSet()
-
         self.active_functions = OrderedSet()
         self.cross_used_functions = OrderedSet()
 
-        # SSA trace based information about the module.
-        self.constraint_collection = None
-
-        self.code_object = CodeObjectSpec(
-            code_name     = "<module>" if self.isMainModule() else self.getName(),
-            code_kind     = "Module",
-            arg_names     = (),
-            kw_only_count = 0,
-            has_stardict  = False,
-            has_starlist  = False,
-        )
-
-    def getCodeObject(self):
-        return self.code_object
+        # Often "None" until tree building finishes its part.
+        self.future_spec = future_spec
 
     def getDetails(self):
         return {
@@ -239,16 +248,24 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
             "name"     : self.name
         }
 
-    def asXml(self):
-        result = super(CompiledPythonModule, self).asXml()
+    def getDetailsForDisplay(self):
+        result = self.getDetails()
 
-        for function_body in self.active_functions:
-            result.append(function_body.asXml())
+        if self.future_spec is not None:
+            result["code_flags"] = ','.join(self.future_spec.asFlags())
 
         return result
 
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        # Modules are not having any provider, must not be used,
+        assert False
+
+    def getFutureSpec(self):
+        return self.future_spec
+
     def asGraph(self, computation_counter):
-        from graphviz import Digraph # @UnresolvedImport pylint: disable=F0401,I0021
+        from graphviz import Digraph # @UnresolvedImport pylint: disable=I0021,import-error
 
         graph = Digraph("cluster_%d" % computation_counter, comment = "Graph for %s" % self.getName())
         graph.body.append("style=filled")
@@ -265,9 +282,9 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
             )
 
         for function_body in self.active_functions:
-            constraint_collection = function_body.constraint_collection
+            trace_collection = function_body.trace_collection
 
-            for (_variable, _version), variable_trace in constraint_collection.getVariableTracesAll().items():
+            for (_variable, _version), variable_trace in trace_collection.getVariableTracesAll().items():
                 node = makeTraceNodeName(variable_trace)
 
                 previous = variable_trace.getPrevious()
@@ -292,14 +309,19 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
     getBody = ChildrenHavingMixin.childGetter("body")
     setBody = ChildrenHavingMixin.childSetter("body")
 
+    getFunctions = ChildrenHavingMixin.childGetter("functions")
+    setFunctions = ChildrenHavingMixin.childSetter("functions")
+
     @staticmethod
     def isCompiledPythonModule():
         return True
 
     def getParent(self):
-        assert False
+        # We have never have a parent
+        return None
 
     def getParentVariableProvider(self):
+        # We have never have a provider
         return None
 
     def hasVariableName(self, variable_name):
@@ -338,43 +360,37 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
     def getContainingClassDictCreation():
         return None
 
-    def isEarlyClosure(self):
+    @staticmethod
+    def isEarlyClosure():
         # Modules should immediately closure variables on use.
-        # pylint: disable=R0201
         return True
 
-    def getCodeName(self):
-        def r(match):
-            c = match.group()
-            if c == '.':
-                return '$'
-            else:
-                return "$$%d$" % ord(c)
+    def getEntryPoint(self):
+        return self
 
-        return "".join(
-            re.sub("[^a-zA-Z0-9_]", r ,c)
-            for c in
-            self.getFullName()
-        )
+    def getCodeName(self):
+        # For code name of modules, we need to translate to C identifiers,
+        # removing characters illegal for that.
+
+        return encodePythonIdentifierToC(self.getFullName())
 
     def addFunction(self, function_body):
-        assert function_body not in self.functions
-
-        self.functions.add(function_body)
-
-    def getFunctions(self):
-        return self.functions
+        functions = self.getFunctions()
+        assert function_body not in functions
+        functions += (function_body,)
+        self.setFunctions(functions)
 
     def startTraversal(self):
         self.active_functions = OrderedSet()
 
     def addUsedFunction(self, function_body):
-        assert function_body in self.functions
+        assert function_body in self.getFunctions()
 
         assert function_body.isExpressionFunctionBody() or \
                function_body.isExpressionClassBody() or \
                function_body.isExpressionGeneratorObjectBody() or \
-               function_body.isExpressionCoroutineObjectBody()
+               function_body.isExpressionCoroutineObjectBody() or \
+               function_body.isExpressionAsyncgenObjectBody()
 
         if function_body not in self.active_functions:
             self.active_functions.add(function_body)
@@ -383,7 +399,7 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
         return self.active_functions
 
     def getUnusedFunctions(self):
-        for function in self.functions:
+        for function in self.getFunctions():
             if function not in self.active_functions:
                 yield function
 
@@ -394,11 +410,18 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
     def getCrossUsedFunctions(self):
         return self.cross_used_functions
 
+    def getFunctionFromCodeName(self, code_name):
+        for function in self.getFunctions():
+            if function.getCodeName() == code_name:
+                return function
+
     def getOutputFilename(self):
         main_filename = self.getFilename()
 
         if main_filename.endswith(".py"):
             result = main_filename[:-3]
+        elif main_filename.endswith(".pyw"):
+            result = main_filename[:-4]
         else:
             result = main_filename
 
@@ -406,22 +429,16 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
         # Scons or unknown, so lets avoid them for now.
         return result.replace(')',"").replace('(',"")
 
-    # TODO: Can't really use locals for modules, this should probably be made
-    # sure to not be used.
-    @staticmethod
-    def getLocalsMode():
-        return "copy"
-
     def computeModule(self):
-        old_collection = self.constraint_collection
+        old_collection = self.trace_collection
 
-        self.constraint_collection = ConstraintCollectionModule(self)
+        self.trace_collection = TraceCollectionModule(self)
 
         module_body = self.getBody()
 
         if module_body is not None:
             result = module_body.computeStatementsSequence(
-                constraint_collection = self.constraint_collection
+                trace_collection = self.trace_collection
             )
 
             if result is not module_body:
@@ -430,37 +447,84 @@ class CompiledPythonModule(PythonModuleMixin, ChildrenHavingMixin,
         new_modules = self.attemptRecursion()
 
         for new_module in new_modules:
-            self.constraint_collection.signalChange(
+            self.trace_collection.signalChange(
                 source_ref = new_module.getSourceReference(),
                 tags       = "new_code",
                 message    = "Recursed to module package."
             )
 
-        self.constraint_collection.updateFromCollection(old_collection)
+        self.trace_collection.updateVariablesFromCollection(old_collection)
 
     def getTraceCollections(self):
-        yield self.constraint_collection
+        yield self.trace_collection
 
         for function in self.getUsedFunctions():
-            yield function.constraint_collection
+            yield function.trace_collection
+
+    def markAsLocalsDict(self):
+        # Does not apply to modules.
+        pass
+
+    def hasLocalsDict(self):
+        return False
+
+    def isUnoptimized(self):
+        return False
+
+    def getLocalVariables(self):
+        return ()
+
+    def getUserLocalVariables(self):
+        return ()
+
+    def getOutlineLocalVariables(self):
+        outlines = self.getTraceCollection().getOutlineFunctions()
+
+        if outlines is None:
+            return ()
+
+        result = []
+
+        for outline in outlines:
+            result.extend(outline.getUserLocalVariables())
+
+        return tuple(result)
+
+    def hasClosureVariable(self, variable):
+        return False
+
+    def removeUserVariable(self, variable):
+        outlines = self.getTraceCollection().getOutlineFunctions()
+
+        for outline in outlines:
+            user_locals = outline.getUserLocalVariables()
+
+            if variable in user_locals:
+                outline.removeUserVariable(variable)
+                break
 
 
 class CompiledPythonPackage(CompiledPythonModule):
     kind = "COMPILED_PYTHON_PACKAGE"
 
-    def __init__(self, name, package_name, mode, source_ref):
-        assert name
+    def __init__(self, name, package_name, mode, future_spec, source_ref):
+        assert name, source_ref
 
         CompiledPythonModule.__init__(
             self,
             name         = name,
             package_name = package_name,
             mode         = mode,
+            future_spec  = future_spec,
             source_ref   = source_ref
         )
 
     def getOutputFilename(self):
-        return Utils.dirname(self.getFilename())
+        return os.path.dirname(self.getFilename())
+
+    @staticmethod
+    def canHaveExternalImports():
+        return True
 
 
 def makeUncompiledPythonModule(module_name, filename, bytecode, is_package,
@@ -493,24 +557,22 @@ def makeUncompiledPythonModule(module_name, filename, bytecode, is_package,
         )
 
 
-class UncompiledPythonModule(PythonModuleMixin, NodeBase):
+class UncompiledPythonModule(PythonModuleBase):
     """ Compiled Python Module
 
     """
 
     kind = "UNCOMPILED_PYTHON_MODULE"
 
+    __slots__ = "bytecode", "filename", "user_provided", "technical", "used_modules"
+
     def __init__(self, name, package_name, bytecode, filename, user_provided,
                  technical, source_ref):
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-        PythonModuleMixin.__init__(
+        PythonModuleBase.__init__(
             self,
             name         = name,
-            package_name = package_name
+            package_name = package_name,
+            source_ref   = source_ref
         )
 
         self.bytecode = bytecode
@@ -552,42 +614,102 @@ class UncompiledPythonPackage(UncompiledPythonModule):
     kind = "UNCOMPILED_PYTHON_PACKAGE"
 
 
-class SingleCreationMixin:
-    created = set()
-
-    def __init__(self):
-        assert self.__class__ not in self.created
-        self.created.add(self.__class__)
-
-
-class PythonMainModule(CompiledPythonModule, SingleCreationMixin):
+class PythonMainModule(CompiledPythonModule):
     kind = "PYTHON_MAIN_MODULE"
 
-    def __init__(self, main_added, mode, source_ref):
+    def __init__(self, main_added, mode, future_spec, source_ref):
         CompiledPythonModule.__init__(
             self,
             name         = "__main__",
             package_name = None,
             mode         = mode,
+            future_spec  = future_spec,
             source_ref   = source_ref
         )
 
-        SingleCreationMixin.__init__(self)
-
         self.main_added = main_added
 
+    def getDetails(self):
+        return {
+            "filename"   : self.source_ref.getFilename(),
+            "main_added" : self.main_added,
+            "mode"       : self.mode
+        }
+
+    @classmethod
+    def fromXML(cls, provider, source_ref, **args):
+        if "code_flags" in args:
+            future_spec = fromFlags(args["code_flags"])
+
+        result = cls(
+            main_added  = args["main_added"] == "True",
+            mode        = args["mode"],
+            future_spec = future_spec,
+            source_ref  = source_ref
+        )
+
+        from nuitka.ModuleRegistry import addRootModule
+        addRootModule(result)
+
+        function_work = []
+
+        for xml in args["functions"]:
+            _kind, node_class, func_args, source_ref = extractKindAndArgsFromXML(xml, source_ref)
+
+            if "provider" in func_args:
+                func_args["provider"] = getOwnerFromCodeName(func_args["provider"])
+            else:
+                func_args["provider"] = result
+
+            if "flags" in args:
+                func_args["flags"] = set(func_args["flags"].split(','))
+
+            if "doc" not in args:
+                func_args["doc"] = None
+
+            function = node_class.fromXML(
+                source_ref = source_ref,
+                **func_args
+            )
+
+            # Could do more checks for look up of body here, but so what...
+            function_work.append(
+                (function, iter(iter(xml).next()).next())
+            )
+
+        for function, xml in function_work:
+            function.setChild(
+                "body",
+                fromXML(
+                    provider   = function,
+                    xml        = xml,
+                    source_ref = function.getSourceReference()
+                )
+            )
+
+        result.setChild(
+            "body",
+            fromXML(
+                provider   = result,
+                xml        = args["body"][0],
+                source_ref = source_ref
+            )
+        )
+
+        return result
+
     @staticmethod
     def isMainModule():
         return True
 
     def getOutputFilename(self):
         if self.main_added:
-            return Utils.dirname(self.getFilename())
+            return os.path.dirname(self.getFilename())
         else:
             return CompiledPythonModule.getOutputFilename(self)
 
 
-class PythonInternalModule(CompiledPythonModule, SingleCreationMixin):
+class PythonInternalModule(CompiledPythonModule):
     kind = "PYTHON_INTERNAL_MODULE"
 
     def __init__(self):
@@ -597,14 +719,12 @@ class PythonInternalModule(CompiledPythonModule, SingleCreationMixin):
             package_name = None,
             mode         = "compiled",
             source_ref   = SourceCodeReference.fromFilenameAndLine(
-                filename    = "internal",
-                line        = 0,
-                future_spec = FutureSpec()
-            )
+                filename = "internal",
+                line     = 0
+            ),
+            future_spec  = FutureSpec()
         )
 
-        SingleCreationMixin.__init__(self)
-
     @staticmethod
     def isInternalModule():
         return True
@@ -613,25 +733,23 @@ class PythonInternalModule(CompiledPythonModule, SingleCreationMixin):
         return "__internal"
 
 
-class PythonShlibModule(PythonModuleMixin, NodeBase):
+class PythonShlibModule(PythonModuleBase):
     kind = "PYTHON_SHLIB_MODULE"
 
+    __slots__ = ("used_modules",)
+
     avoid_duplicates = set()
 
     def __init__(self, name, package_name, source_ref):
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
-
-        PythonModuleMixin.__init__(
+        PythonModuleBase.__init__(
             self,
             name         = name,
-            package_name = package_name
+            package_name = package_name,
+            source_ref   = source_ref
         )
 
         # That would be a mistake we just made.
-        assert Utils.basename(source_ref.getFilename()) != "<frozen>"
+        assert os.path.basename(source_ref.getFilename()) != "<frozen>"
 
         # That is too likely a bug.
         assert name != "__main__"
@@ -641,6 +759,8 @@ class PythonShlibModule(PythonModuleMixin, NodeBase):
         assert self.getFullName() not in self.avoid_duplicates, self.getFullName()
         self.avoid_duplicates.add(self.getFullName())
 
+        self.used_modules = None
+
     def getDetails(self):
         return {
             "name"         : self.name,
@@ -653,12 +773,78 @@ class PythonShlibModule(PythonModuleMixin, NodeBase):
     def startTraversal(self):
         pass
 
+    def getPyIFilename(self):
+        """ Get Python type description filename. """
 
-class ExpressionModuleFileAttributeRef(NodeBase, ExpressionMixin):
+        path = self.getFilename()
+        filename = os.path.basename(path)
+        dirname = os.path.dirname(path)
+
+        return os.path.join(dirname, filename.split('.')[0]) + ".pyi"
+
+    def _readPyPIFile(self):
+        """ Read the .pyi file if present and scan for dependencies. """
+
+        if self.used_modules is None:
+            pyi_filename = self.getPyIFilename()
+
+            if os.path.exists(pyi_filename):
+                pyi_deps = OrderedSet()
+
+                for line in open(pyi_filename):
+                    line = line.strip()
+
+                    if line.startswith("import "):
+                        imported = line[7:]
+
+                        pyi_deps.add(imported)
+                    elif line.startswith("from "):
+                        parts = line.split(None, 3)
+                        assert parts[0] == "from"
+                        assert parts[2] == "import"
+
+                        if parts[1] == "typing":
+                            continue
+
+                        pyi_deps.add(parts[1])
+
+                        imported = parts[3]
+                        if imported.startswith('('):
+                            # No multiline imports please
+                            assert imported.endswith(')')
+                            imported = imported[1:-1]
+
+                            assert imported
+
+                        if imported == '*':
+                            continue
+
+                        for name in imported.split(','):
+                            name = name.strip()
+
+                            pyi_deps.add(parts[1] + '.' + name)
+
+                if "typing" in pyi_deps:
+                    pyi_deps.discard("typing")
+
+                self.used_modules = tuple(pyi_deps)
+            else:
+                self.used_modules = ()
+
+    def getUsedModules(self):
+        self._readPyPIFile()
+
+        return self.used_modules
+
+    def getParentModule(self):
+        return self
+
+
+class ExpressionModuleFileAttributeRef(ExpressionBase):
     kind = "EXPRESSION_MODULE_FILE_ATTRIBUTE_REF"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
+        ExpressionBase.__init__(
             self,
             source_ref = source_ref
         )
@@ -666,11 +852,11 @@ class ExpressionModuleFileAttributeRef(NodeBase, ExpressionMixin):
     def mayRaiseException(self, exception_type):
         return False
 
-    def computeExpression(self, constraint_collection):
+    def computeExpressionRaw(self, trace_collection):
         # There is not a whole lot to do here, the path will change at run
         # time
         if Options.getFileReferenceMode() != "runtime":
-            result = ExpressionConstantRef(
+            result = makeConstantRefNode(
                 constant   = self.getRunTimeFilename(),
                 source_ref = self.getSourceReference()
             )
@@ -684,3 +870,19 @@ class ExpressionModuleFileAttributeRef(NodeBase, ExpressionMixin):
 
     def getRunTimeFilename(self):
         return self.getParentModule().getRunTimeFilename()
+
+
+class ExpressionModuleLoaderRef(ExpressionBase):
+    kind = "EXPRESSION_MODULE_LOADER_REF"
+
+    def __init__(self, source_ref):
+        ExpressionBase.__init__(
+            self,
+            source_ref = source_ref
+        )
+
+    def mayRaiseException(self, exception_type):
+        return False
+
+    def computeExpressionRaw(self, trace_collection):
+        return self, None, None
