@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,7 +21,8 @@ Sometimes, the effect of an expression needs to be had, but the value itself
 does not matter at all.
 """
 
-from .NodeBases import ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionChildrenHavingBase
+from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
 
 def checkSideEffects(value):
@@ -52,6 +53,9 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
     }
 
     def __init__(self, side_effects, expression, source_ref):
+        # We expect to be not used without there actually being side effects.
+        assert side_effects
+
         ExpressionChildrenHavingBase.__init__(
             self,
             values     = {
@@ -66,7 +70,10 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
 
     getExpression = ExpressionChildrenHavingBase.childGetter("expression")
 
-    def computeExpression(self, constraint_collection):
+    def isExpressionSideEffects(self):
+        return True
+
+    def computeExpression(self, trace_collection):
         side_effects = self.getSideEffects()
         new_side_effects = []
 
@@ -85,26 +92,26 @@ class ExpressionSideEffects(ExpressionChildrenHavingBase):
 
             return expression, "new_expression", "Remove nested side effects."
 
-        if new_side_effects != side_effects:
-            self.setSideEffects(new_side_effects)
-
         if not new_side_effects:
             return expression, "new_expression", "Removed empty side effects."
 
+        if new_side_effects != side_effects:
+            self.setSideEffects(new_side_effects)
+
         return self, None, None
 
     def willRaiseException(self, exception_type):
         for child in self.getVisitableNodes():
             if child.willRaiseException(exception_type):
                 return True
+
         return False
 
     def getTruthValue(self):
         return self.getExpression().getTruthValue()
 
-    def computeExpressionDrop(self, statement, constraint_collection):
+    def computeExpressionDrop(self, statement, trace_collection):
         # Side effects can  become statements.
-        from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
         expressions = self.getSideEffects() + (self.getExpression(),)
 
