@@ -45,7 +45,7 @@ class ExpressionVariableNameRef(ExpressionBase):
 
     kind = "EXPRESSION_VARIABLE_NAME_REF"
 
-    __slots__ = "variable_name",
+    __slots__ = ("variable_name",)
 
     def __init__(self, variable_name, source_ref):
         ExpressionBase.__init__(
@@ -225,15 +225,6 @@ Module variable '%s' found to be built-in reference.""" % (
                 change_tags = "new_constant"
                 change_desc = """\
 Replaced read-only module attribute '__name__' with constant value."""
-            elif variable_name == "__package__":
-                new_node = makeConstantRefNode(
-                    constant   = variable.getOwner().getPackage(),
-                    source_ref = self.getSourceReference()
-                )
-
-                change_tags = "new_constant"
-                change_desc = """\
-Replaced read-only module attribute '__package__' with constant value."""
             else:
                 self.variable_trace.addUsage()
 
@@ -273,7 +264,7 @@ local variable '%s' referenced before assignment""" % variable_name
            self.variable.getName() in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
            self.variable.isModuleVariable():
             # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage()
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return call_node, None, None
 
@@ -652,7 +643,7 @@ Replaced read-only module attribute '__package__' with constant value."""
            self.variable_name in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
            self.fallback_variable.isModuleVariable():
             # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage()
+            trace_collection.onLocalsUsage(self.getParentVariableProvider())
 
         return call_node, None, None
 
