@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,14 +32,28 @@ Variable version can start as:
 
 from logging import debug
 
+from nuitka.codegen.c_types.CTypePyObjectPtrs import (
+    CTypeCellObject,
+    CTypePyObjectPtr,
+    CTypePyObjectPtrPtr
+)
+from nuitka.Options import isExperimental
 from nuitka.utils import InstanceCounters
 
+enable_bool_ctype = isExperimental("enable_bool_ctype")
 
-class VariableTraceBase:
-    # We are going to have many instance attributes, pylint: disable=R0902
+
+class VariableTraceBase(object):
+    # We are going to have many instance attributes, pylint: disable=too-many-instance-attributes
+
+    __slots__ = (
+        "owner", "variable", "version", "usage_count", "has_potential_usages",
+        "name_usages", "closure_usages", "is_escaped", "previous"
+    )
 
     @InstanceCounters.counted_init
-    def __init__(self, variable, version, previous):
+    def __init__(self, owner, variable, version, previous):
+        self.owner = owner
         self.variable = variable
         self.version = version
 
@@ -49,11 +63,10 @@ class VariableTraceBase:
         # Potential usages indicator that an assignment value may be used.
         self.has_potential_usages = False
 
-        # If False, this indicates the trace has no explicit releases.
-        self.has_releases = False
+        # If 0, this indicates, the variable name needs to be assigned as name.
+        self.name_usages = 0
 
-        # If False, this indicates, the variable name needs to be assigned.
-        self.has_name_usages = False
+        self.closure_usages = False
 
         # If False, this indicates that the value is not yet escaped.
         self.is_escaped = False
@@ -66,21 +79,25 @@ class VariableTraceBase:
     def getVariable(self):
         return self.variable
 
+    def getOwner(self):
+        return self.owner
+
     def getVersion(self):
         return self.version
 
+    def addClosureUsage(self):
+        self.addUsage()
+        self.closure_usages = True
+
     def addUsage(self):
         self.usage_count += 1
 
     def addPotentialUsage(self):
         self.has_potential_usages = True
 
-    def addRelease(self):
-        self.has_releases = True
-
     def addNameUsage(self):
         self.usage_count += 1
-        self.has_name_usages = True
+        self.name_usages += 1
 
     def onValueEscape(self):
         self.is_escaped = True
@@ -97,12 +114,42 @@ class VariableTraceBase:
     def hasPotentialUsages(self):
         return self.has_potential_usages
 
-    def hasNameUsages(self):
-        return self.has_name_usages
+    def getNameUsageCount(self):
+        return self.name_usages
 
     def getPrevious(self):
         return self.previous
 
+    def getPickedCType(self, context):
+        """ Return type to use for specific context. """
+
+        user = context.getEntryPoint()
+        owner = self.variable.getEntryPoint()
+
+        if owner is user:
+            if self.variable.isSharedTechnically():
+                result = CTypeCellObject
+            else:
+                if enable_bool_ctype:
+                    shapes = self.variable.getTypeShapes()
+
+                    if len(shapes) > 1:
+                        return CTypePyObjectPtr
+                    else:
+                        assert shapes, self
+                        return shapes.pop().getCType()
+                else:
+                    return CTypePyObjectPtr
+        elif context.isForDirectCall():
+            if self.variable.isSharedTechnically():
+                result = CTypeCellObject
+            else:
+                result = CTypePyObjectPtrPtr
+        else:
+            result = CTypeCellObject
+
+        return result
+
     @staticmethod
     def isAssignTrace():
         return False
@@ -127,8 +174,7 @@ class VariableTraceBase:
         # TODO: Temporarily disable far reaching of assumptions, until value
         # escaping can be trusted.
         if self.variable.isModuleVariable() or \
-           self.variable.isMaybeLocalVariable() or \
-           self.variable.isSharedTechnically():
+           self.variable.isSharedTechnically() is not False:
             return False
 
         # Merge traces have this overloaded.
@@ -137,21 +183,29 @@ class VariableTraceBase:
 
     def mustNotHaveValue(self):
         if self.variable.isModuleVariable() or \
-           self.variable.isSharedTechnically():
+           self.variable.isSharedTechnically() is not False:
             return False
 
         return self.isUninitTrace()
 
     def getReplacementNode(self, usage):
-        # Virtual method, pylint: disable=R0201,W0613
+        # Virtual method, pylint: disable=no-self-use,unused-argument
 
         return None
 
+    def hasShapeDictionaryExact(self):
+        # Virtual method, pylint: disable=no-self-use
+        return False
+
+
 
 class VariableTraceUninit(VariableTraceBase):
-    def __init__(self, variable, version, previous):
+    __slots__ = ()
+
+    def __init__(self, owner, variable, version, previous):
         VariableTraceBase.__init__(
             self,
+            owner    = owner,
             variable = variable,
             version  = version,
             previous = previous
@@ -181,14 +235,14 @@ class VariableTraceUninit(VariableTraceBase):
         if self.is_escaped:
             debug("  -> value escapes")
 
-        if self.has_releases:
-            debug("   -> has released")
-
 
 class VariableTraceInit(VariableTraceBase):
-    def __init__(self, variable, version):
+    __slots__ = ()
+
+    def __init__(self, owner, variable, version):
         VariableTraceBase.__init__(
             self,
+            owner    = owner,
             variable = variable,
             version  = version,
             previous = None
@@ -214,18 +268,16 @@ class VariableTraceInit(VariableTraceBase):
         if self.is_escaped:
             debug("  -> value escapes")
 
-        if self.has_releases:
-            debug("   -> has released")
-
     @staticmethod
     def isInitTrace():
         return True
 
 
 class VariableTraceUnknown(VariableTraceBase):
-    def __init__(self, variable, version, previous):
+    def __init__(self, owner, variable, version, previous):
         VariableTraceBase.__init__(
             self,
+            owner    = owner,
             variable = variable,
             version  = version,
             previous = previous
@@ -251,9 +303,6 @@ class VariableTraceUnknown(VariableTraceBase):
         if self.is_escaped:
             debug("  -> value escapes")
 
-        if self.has_releases:
-            debug("   -> has released")
-
     @staticmethod
     def isUnknownTrace():
         return True
@@ -281,9 +330,12 @@ class VariableTraceUnknown(VariableTraceBase):
 
 
 class VariableTraceAssign(VariableTraceBase):
-    def __init__(self, assign_node, variable, version, previous):
+    __slots__ = ("assign_node", "replace_it")
+
+    def __init__(self, owner, assign_node, variable, version, previous):
         VariableTraceBase.__init__(
             self,
+            owner    = owner,
             variable = variable,
             version  = version,
             previous = previous
@@ -301,9 +353,7 @@ class VariableTraceAssign(VariableTraceBase):
         )
 
     def dump(self):
-        debug("Trace of %s %d:",
-            self.variable,
-            self.version)
+        debug("Trace of %s %d:", self.variable, self.version)
         debug("  Starts assigned")
 
         if self.usage_count:
@@ -312,9 +362,6 @@ class VariableTraceAssign(VariableTraceBase):
         if self.is_escaped:
             debug("  -> value escapes")
 
-        if self.has_releases:
-            debug("   -> has released")
-
     @staticmethod
     def isAssignTrace():
         return True
@@ -326,12 +373,14 @@ class VariableTraceAssign(VariableTraceBase):
         self.replace_it = replacement
 
     def getReplacementNode(self, usage):
-
         if self.replace_it is not None:
             return self.replace_it(usage)
         else:
             return None
 
+    def hasShapeDictionaryExact(self):
+        return self.assign_node.getAssignSource().hasShapeDictionaryExact()
+
 
 class VariableTraceMerge(VariableTraceBase):
     """ Merge of two or more traces.
@@ -340,9 +389,13 @@ class VariableTraceMerge(VariableTraceBase):
         SSA theory. Also used for merging multiple "return", "break" or
         "continue" exits.
     """
+
+    __slots__ = ()
+
     def __init__(self, variable, version, traces):
         VariableTraceBase.__init__(
             self,
+            owner    = traces[0].owner,
             variable = variable,
             version  = version,
             previous = tuple(traces)
@@ -362,11 +415,7 @@ class VariableTraceMerge(VariableTraceBase):
         return True
 
     def dump(self):
-        debug(
-            "Trace of %s %d:",
-            self.variable,
-            self.version
-        )
+        debug("Trace of %s %d:", self.variable, self.version)
 
         debug(
             "  Merge of %s",
@@ -377,7 +426,7 @@ class VariableTraceMerge(VariableTraceBase):
         # TODO: Temporarily disable far reaching of assumptions, until value
         # escaping can be trusted.
         if self.variable.isModuleVariable() or \
-           self.variable.isSharedTechnically():
+           self.variable.isSharedTechnically() is not False:
             return False
 
         for previous in self.previous:
@@ -388,7 +437,7 @@ class VariableTraceMerge(VariableTraceBase):
 
     def mustNotHaveValue(self):
         if self.variable.isModuleVariable() or \
-           self.variable.isSharedTechnically():
+           self.variable.isSharedTechnically() is not False:
             return False
 
         for previous in self.previous:
@@ -419,6 +468,13 @@ class VariableTraceMerge(VariableTraceBase):
             for previous in self.previous:
                 previous.addPotentialUsage()
 
+    def hasShapeDictionaryExact(self):
+        for previous in self.previous:
+            if not previous.hasShapeDictionaryExact():
+                return False
+
+        return True
+
 
 class VariableTraceLoopMerge(VariableTraceBase):
     """ Merge of loop wrap around with loop start value.
@@ -429,11 +485,14 @@ class VariableTraceLoopMerge(VariableTraceBase):
 
         They will start out with just one previous, and later be updated with
         all of the variable versions at loop continue times.
-        .
     """
+
+    __slots__ = ("loop_finished",)
+
     def __init__(self, variable, version, previous):
         VariableTraceBase.__init__(
             self,
+            owner    = previous.owner,
             variable = variable,
             version  = version,
             previous = previous
@@ -455,11 +514,11 @@ class VariableTraceLoopMerge(VariableTraceBase):
 
         return self.has_potential_usages
 
-    def hasNameUsages(self):
+    def getNameUsageCount(self):
         if not self.loop_finished:
-            return True
+            return 10000
 
-        return self.has_name_usages
+        return self.name_usages
 
     def getPrevious(self):
         assert self.loop_finished
