@@ -124,6 +124,8 @@ print(sys.executable);\
         os.environ["COVERAGE_FILE"] = os.path.join(
             os.path.dirname(__file__),
             "..",
+            "..",
+            "..",
             ".coverage"
         )
 
@@ -276,6 +278,24 @@ def decideFilenameVersionSkip(filename):
     return True
 
 
+def _removeCPythonTestSuiteDir():
+    # Cleanup, some tests apparently forget that.
+    try:
+        if os.path.isdir("@test"):
+            removeDirectory("@test", ignore_errors = False)
+        elif os.path.isfile("@test"):
+            os.unlink("@test")
+    except OSError:
+        # TODO: Move this into removeDirectory maybe. Doing an external
+        # call as last resort could be a good idea.
+
+        # This seems to work for broken "lnk" files.
+        if os.name == "nt":
+            os.system("rmdir /S /Q @test")
+
+        if os.path.exists("@test"):
+            raise
+
 def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
     """ Call the comparison tool. For a given directory filename.
 
@@ -284,7 +304,6 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
 
     """
 
-    # pylint: disable=too-many-branches
     if dirname is None:
         path = filename
     else:
@@ -308,6 +327,9 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
 
     command += search_mode.getExtraFlags(dirname, filename)
 
+    # Cleanup before and after test stage directory.
+    _removeCPythonTestSuiteDir()
+
     try:
         result = subprocess.call(
             command
@@ -315,22 +337,8 @@ def compareWithCPython(dirname, filename, extra_flags, search_mode, needs_2to3):
     except KeyboardInterrupt:
         result = 2
 
-    # Cleanup, some tests apparently forget that.
-    try:
-        if os.path.isdir("@test"):
-            removeDirectory("@test", ignore_errors = False)
-        elif os.path.isfile("@test"):
-            os.unlink("@test")
-    except OSError:
-        # TODO: Move this into removeDirectory maybe. Doing an external
-        # call as last resort could be a good idea.
-
-        # This seems to work for broken "lnk" files.
-        if os.name == "nt":
-            os.system("rmdir /S /Q @test")
-
-        if os.path.exists("@test"):
-            raise
+    # Cleanup before and after test stage directory.
+    _removeCPythonTestSuiteDir()
 
     if result != 0 and \
        result != 2 and \
@@ -353,7 +361,7 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
         path = os.path.join(dirname, filename)
 
     command = [
-        sys.executable,
+        _python_executable,
         "-mcompileall",
         path
     ]
@@ -372,6 +380,21 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
         sys.exit(result)
 
 
+def checkSucceedsWithCPython(filename):
+    command = [
+        _python_executable,
+        filename
+    ]
+
+    result = subprocess.call(
+        command,
+        stdout = open(os.devnull,'w'),
+        stderr = subprocess.STDOUT
+    )
+
+    return result == 0
+
+
 def hasDebugPython():
     # On Debian systems, these work.
     debug_python = os.path.join("/usr/bin/", os.environ["PYTHON"] + "-dbg")
@@ -602,6 +625,100 @@ Error, needs 'strace' on your system to scan used libraries."""
     return result
 
 
+def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
+    result = []
+
+    for loaded_filename in loaded_filenames:
+        loaded_filename = os.path.normpath(loaded_filename)
+        loaded_filename = os.path.normcase(loaded_filename)
+        loaded_basename = os.path.basename(loaded_filename)
+
+        ok = False
+        for entry in white_list:
+            if loaded_filename.startswith(entry):
+                ok = True
+
+            while entry:
+                old_entry = entry
+                entry = os.path.dirname(entry)
+
+                if old_entry == entry:
+                    break
+
+                if loaded_filename == entry:
+                    ok = True
+                    break
+        if ok:
+            continue
+
+        if loaded_filename.startswith("/etc/"):
+            continue
+
+        if loaded_filename.startswith("/proc/") or loaded_filename == "/proc":
+            continue
+
+        if loaded_filename.startswith("/dev/"):
+            continue
+
+        if loaded_filename.startswith("/tmp/"):
+            continue
+
+        if loaded_filename.startswith("/run/"):
+            continue
+
+        if loaded_filename.startswith("/sys/"):
+            continue
+
+        if loaded_filename.startswith("/usr/lib/locale/"):
+            continue
+
+        if loaded_filename.startswith("/usr/share/locale/"):
+            continue
+
+        if loaded_filename.startswith("/usr/share/X11/locale/"):
+            continue
+
+        # Themes may of course be loaded.
+        if loaded_filename.startswith("/usr/share/themes"):
+            continue
+        if "gtk" in loaded_filename and "/engines/" in loaded_filename:
+            continue
+
+        # Terminal info files are OK too.
+        if loaded_filename.startswith("/lib/terminfo/"):
+            continue
+
+        # System C libraries are to be expected.
+        if loaded_basename.startswith((
+            "libc.so.",
+            "libpthread.so.",
+            "libdl.so.",
+            "libm.so.",
+        )):
+            continue
+
+        # Taking these from system is harmless and desirable
+        if loaded_basename.startswith((
+            "libz.so",
+            "libutil.so",
+            "libgcc_s.so",
+        )):
+            continue
+
+        # TODO: Unclear, loading gconv from filesystem of installed system
+        # may be OK or not. I think it should be.
+        if loaded_basename == "gconv-modules.cache":
+            continue
+        if "/gconv/" in loaded_filename:
+            continue
+        if loaded_basename.startswith("libicu"):
+            continue
+
+        result.append(loaded_filename)
+
+    return result
+
+
 def hasModule(module_name):
     result = subprocess.call(
         (
@@ -720,7 +837,7 @@ def createSearchMode():
         )
     elif search_mode and start_at:
         start_at = start_at.replace('/', os.path.sep)
-        return SearchModeByPattern()
+        return SearchModeByPattern(start_at)
     else:
         class SearchModeImmediate(SearchModeBase):
             def abortOnFinding(self, dirname, filename):
