@@ -15,6 +15,11 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+""" Search modes for Nuitka's test runner.
+
+The test runner can handle found errors, skip tests, etc. with search
+modes, which are implemented here.
+"""
 
 import hashlib
 import os
@@ -103,14 +108,20 @@ class SearchModeResume(SearchModeBase):
         SearchModeBase.__init__(self)
 
         tests_path = os.path.normcase(os.path.abspath(tests_path))
+        version = sys.version
+
         if str is not bytes:
             tests_path = tests_path.encode("utf8")
+            version = version.encode("utf8")
+
+        case_hash = hashlib.md5(tests_path)
+        case_hash.update(version)
 
         from .Common import getTestingCacheDir
 
         cache_filename = os.path.join(
             getTestingCacheDir(),
-            hashlib.md5(tests_path).hexdigest()
+            case_hash.hexdigest()
         )
 
         self.cache_filename = cache_filename
