@@ -20,7 +20,7 @@
 Does all the Python parsing and puts it into a tree structure for use in later
 stages of the compilation process.
 
-In the "nuitka.tree.Helpers" module, the dispatching is happening. One function
+In the "nuitka.tree.TreeHelpers" module, the dispatching is happening. One function
 deals with every node kind as found in the AST. The parsing is centered around
 the module "ast" output.
 
@@ -80,11 +80,13 @@ from nuitka.nodes.ConstantRefNodes import (
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import StatementRaiseException
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
+from nuitka.nodes.ImportNodes import ExpressionImportModuleNameHard
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleNodes import (
     CompiledPythonModule,
     CompiledPythonPackage,
     ExpressionModuleFileAttributeRef,
+    ExpressionModuleLoaderRef,
     PythonMainModule,
     PythonShlibModule
 )
@@ -107,22 +109,6 @@ from nuitka.utils import MemoryUsage
 from nuitka.utils.FileOperations import splitPath
 
 from . import SyntaxErrors
-from .Helpers import (
-    buildNode,
-    buildNodeList,
-    buildStatementsNode,
-    extractDocFromBody,
-    getBuildContext,
-    getKind,
-    makeAbsoluteImportNode,
-    makeModuleFrame,
-    makeStatementsSequence,
-    makeStatementsSequenceFromStatement,
-    mangleName,
-    mergeStatements,
-    parseSourceCodeToAst,
-    setBuildingDispatchers
-)
 from .ReformulationAssertStatements import buildAssertNode
 from .ReformulationAssignmentStatements import (
     buildAnnAssignNode,
@@ -175,6 +161,23 @@ from .SourceReading import (
     checkPythonVersionFromCode,
     readSourceCodeFromFilename
 )
+from .TreeHelpers import (
+    buildNode,
+    buildNodeList,
+    buildStatementsNode,
+    extractDocFromBody,
+    getBuildContext,
+    getKind,
+    makeAbsoluteImportNode,
+    makeCallNode,
+    makeModuleFrame,
+    makeStatementsSequence,
+    makeStatementsSequenceFromStatement,
+    mangleName,
+    mergeStatements,
+    parseSourceCodeToAst,
+    setBuildingDispatchers
+)
 from .VariableClosure import completeVariableClosures
 
 
@@ -803,9 +806,52 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
         if provider.isCompiledPythonPackage():
             # This assigns "__path__" value.
             statements.append(
-                createPathAssignment(internal_source_ref)
+                createPathAssignment(provider, internal_source_ref)
             )
 
+    if python_version >= 330:
+        statements.append(
+            StatementAssignmentVariableName(
+                variable_name = "__loader__",
+                source        = ExpressionModuleLoaderRef(
+                    source_ref = internal_source_ref,
+                ),
+                source_ref    = internal_source_ref
+            )
+        )
+
+    if python_version >= 340:
+        if provider.isMainModule():
+            spec_value = makeConstantRefNode(
+                constant   = None,
+                source_ref = internal_source_ref
+            )
+        else:
+            spec_value = makeCallNode(
+                ExpressionImportModuleNameHard(
+                    module_name = "importlib._bootstrap",
+                    import_name = "ModuleSpec",
+                    source_ref  = internal_source_ref,
+                ),
+                makeConstantRefNode(
+                    constant      = provider.getFullName(),
+                    source_ref    = internal_source_ref,
+                    user_provided = True
+                ),
+                ExpressionModuleLoaderRef(
+                    source_ref = internal_source_ref,
+                ),
+                internal_source_ref
+            )
+
+        statements.append(
+            StatementAssignmentVariableName(
+                variable_name = "__spec__",
+                source        = spec_value,
+                source_ref    = internal_source_ref
+            )
+        )
+
     if python_version >= 300:
         statements.append(
             StatementAssignmentVariableName(
@@ -818,22 +864,33 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
             )
         )
 
-
-    if python_version >= 330:
-        # For Python3.3, it's set for both packages and non-packages.
-        statements.append(
-            StatementAssignmentVariableName(
-                variable_name = "__package__",
-                source        = makeConstantRefNode(
-                    constant      = provider.getFullName()
-                                      if provider.isCompiledPythonPackage() else
-                                    provider.getPackage(),
-                    source_ref    = internal_source_ref,
-                    user_provided = True
-                ),
-                source_ref    = internal_source_ref
-            )
+    # The "__package__" attribute is set to proper value for 3.3 or higher even
+    # for normal modules, previously for packages only.
+    statements.append(
+        StatementAssignmentVariableName(
+            variable_name = "__package__",
+            source        = makeConstantRefNode(
+                constant      = (
+                                  (
+                                    None
+                                      if '.' not in provider.getFullName() or python_version >= 320 else
+                                    provider.getFullName()
+                                  )
+                                    if python_version < 330 else
+                                  provider.getFullName()
+                                )
+                                  if provider.isCompiledPythonPackage() else
+                                (
+                                    provider.getPackage()
+                                      if python_version >= 330 else
+                                    None
+                                ),
+                source_ref    = internal_source_ref,
+                user_provided = True
+            ),
+            source_ref    = internal_source_ref
         )
+    )
 
     needs__initializing__ = not provider.isMainModule() and \
       (python_version >= 330 and python_version < 340)
@@ -1093,7 +1150,10 @@ def buildModuleTree(filename, package, is_top, is_main):
     # If there is source code associated (not the case for namespace packages of
     # Python3.3 or higher, then read it.
     if source_filename is not None:
-        source_code = readSourceCodeFromFilename(module.getFullName(), source_filename)
+        source_code = readSourceCodeFromFilename(
+            module_name     = module.getFullName(),
+            source_filename = source_filename
+        )
 
         if is_main:
             checkPythonVersionFromCode(source_code)
