@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,13 +31,14 @@ from nuitka.nodes.NodeMakingHelpers import makeComparisonNode
 from nuitka.nodes.OperatorNodes import ExpressionOperationNOT
 from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ReturnNodes import StatementReturn
-from nuitka.nodes.VariableRefNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef
-)
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
-from .Helpers import buildNode, getKind, makeStatementsSequenceFromStatement
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
+    buildNode,
+    getKind,
+    makeStatementsSequenceFromStatement
+)
 
 
 def buildComparisonNode(provider, node, source_ref):
@@ -78,12 +79,11 @@ def buildComparisonNode(provider, node, source_ref):
 
 def buildComplexComparisonNode(provider, left, rights, comparators, source_ref):
 
-    # This is a bit complex, due to the many details, pylint: disable=R0914
+    # This is a bit complex, due to the many details, pylint: disable=too-many-locals
 
     outline_body = ExpressionOutlineBody(
         provider   = provider,
         name       = "comparison_chain",
-        body       = None, # later
         source_ref = source_ref
     )
 
@@ -103,12 +103,9 @@ def buildComplexComparisonNode(provider, left, rights, comparators, source_ref):
 
     def makeTempAssignment(count, value):
         return StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = variables[count],
-                source_ref = source_ref
-            ),
-            source       = value,
-            source_ref   = source_ref,
+            variable   = variables[count],
+            source     = value,
+            source_ref = source_ref,
         )
 
     def makeReleaseStatement(count):
@@ -119,17 +116,14 @@ def buildComplexComparisonNode(provider, left, rights, comparators, source_ref):
 
     def makeValueComparisonReturn(left, right, comparator):
         yield StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_variable,
-                source_ref = source_ref
-            ),
-            source       = makeComparisonNode(
+            variable   = tmp_variable,
+            source     = makeComparisonNode(
                 left       = left,
                 right      = right,
                 comparator = comparator,
                 source_ref = source_ref
             ),
-            source_ref   = source_ref,
+            source_ref = source_ref,
         )
 
         yield StatementConditional(
