@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,82 +22,137 @@ source code comments with developer manual sections.
 
 """
 
+from nuitka.__past__ import intern  # pylint: disable=I0021,redefined-builtin
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
-from nuitka.nodes.BuiltinIteratorNodes import (
-    ExpressionBuiltinIter1,
-    ExpressionBuiltinNext1
-)
+from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
+from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerOperationNodes import (
-    ExpressionDictOperationSet,
-    ExpressionListOperationAppend,
-    ExpressionSetOperationAdd
+    StatementListOperationAppend,
+    StatementSetOperationAdd
 )
-from nuitka.nodes.FrameNodes import StatementsFrame
-from nuitka.nodes.FunctionNodes import (
-    ExpressionFunctionBody,
-    ExpressionFunctionCall,
-    ExpressionFunctionCreation,
-    ExpressionFunctionRef
+from nuitka.nodes.DictionaryNodes import StatementDictOperationSet
+from nuitka.nodes.FrameNodes import (
+    StatementsFrameFunction,
+    StatementsFrameGenerator
 )
+from nuitka.nodes.FunctionNodes import ExpressionFunctionRef
 from nuitka.nodes.GeneratorNodes import (
     ExpressionGeneratorObjectBody,
     ExpressionMakeGeneratorObject
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
-from nuitka.nodes.NodeMakingHelpers import (
-    makeVariableRefNode,
-    makeVariableTargetRefNode
+from nuitka.nodes.NodeMakingHelpers import makeVariableRefNode
+from nuitka.nodes.OutlineNodes import (
+    ExpressionOutlineBody,
+    ExpressionOutlineFunction
 )
-from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
-from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
-    StatementGeneratorEntry,
     StatementsSequence
 )
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.nodes.YieldNodes import ExpressionYield
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
-    buildNode,
-    buildNodeList,
-    getKind,
-    makeStatementsSequenceFromStatement,
-    mergeStatements
-)
 from .ReformulationAssignmentStatements import buildAssignmentStatements
 from .ReformulationBooleanExpressions import buildAndNode
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
+    buildNode,
+    buildNodeList,
+    getKind,
+    makeStatementsSequenceFromStatement,
+    makeStatementsSequenceFromStatements,
+    mergeStatements
+)
+
+
+def _buildPython2ListContraction(provider, node, source_ref):
+    # The contraction nodes are reformulated to function bodies, with loops as
+    # described in the developer manual. They use a lot of temporary names,
+    # nested blocks, etc. and so a lot of variable names.
+
+    # Note: The assign_provider is only to cover Python2 list contractions,
+    # assigning one of the loop variables to the outside scope.
+    function_body = ExpressionOutlineBody(
+        provider   = provider,
+        name       = "list_contraction",
+        source_ref = source_ref
+    )
+
+    iter_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = ".0"
+    )
+
+    container_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = "contraction_result"
+    )
+
+    statements, release_statements = _buildContractionBodyNode(
+        function_body   = function_body,
+        assign_provider = True,
+        provider        = provider,
+        node            = node,
+        emit_class      = StatementListOperationAppend,
+        iter_tmp        = iter_tmp,
+        temp_scope      = None,
+        start_value     = [],
+        container_tmp   = container_tmp,
+        source_ref      = source_ref,
+    )
+
+    statements.append(
+        StatementReturn(
+            expression = ExpressionTempVariableRef(
+                variable   = container_tmp,
+                source_ref = source_ref
+            ),
+            source_ref = source_ref
+        )
+    )
+
+    statement = makeTryFinallyStatement(
+        provider   = function_body,
+        tried      = statements,
+        final      = release_statements,
+        source_ref = source_ref.atInternal()
+    )
+
+    function_body.setBody(
+        makeStatementsSequenceFromStatement(
+            statement = statement
+        )
+    )
+
+    return function_body
 
 
 def buildListContractionNode(provider, node, source_ref):
     # List contractions are dealt with by general code.
+    if python_version < 300:
+        return _buildPython2ListContraction(
+            provider   = provider,
+            node       = node,
+            source_ref = source_ref
+        )
 
     return _buildContractionNode(
-        provider        = provider,
-        node            = node,
-        name            = "list_contraction"
-                            if python_version < 300 else
-                          "<listcontraction>",
-        emit_class      = ExpressionListOperationAppend,
-        start_value     = ExpressionConstantRef(
-            constant   = [],
-            source_ref = source_ref
-        ),
-        # Note: For Python3, the list contractions no longer assign to the outer
-        # scope.
-        assign_provider = python_version < 300,
-        source_ref      = source_ref
+        provider    = provider,
+        node        = node,
+        name        = "<listcontraction>",
+        emit_class  = StatementListOperationAppend,
+        start_value = [],
+        source_ref  = source_ref
     )
 
 
@@ -105,16 +160,12 @@ def buildSetContractionNode(provider, node, source_ref):
     # Set contractions are dealt with by general code.
 
     return _buildContractionNode(
-        provider        = provider,
-        node            = node,
-        name            = "<setcontraction>",
-        emit_class      = ExpressionSetOperationAdd,
-        start_value     = ExpressionConstantRef(
-            constant   = set(),
-            source_ref = source_ref
-        ),
-        assign_provider = False,
-        source_ref      = source_ref
+        provider    = provider,
+        node        = node,
+        name        = "<setcontraction>",
+        emit_class  = StatementSetOperationAdd,
+        start_value = set(),
+        source_ref  = source_ref
     )
 
 
@@ -122,16 +173,12 @@ def buildDictContractionNode(provider, node, source_ref):
     # Dict contractions are dealt with by general code.
 
     return _buildContractionNode(
-        provider        = provider,
-        node            = node,
-        name            = "<dictcontraction>",
-        emit_class      = ExpressionDictOperationSet,
-        start_value     = ExpressionConstantRef(
-            constant   = {},
-            source_ref = source_ref
-        ),
-        assign_provider = False,
-        source_ref      = source_ref
+        provider    = provider,
+        node        = node,
+        name        = "<dictcontraction>",
+        emit_class  = StatementDictOperationSet,
+        start_value = {},
+        source_ref  = source_ref
     )
 
 
@@ -140,27 +187,124 @@ def buildGeneratorExpressionNode(provider, node, source_ref):
 
     assert getKind(node) == "GeneratorExp"
 
-    return _buildContractionNode(
+    function_body = ExpressionOutlineBody(
+        provider   = provider,
+        name       = "genexpr",
+        source_ref = source_ref
+    )
+
+    iter_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = ".0"
+    )
+
+    parent_module = provider.getParentModule()
+
+    code_object = CodeObjectSpec(
+        co_name           = "<genexpr>",
+        co_kind           = "Generator",
+        co_varnames       = (".0",),
+        co_argcount       = 1,
+        co_kwonlyargcount = 0,
+        co_has_starlist   = False,
+        co_has_stardict   = False,
+        co_filename       = parent_module.getRunTimeFilename(),
+        co_lineno         = source_ref.getLineNumber(),
+        future_spec       = parent_module.getFutureSpec()
+    )
+
+    code_body = ExpressionGeneratorObjectBody(
+        provider   = provider,
+        name       = "<genexpr>",
+        flags      = set(),
+        source_ref = source_ref
+    )
+
+    function_body.setBody(
+        makeStatementsSequenceFromStatements(
+            StatementAssignmentVariable(
+                variable   = iter_tmp,
+                source     = ExpressionBuiltinIter1(
+                    value      = buildNode(
+                        provider   = provider,
+                        node       = node.generators[0].iter,
+                        source_ref = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            ),
+            makeTryFinallyStatement(
+                provider   = function_body,
+                tried      = StatementReturn(
+                    expression = ExpressionMakeGeneratorObject(
+                        generator_ref = ExpressionFunctionRef(
+                            function_body = code_body,
+                            source_ref    = source_ref
+                        ),
+                        code_object   = code_object,
+                        source_ref    = source_ref
+                    ),
+                    source_ref = source_ref
+                ),
+                final      = StatementReleaseVariable(
+                    variable   = iter_tmp,
+                    source_ref = source_ref
+                ),
+                source_ref = source_ref
+            )
+        )
+    )
+
+    statements, release_statements = _buildContractionBodyNode(
+        function_body   = code_body,
         provider        = provider,
         node            = node,
-        name            = "<genexpr>",
         emit_class      = ExpressionYield,
+        iter_tmp        = iter_tmp,
+        temp_scope      = None,
         start_value     = None,
+        container_tmp   = None,
         assign_provider = False,
-        source_ref      = source_ref
+        source_ref      = source_ref,
     )
 
+    statements = (
+        makeTryFinallyStatement(
+            provider   = function_body,
+            tried      = statements,
+            final      = release_statements,
+            source_ref = source_ref.atInternal()
+        ),
+    )
+
+    code_body.setBody(
+        makeStatementsSequenceFromStatement(
+            statement = StatementsFrameGenerator(
+                statements  = mergeStatements(statements, False),
+                code_object = code_object,
+                source_ref  = source_ref
+            )
+        )
+    )
+
+    return function_body
+
 
 def _buildContractionBodyNode(provider, node, emit_class, start_value,
                               container_tmp, iter_tmp, temp_scope,
-                              assign_provider, function_body, source_ref):
+                              assign_provider, function_body,
+                              source_ref):
 
     # This uses lots of variables and branches. There is no good way
     # around that, and we deal with many cases, due to having generator
-    # expressions sharing this code, pylint: disable=R0912,R0914
-    tmp_variables = []
+    # expressions sharing this code, pylint: disable=too-many-branches,too-many-locals
 
-    if assign_provider:
+    # Note: The assign_provider is only to cover Python2 list contractions,
+    # assigning one of the loop variables to the outside scope.
+
+    tmp_variables = []
+    if emit_class is not ExpressionYield:
         tmp_variables.append(iter_tmp)
 
     if container_tmp is not None:
@@ -170,11 +314,8 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     if assign_provider:
         statements = [
             StatementAssignmentVariable(
-                variable_ref = makeVariableTargetRefNode(
-                    variable   = iter_tmp,
-                    source_ref = source_ref
-                ),
-                source       = ExpressionBuiltinIter1(
+                variable   = iter_tmp,
+                source     = ExpressionBuiltinIter1(
                     value      = buildNode(
                         provider   = provider,
                         node       = node.generators[0].iter,
@@ -182,7 +323,7 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
                     ),
                     source_ref = source_ref
                 ),
-                source_ref   = source_ref.atInternal()
+                source_ref = source_ref.atInternal()
             )
         ]
     else:
@@ -191,16 +332,15 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     if start_value is not None:
         statements.append(
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = container_tmp,
+                variable   = container_tmp,
+                source     = makeConstantRefNode(
+                    constant   = start_value,
                     source_ref = source_ref
                 ),
-                source       = start_value,
-                source_ref   = source_ref.atInternal()
+                source_ref = source_ref.atInternal()
             )
         )
 
-
     if hasattr(node, "elt"):
         if start_value is not None:
             current_body = emit_class(
@@ -227,9 +367,9 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
                 source_ref = source_ref
             )
     else:
-        assert emit_class is ExpressionDictOperationSet
+        assert emit_class is StatementDictOperationSet
 
-        current_body = ExpressionDictOperationSet(
+        current_body = StatementDictOperationSet(
             dict_arg   = ExpressionTempVariableRef(
                 variable   = container_tmp,
                 source_ref = source_ref
@@ -247,10 +387,11 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
             source_ref = source_ref
         )
 
-    current_body = StatementExpressionOnly(
-        expression = current_body,
-        source_ref = source_ref
-    )
+    if current_body.isExpression():
+        current_body = StatementExpressionOnly(
+            expression = current_body,
+            source_ref = source_ref
+        )
 
     for count, qual in enumerate(reversed(node.generators)):
         tmp_value_variable = function_body.allocateTempVariable(
@@ -293,12 +434,9 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
 
             nested_statements = [
                 StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_iter_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = value_iterator,
-                    source_ref   = source_ref
+                    variable   = tmp_iter_variable,
+                    source     = value_iterator,
+                    source_ref = source_ref
                 )
             ]
 
@@ -310,15 +448,12 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
         loop_statements = [
             makeTryExceptSingleHandlerNode(
                 tried          = StatementAssignmentVariable(
-                    variable_ref = ExpressionTargetTempVariableRef(
-                        variable   = tmp_value_variable,
-                        source_ref = source_ref
-                    ),
-                    source       = ExpressionBuiltinNext1(
+                    variable   = tmp_value_variable,
+                    source     = ExpressionBuiltinNext1(
                         value      = iterator_ref,
                         source_ref = source_ref
                     ),
-                    source_ref   = source_ref
+                    source_ref = source_ref
                 ),
                 exception_name = "StopIteration",
                 handler_body   = StatementLoopBreak(
@@ -387,14 +522,6 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
     statements.append(current_body)
     statements = mergeStatements(statements)
 
-    if emit_class is ExpressionYield:
-        statements.insert(
-            0,
-            StatementGeneratorEntry(
-                source_ref = source_ref
-            )
-        )
-
     release_statements = [
         StatementReleaseVariable(
             variable   = tmp_variable,
@@ -408,99 +535,29 @@ def _buildContractionBodyNode(provider, node, emit_class, start_value,
 
 
 def _buildContractionNode(provider, node, name, emit_class, start_value,
-                          assign_provider, source_ref):
+                          source_ref):
     # The contraction nodes are reformulated to function bodies, with loops as
     # described in the developer manual. They use a lot of temporary names,
     # nested blocks, etc. and so a lot of variable names.
 
-    # Note: The assign_provider is only to cover Python2 list contractions,
-    # assigning one of the loop variables to the outside scope.
-    if assign_provider:
-        function_body = ExpressionOutlineBody(
-            provider   = provider,
-            name       = name,
-            source_ref = source_ref
-        )
-
-        iter_tmp = function_body.allocateTempVariable(
-            temp_scope = None,
-            name       = ".0"
-        )
-    else:
-        # TODO: No function ought to be necessary.
-
-        function_body = ExpressionFunctionBody(
-            provider   = provider,
-            name       = name,
-            doc        = None,
-            parameters = ParameterSpec(
-                name          = name,
-                normal_args   = (".0",),
-                list_star_arg = None,
-                dict_star_arg = None,
-                default_count = 0,
-                kw_only_args  = ()
-            ),
-            flags      = set(),
-            source_ref = source_ref
-        )
-
-        iter_tmp = function_body.getVariableForAssignment(
-            variable_name = ".0"
-        )
-        assert iter_tmp.isParameterVariable()
-
-    code_object = CodeObjectSpec(
-        code_name     = name,
-        code_kind     = "Generator" if emit_class is ExpressionYield else "Function",
-        arg_names     = () if assign_provider else function_body.getParameters().getParameterNames(),
-        kw_only_count = 0,
-        has_starlist  = False,
-        has_stardict  = False,
+    function_body = ExpressionOutlineFunction(
+        provider   = provider,
+        name       = intern(name[1:-1]),
+        source_ref = source_ref
     )
 
-    if emit_class is ExpressionYield:
-        code_body = ExpressionGeneratorObjectBody(
-            provider   = function_body,
-            name       = "<genexpr>",
-            flags      = set(),
-            source_ref = source_ref
-        )
+    iter_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = ".0"
+    )
 
-        iter_tmp = code_body.getVariableForReference(
-            variable_name = ".0"
-        )
-        assert iter_tmp.isLocalVariable()
-
-        function_body.setBody(
-            makeStatementsSequenceFromStatement(
-                statement = StatementReturn(
-                    expression = ExpressionMakeGeneratorObject(
-                        generator_ref = ExpressionFunctionRef(
-                            function_body = code_body,
-                            source_ref    = source_ref
-                        ),
-                        code_object   = code_object,
-                        source_ref    = source_ref
-                    ),
-                    source_ref = source_ref
-                )
-            )
-        )
-    else:
-        code_body = function_body
-
-    if start_value is not None:
-        container_tmp = code_body.allocateTempVariable(
-            temp_scope = None,
-            name       = "contraction_result"
-        )
-    else:
-        container_tmp = None
+    container_tmp = function_body.allocateTempVariable(
+        temp_scope = None,
+        name       = "contraction"
+    )
 
     statements, release_statements = _buildContractionBodyNode(
-        function_body   = code_body,
-        assign_provider = assign_provider,
+        function_body   = function_body,
         provider        = provider,
         node            = node,
         emit_class      = emit_class,
@@ -508,19 +565,32 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
         temp_scope      = None,
         start_value     = start_value,
         container_tmp   = container_tmp,
+        assign_provider = False,
         source_ref      = source_ref,
     )
 
-    if start_value is not None:
-        statements.append(
-            StatementReturn(
-                expression = ExpressionTempVariableRef(
-                    variable   = container_tmp,
-                    source_ref = source_ref
-                ),
+    assign_iter_statement = StatementAssignmentVariable(
+        source     = ExpressionBuiltinIter1(
+            value      = buildNode(
+                provider   = provider,
+                node       = node.generators[0].iter,
                 source_ref = source_ref
-            )
+            ),
+            source_ref = source_ref
+        ),
+        variable   = iter_tmp,
+        source_ref = source_ref
+    )
+
+    statements.append(
+        StatementReturn(
+            expression = ExpressionTempVariableRef(
+                variable   = container_tmp,
+                source_ref = source_ref
+            ),
+            source_ref = source_ref
         )
+    )
 
     statements = (
         makeTryFinallyStatement(
@@ -531,43 +601,36 @@ def _buildContractionNode(provider, node, name, emit_class, start_value,
         ),
     )
 
-    code_body.setBody(
-        makeStatementsSequenceFromStatement(
-            statement = StatementsFrame(
+    if python_version < 300:
+        body = makeStatementsSequenceFromStatements(
+            assign_iter_statement,
+            statements
+        )
+    else:
+        parent_module = provider.getParentModule()
+
+        code_object = CodeObjectSpec(
+            co_name           = name,
+            co_kind           = "Function",
+            co_varnames       = (),
+            co_argcount       = 1,
+            co_kwonlyargcount = 0,
+            co_has_starlist   = False,
+            co_has_stardict   = False,
+            co_filename       = parent_module.getRunTimeFilename(),
+            co_lineno         = source_ref.getLineNumber(),
+            future_spec       = parent_module.getFutureSpec()
+        )
+
+        body = makeStatementsSequenceFromStatements(
+            assign_iter_statement,
+            StatementsFrameFunction(
                 statements  = mergeStatements(statements, False),
-                guard_mode  = "pass_through"
-                                  if emit_class is not ExpressionYield else
-                                "generator",
                 code_object = code_object,
                 source_ref  = source_ref
             )
         )
-    )
 
-    if not assign_provider:
-        return ExpressionFunctionCall(
-            function   = ExpressionFunctionCreation(
-                function_ref = ExpressionFunctionRef(
-                    function_body = function_body,
-                    source_ref    = source_ref
-                ),
-                code_object  = code_object,
-                defaults     = (),
-                kw_defaults  = None,
-                annotations  = None,
-                source_ref   = source_ref
-            ),
-            values     = (
-                ExpressionBuiltinIter1(
-                    value      = buildNode(
-                        provider   = provider,
-                        node       = node.generators[0].iter,
-                        source_ref = source_ref
-                    ),
-                    source_ref = source_ref
-                ),
-            ),
-            source_ref = source_ref
-        )
-    else:
-        return function_body
+    function_body.setBody(body)
+
+    return function_body
