@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -29,16 +29,13 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
 from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
-from nuitka.nodes.BuiltinIteratorNodes import (
-    ExpressionBuiltinIter1,
-    ExpressionBuiltinNext1
-)
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.BuiltinIteratorNodes import ExpressionBuiltinIter1
+from nuitka.nodes.BuiltinNextNodes import ExpressionBuiltinNext1
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.DictionaryNodes import (
     ExpressionKeyValuePair,
@@ -56,7 +53,7 @@ from nuitka.nodes.FunctionNodes import (
     ExpressionFunctionRef
 )
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
-from nuitka.nodes.OperatorNodes import ExpressionOperationBinary
+from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
 from nuitka.nodes.ParameterSpecs import ParameterSpec
 from nuitka.nodes.ReturnNodes import StatementReturn
 from nuitka.nodes.TypeNodes import ExpressionBuiltinType1
@@ -66,13 +63,6 @@ from nuitka.nodes.VariableRefNodes import (
 )
 from nuitka.PythonVersions import python_version
 
-from .Helpers import (
-    buildNode,
-    buildNodeList,
-    makeDictCreationOrConstant,
-    makeStatementsSequenceFromStatement,
-    makeStatementsSequenceFromStatements
-)
 from .InternalModule import (
     getInternalModule,
     internal_source_ref,
@@ -80,6 +70,13 @@ from .InternalModule import (
 )
 from .ReformulationTryExceptStatements import makeTryExceptSingleHandlerNode
 from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
+    buildNode,
+    buildNodeList,
+    makeDictCreationOrConstant,
+    makeStatementsSequenceFromStatement,
+    makeStatementsSequenceFromStatements
+)
 
 
 def buildDictionaryNode(provider, node, source_ref):
@@ -107,12 +104,12 @@ def getDictUnpackingHelper():
         name       = helper_name,
         doc        = None,
         parameters = ParameterSpec(
-            name          = helper_name,
-            normal_args   = (),
-            list_star_arg = "args",
-            dict_star_arg = None,
-            default_count = 0,
-            kw_only_args  = ()
+            ps_name          = helper_name,
+            ps_normal_args   = (),
+            ps_list_star_arg = "args",
+            ps_dict_star_arg = None,
+            ps_default_count = 0,
+            ps_kw_only_args  = ()
         ),
         flags      = set(),
         source_ref = internal_source_ref
@@ -127,18 +124,15 @@ def getDictUnpackingHelper():
     loop_body = makeStatementsSequenceFromStatements(
         makeTryExceptSingleHandlerNode(
             tried          = StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_item_variable,
-                    source_ref = internal_source_ref
-                ),
-                source       = ExpressionBuiltinNext1(
+                variable   = tmp_item_variable,
+                source     = ExpressionBuiltinNext1(
                     value      = ExpressionTempVariableRef(
                         variable   = tmp_iter_variable,
                         source_ref = internal_source_ref
                     ),
                     source_ref = internal_source_ref
                 ),
-                source_ref   = internal_source_ref
+                source_ref = internal_source_ref
             ),
             exception_name = "StopIteration",
             handler_body   = StatementLoopBreak(
@@ -163,9 +157,9 @@ def getDictUnpackingHelper():
                 exception_type  = ExpressionBuiltinMakeException(
                     exception_name = "TypeError",
                     args           = (
-                        ExpressionOperationBinary(
+                        makeBinaryOperationNode(
                             operator   = "Mod",
-                            left       =  ExpressionConstantRef(
+                            left       =  makeConstantRefNode(
                                 constant      = """\
 '%s' object is not a mapping""",
                                 source_ref    = internal_source_ref,
@@ -222,30 +216,23 @@ def getDictUnpackingHelper():
 
     tried = makeStatementsSequenceFromStatements(
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_iter_variable,
-                source_ref = internal_source_ref
-            ),
-            source       = ExpressionBuiltinIter1(
+            variable   = tmp_iter_variable,
+            source     = ExpressionBuiltinIter1(
                 value      = ExpressionVariableRef(
-                    variable_name = "args",
-                    variable      = args_variable,
-                    source_ref    = internal_source_ref
+                    variable   = args_variable,
+                    source_ref = internal_source_ref
                 ),
                 source_ref = internal_source_ref
             ),
-            source_ref   = internal_source_ref
+            source_ref = internal_source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_result_variable,
-                source_ref = internal_source_ref
-            ),
-            source       = ExpressionConstantRef(
+            variable   = tmp_result_variable,
+            source     = makeConstantRefNode(
                 constant   = {},
                 source_ref = internal_source_ref
             ),
-            source_ref   = internal_source_ref
+            source_ref = internal_source_ref
         ),
         StatementLoop(
             body       = loop_body,
@@ -289,9 +276,9 @@ def buildDictionaryUnpackingArgs(provider, keys, values, source_ref):
                 ExpressionMakeDict(
                     pairs      = (
                         ExpressionKeyValuePair(
-                            key        = ExpressionConstantRef(
-                                            constant   = key,
-                                            source_ref = source_ref
+                            key        = makeConstantRefNode(
+                                constant   = key,
+                                source_ref = source_ref
                             ),
                             value      = buildNode(provider, value, source_ref),
                             source_ref = source_ref
