@@ -23,8 +23,6 @@ source code comments with developer manual sections.
 """
 
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -46,10 +44,17 @@ from nuitka.nodes.StatementNodes import (
     StatementsSequence
 )
 from nuitka.nodes.TryNodes import StatementTry
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.PythonVersions import python_version
-from nuitka.tree import SyntaxErrors
 
-from .Helpers import (
+from .ReformulationAssignmentStatements import (
+    buildAssignmentStatements,
+    buildDeleteStatementFromDecoded,
+    decodeAssignTarget
+)
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .SyntaxErrors import raiseSyntaxError
+from .TreeHelpers import (
     buildNode,
     buildStatementsNode,
     makeReraiseExceptionStatement,
@@ -58,12 +63,6 @@ from .Helpers import (
     makeStatementsSequenceFromStatements,
     mergeStatements
 )
-from .ReformulationAssignmentStatements import (
-    buildAssignmentStatements,
-    buildDeleteStatementFromDecoded,
-    decodeAssignTarget
-)
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
@@ -82,15 +81,12 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
     statements = mergeStatements(
         (
             StatementAssignmentVariable(
-                variable_ref = ExpressionTargetTempVariableRef(
-                    variable   = tmp_handler_indicator_variable,
-                    source_ref = source_ref.atInternal()
-                ),
-                source       = makeConstantRefNode(
+                variable   = tmp_handler_indicator_variable,
+                source     = makeConstantRefNode(
                     constant   = False,
                     source_ref = source_ref
                 ),
-                source_ref   = no_raise.getSourceReference().atInternal()
+                source_ref = no_raise.getSourceReference().atInternal()
             ),
             handling
         ),
@@ -136,15 +132,12 @@ def makeTryExceptNoRaise(provider, temp_scope, tried, handling, no_raise,
 
     return makeStatementsSequenceFromStatements(
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_handler_indicator_variable,
-                source_ref = source_ref.atInternal()
-            ),
-            source       = makeConstantRefNode(
+            variable   = tmp_handler_indicator_variable,
+            source     = makeConstantRefNode(
                 constant   = True,
                 source_ref = source_ref
             ),
-            source_ref   = source_ref.atInternal()
+            source_ref = source_ref.atInternal()
         ),
         makeTryFinallyStatement(
             provider   = provider,
@@ -264,7 +257,7 @@ def buildTryExceptionNode(provider, node, source_ref):
     # exception types and hides away that they may be built or not.
 
     # Many variables and branches, due to the re-formulation that is going on
-    # here, which just has the complexity, pylint: disable=R0912,R0914
+    # here, which just has the complexity, pylint: disable=too-many-branches,too-many-locals
 
     tried = buildStatementsNode(
         provider   = provider,
@@ -335,7 +328,6 @@ def buildTryExceptionNode(provider, node, source_ref):
                         source_ref = source_ref
                     ),
                     final      = buildDeleteStatementFromDecoded(
-                        node       = node, # Won't be used.
                         kind       = kind,
                         detail     = detail,
                         source_ref = source_ref
@@ -360,7 +352,7 @@ def buildTryExceptionNode(provider, node, source_ref):
         # The exception types should be a tuple, so as to be most general.
         if exception_types is None:
             if handler is not node.handlers[-1]:
-                SyntaxErrors.raiseSyntaxError(
+                raiseSyntaxError(
                     "default 'except:' must be last",
                     source_ref.atLineNumber(handler.lineno).atColumnNumber(
                         handler.col_offset
