@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,8 +24,6 @@ source code comments with developer manual sections.
 
 from nuitka import Options
 from nuitka.nodes.AssignNodes import (
-    ExpressionTargetTempVariableRef,
-    ExpressionTempVariableRef,
     StatementAssignmentVariable,
     StatementReleaseVariable
 )
@@ -39,7 +37,7 @@ from nuitka.nodes.CallNodes import (
 )
 from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import StatementConditional
-from nuitka.nodes.ConstantRefNodes import ExpressionConstantRef
+from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import ExpressionMakeTuple
 from nuitka.nodes.CoroutineNodes import ExpressionAsyncWait
 from nuitka.nodes.ExceptionNodes import (
@@ -51,26 +49,28 @@ from nuitka.nodes.StatementNodes import (
     StatementExpressionOnly,
     StatementsSequence
 )
+from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 from nuitka.PythonVersions import python_version
-from nuitka.tree.Helpers import makeReraiseExceptionStatement
 
-from .Helpers import (
+from .ReformulationAssignmentStatements import buildAssignmentStatements
+from .ReformulationTryExceptStatements import (
+    makeTryExceptSingleHandlerNodeWithPublish
+)
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
+from .TreeHelpers import (
     buildNode,
     buildStatementsNode,
     getKind,
     makeConditionalStatement,
+    makeReraiseExceptionStatement,
     makeStatementsSequence,
     makeStatementsSequenceFromStatement
 )
-from .ReformulationAssignmentStatements import buildAssignmentStatements
-from .ReformulationTryExceptStatements import \
-    makeTryExceptSingleHandlerNodeWithPublish
-from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 
 
 def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                    sync, source_ref):
-    # Many details, pylint: disable=R0914
+    # Many details, pylint: disable=too-many-locals
     with_source = buildNode(provider, context_expr, source_ref)
 
     if Options.isFullCompat():
@@ -174,7 +174,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
             variable   = tmp_exit_variable,
             source_ref = source_ref
         ),
-        args       = ExpressionConstantRef(
+        args       = makeConstantRefNode(
             constant   = (None, None, None),
             source_ref = source_ref
         ),
@@ -199,21 +199,18 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
     statements = [
         # First assign the with context to a temporary variable.
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_source_variable,
-                source_ref = source_ref
-            ),
-            source       = with_source,
-            source_ref   = source_ref
-        ),
+            variable   = tmp_source_variable,
+            source     = with_source,
+            source_ref = source_ref
+        )
+    ]
+
+    attribute_assignments = [
         # Next, assign "__enter__" and "__exit__" attributes to temporary
         # variables.
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_exit_variable,
-                source_ref = source_ref
-            ),
-            source       = attribute_lookup_class(
+            variable   = tmp_exit_variable,
+            source     = attribute_lookup_class(
                 source         = ExpressionTempVariableRef(
                     variable   = tmp_source_variable,
                     source_ref = source_ref
@@ -221,28 +218,30 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                 attribute_name = "__exit__" if sync else "__aexit__",
                 source_ref     = source_ref
             ),
-            source_ref   = source_ref
+            source_ref = source_ref
         ),
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_enter_variable,
-                source_ref = source_ref
-            ),
-            source       = enter_value,
-            source_ref   = source_ref
-        ),
+            variable   = tmp_enter_variable,
+            source     = enter_value,
+            source_ref = source_ref
+        )
+    ]
+
+    if python_version >= 360 and sync:
+        attribute_assignments.reverse()
+
+    statements += attribute_assignments
+
+    statements.append(
         StatementAssignmentVariable(
-            variable_ref = ExpressionTargetTempVariableRef(
-                variable   = tmp_indicator_variable,
-                source_ref = source_ref
-            ),
-            source       = ExpressionConstantRef(
+            variable   = tmp_indicator_variable,
+            source     = makeConstantRefNode(
                 constant   = True,
                 source_ref = source_ref
             ),
-            source_ref   = source_ref
-        ),
-    ]
+            source_ref = source_ref
+        )
+    )
 
     statements += [
         makeTryFinallyStatement(
@@ -256,15 +255,12 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                         # Prevents final block from calling __exit__ as
                         # well.
                         StatementAssignmentVariable(
-                            variable_ref = ExpressionTargetTempVariableRef(
-                                variable   = tmp_indicator_variable,
-                                source_ref = source_ref
-                            ),
-                            source       = ExpressionConstantRef(
+                            variable   = tmp_indicator_variable,
+                            source     = makeConstantRefNode(
                                 constant   = False,
                                 source_ref = source_ref
                             ),
-                            source_ref   = source_ref
+                            source_ref = source_ref
                         ),
                         makeConditionalStatement(
                             condition  = exit_value_exception,
@@ -286,7 +282,7 @@ def _buildWithNode(provider, context_expr, assign_target, body, body_lineno,
                         variable   = tmp_indicator_variable,
                         source_ref = source_ref
                     ),
-                    right      = ExpressionConstantRef(
+                    right      = makeConstantRefNode(
                         constant   = True,
                         source_ref = source_ref
                     ),
@@ -349,7 +345,7 @@ def buildWithNode(provider, node, source_ref):
     # The body for the first context manager is the other things.
     body = buildStatementsNode(provider, node.body, source_ref)
 
-    assert len(context_exprs) > 0 and len(context_exprs) == len(assign_targets)
+    assert context_exprs and len(context_exprs) == len(assign_targets)
 
     context_exprs.reverse()
     assign_targets.reverse()
@@ -388,7 +384,7 @@ def buildAsyncWithNode(provider, node, source_ref):
     # The body for the first context manager is the other things.
     body = buildStatementsNode(provider, node.body, source_ref)
 
-    assert len(context_exprs) > 0 and len(context_exprs) == len(assign_targets)
+    assert context_exprs and len(context_exprs) == len(assign_targets)
 
     context_exprs.reverse()
     assign_targets.reverse()
