@@ -1,4 +1,4 @@
-#     Copyright 2015, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -24,17 +24,24 @@ Only after this is executed, variable reference nodes can be considered
 complete.
 """
 
-from nuitka import PythonVersions, Variables
+from nuitka.nodes.AssignNodes import (
+    StatementAssignmentVariable,
+    StatementDelVariable
+)
+from nuitka.nodes.FunctionNodes import MaybeLocalVariableUsage
 from nuitka.nodes.NodeMakingHelpers import makeConstantReplacementNode
-from nuitka.nodes.ReturnNodes import StatementGeneratorReturn
-from nuitka.Options import isFullCompat
-from nuitka.tree import SyntaxErrors
-from nuitka.utils.Utils import python_version
-from nuitka.VariableRegistry import addVariableUsage, isSharedLogically
+from nuitka.nodes.VariableRefNodes import (
+    ExpressionLocalsVariableRef,
+    ExpressionVariableRef
+)
+from nuitka.PythonVersions import (
+    getErrorMessageExecWithNestedFunction,
+    python_version
+)
 
 from .Operations import VisitorNoopMixin, visitTree
 from .ReformulationFunctionStatements import addFunctionVariableReleases
-
+from .SyntaxErrors import raiseSyntaxError
 
 # Note: We do the variable scope assignment, as an extra step from tree
 # building, because tree building creates the tree without any consideration of
@@ -60,35 +67,35 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
     def _handleNonLocal(node):
         # Take closure variables for non-local declarations.
 
-        for non_local_names, source_ref in node.getNonlocalDeclarations():
+        for non_local_names, source_ref in node.consumeNonlocalDeclarations():
             for non_local_name in non_local_names:
-                variable = node.getClosureVariable(
+
+                variable = node.takeVariableForClosure(
                     variable_name = non_local_name
                 )
 
                 node.registerProvidedVariable(variable)
 
                 if variable.isModuleVariable():
-                    SyntaxErrors.raiseSyntaxError(
+                    raiseSyntaxError(
                         "no binding for nonlocal '%s' found" % (
                             non_local_name
                         ),
-                        source_ref   = None
-                                         if isFullCompat() and \
-                                         python_version < 340 else
-                                       source_ref,
-                        display_file = not isFullCompat() or \
-                                       python_version >= 340,
-                        display_line = not isFullCompat() or \
-                                       python_version >= 340
+                        source_ref
                     )
 
+
+                variable.addVariableUser(node)
+
     @staticmethod
     def _handleQualnameSetup(node):
         if node.qualname_setup is not None:
-            if node.isClassDictCreation():
-                class_assign, qualname_assign = node.qualname_setup
-                class_variable = class_assign.getTargetVariableRef().getVariable()
+            if node.isExpressionClassBody():
+                class_variable_name, qualname_assign = node.qualname_setup
+
+                class_variable = node.getParentVariableProvider().getVariableForAssignment(
+                    class_variable_name
+                )
 
                 if class_variable.isModuleVariable():
                     qualname_node = qualname_assign.getAssignSource()
@@ -102,8 +109,9 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
                     node.qualname_provider = node.getParentModule()
             else:
-                function_variable_ref = node.qualname_setup
-                function_variable = function_variable_ref.getVariable()
+                function_variable = node.getParentVariableProvider().getVariableForAssignment(
+                    node.qualname_setup
+                )
 
                 if function_variable.isModuleVariable():
                     node.qualname_provider = node.getParentModule()
@@ -115,70 +123,130 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
             node.qualname_setup = None
 
+    def onLeaveNode(self, node):
+        if node.isStatementAssignmentVariableName():
+            provider = node.getParentVariableProvider()
+
+            variable = provider.getVariableForAssignment(
+                variable_name = node.getVariableName()
+            )
+
+            node.replaceWith(
+                StatementAssignmentVariable(
+                    variable   = variable,
+                    source     = node.subnode_source,
+                    source_ref = node.source_ref
+                )
+            )
+
+            variable.addVariableUser(provider)
+        elif node.isStatementDelVariableName():
+            provider = node.getParentVariableProvider()
+
+            variable = provider.getVariableForAssignment(
+                variable_name = node.getVariableName()
+            )
+
+            node.replaceWith(
+                StatementDelVariable(
+                    variable   = variable,
+                    tolerant   = node.tolerant,
+                    source_ref = node.source_ref
+                )
+            )
+
+            variable.addVariableUser(provider)
+
     def onEnterNode(self, node):
         # Mighty complex code with lots of branches and statements, but it
         # couldn't be less without making it more difficult.
-        # pylint: disable=R0912,R0915
+        # pylint: disable=too-many-branches,too-many-statements
 
-        if node.isExpressionTargetVariableRef():
+        if node.isExpressionVariableNameRef():
             provider = node.getParentVariableProvider()
 
-            if node.getVariable() is None:
-                variable_name = node.getVariableName()
-
-                variable = provider.getVariableForAssignment(
-                    variable_name = variable_name
+            if provider.isEarlyClosure():
+                variable = provider.getVariableForReference(
+                    variable_name = node.getVariableName()
                 )
 
-                node.setVariable(variable)
+                # Python3.4 version respects closure variables taken can be
+                # overridden by writes to locals. It should be done for
+                # globals too, on all versions, but for Python2 the locals
+                # dictionary is avoided unless "exec" appears, so it's not
+                # done.
+                owner = variable.getOwner()
+                user = provider
 
-            addVariableUsage(node.getVariable(), provider)
-        elif node.isExpressionTargetTempVariableRef():
-            provider = node.getParentVariableProvider()
+                while user is not owner:
+                    if user.isExpressionFunctionBody() or \
+                       user.isExpressionClassBody():
+                        break
 
-            addVariableUsage(node.getVariable(), provider)
-        elif node.isExpressionVariableRef():
-            if node.getVariable() is None:
-                provider = node.getParentVariableProvider()
+                    user = user.getParentVariableProvider()
 
-                if provider.isEarlyClosure():
-                    variable = provider.getVariableForReference(
-                        variable_name = node.getVariableName()
+                if owner is not user:
+                    if python_version >= 340 or \
+                       (python_version >= 300 and \
+                        variable.isModuleVariable()):
+
+                        node.replaceWith(
+                            ExpressionLocalsVariableRef(
+                                variable_name     = node.getVariableName(),
+                                fallback_variable = variable,
+                                source_ref        = node.getSourceReference()
+                            )
+                        )
+
+                    else:
+                        node.replaceWith(
+                            ExpressionVariableRef(
+                                variable   = variable,
+                                source_ref = node.source_ref
+                            )
+                        )
+                else:
+                    node.replaceWith(
+                        ExpressionVariableRef(
+                            variable   = variable,
+                            source_ref = node.source_ref
+                        )
                     )
 
-                    # Python3.4 version respects closure variables taken can be
-                    # overridden by writes to locals. It should be done for
-                    # globals too, on all versions, but for Python2 the locals
-                    # dictionary is avoided unless "exec" appears, so it's not
-                    # done.
-                    if variable.getOwner() is not provider:
-                        if python_version >= 340 or \
-                           (python_version >= 300 and \
-                            variable.isModuleVariable()):
-                            variable = Variables.MaybeLocalVariable(
-                                owner          = provider,
-                                maybe_variable = variable
-                            )
-
-                    node.setVariable(variable)
+                variable.addVariableUser(provider)
         elif node.isExpressionTempVariableRef():
             if node.getVariable().getOwner() != node.getParentVariableProvider():
-                node.setVariable(
-                    node.getParentVariableProvider().addClosureVariable(
-                        node.getVariable()
-                    )
+                node.getParentVariableProvider().addClosureVariable(
+                    node.getVariable()
                 )
-        elif node.isExpressionFunctionBody():
-            if python_version >= 300:
-                self._handleNonLocal(node)
+        elif node.isExpressionGeneratorObjectBody():
+            self._handleNonLocal(node)
 
-            for variable in node.getParameters().getAllVariables():
-                addVariableUsage(variable, node)
+            # Only Python3.4 or later allows for generators to have qualname.
+            if python_version >= 340:
+                self._handleQualnameSetup(node)
+        elif node.isExpressionCoroutineObjectBody():
+            self._handleNonLocal(node)
+
+            self._handleQualnameSetup(node)
+        elif node.isExpressionAsyncgenObjectBody():
+            self._handleNonLocal(node)
+
+            self._handleQualnameSetup(node)
+        elif node.isExpressionClassBody():
+            self._handleNonLocal(node)
 
             # Python3.4 allows for class declarations to be made global, even
             # after they were declared, so we need to fix this up.
             if python_version >= 340:
                 self._handleQualnameSetup(node)
+        elif node.isExpressionFunctionBody():
+            self._handleNonLocal(node)
+
+            # Python 3.4 allows for class declarations to be made global, even
+            # after they were declared, so we need to fix this up.
+            if python_version >= 340:
+                self._handleQualnameSetup(node)
         # Attribute access of names of class functions should be mangled, if
         # they start with "__", but do not end in "__" as well.
         elif node.isExpressionAttributeLookup() or \
@@ -195,12 +263,10 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                 while True:
                     current = current.getParentVariableProvider()
 
-                    if current.isPythonModule():
+                    if current.isCompiledPythonModule():
                         break
 
-                    assert current.isExpressionFunctionBody()
-
-                    if current.isClassDictCreation():
+                    if current.isExpressionClassBody():
                         if seen_function:
                             node.setAttributeName(
                                 "_%s%s" % (
@@ -214,69 +280,25 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                         seen_function = True
         # Check if continue and break are properly in loops. If not, raise a
         # syntax error.
-        elif node.isStatementBreakLoop() or node.isStatementContinueLoop():
+        elif node.isStatementLoopBreak() or node.isStatementLoopContinue():
             current = node
 
             while True:
-                if current.isPythonModule() or \
-                   current.isExpressionFunctionBody():
-                    if node.isStatementContinueLoop():
-                        message = "'continue' not properly in loop"
-                        col_offset   = 16 if python_version >= 300 else None
-                        display_line = True
-                        source_line  = None
-                    else:
-                        message = "'break' outside loop"
-
-                        if isFullCompat():
-                            col_offset   = 2 if python_version >= 300 else None
-                            display_line = True
-                            source_line  = "" if python_version >= 300 else None
-                        else:
-                            col_offset   = 13
-                            display_line = True
-                            source_line  = None
-
-                    SyntaxErrors.raiseSyntaxError(
-                        message,
-                        source_ref   = node.getSourceReference(),
-                        col_offset   = col_offset,
-                        display_line = display_line,
-                        source_line  = source_line
-                    )
-
                 current = current.getParent()
 
                 if current.isStatementLoop():
                     break
 
-    def onLeaveNode(self, node):
-        # Return statements in generators are not really that, instead they are
-        # exception raises, fix that up now. Doing it right from the onset,
-        # would be a bit more difficult, as the knowledge that something is a
-        # generator, requires a second pass.
-        if node.isStatementReturn():
-            return_consumer = node.getParentReturnConsumer()
+                if current.isParentVariableProvider():
+                    if node.isStatementLoopContinue():
+                        message = "'continue' not properly in loop"
+                    else:
+                        message = "'break' outside loop"
 
-            if return_consumer.isExpressionFunctionBody() and \
-               return_consumer.isGenerator():
-                return_value = node.getExpression()
-
-                if python_version < 330:
-                    if not return_value.isExpressionReturnedValueRef() and \
-                       (not return_value.isExpressionConstantRef() or \
-                        return_value.getConstant() is not None):
-                        SyntaxErrors.raiseSyntaxError(
-                            "'return' with argument inside generator",
-                            source_ref = node.getSourceReference(),
-                        )
-
-                node.replaceWith(
-                    StatementGeneratorReturn(
-                        expression = return_value,
-                        source_ref = node.getSourceReference()
+                    raiseSyntaxError(
+                        message,
+                        node.getSourceReference(),
                     )
-                )
 
 
 class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
@@ -302,67 +324,78 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
             variable_name = variable_name
         )
 
-        node.setVariable(
-            variable
-        )
-
         # Need to catch functions with "exec" and closure variables not allowed.
         if python_version < 300 and \
            not was_taken and \
-           provider.isExpressionFunctionBody() and \
+           provider.isExpressionFunctionBodyBase() and \
            variable.getOwner() is not provider:
             parent_provider = provider.getParentVariableProvider()
 
-            while parent_provider.isExpressionFunctionBody() and \
-                  parent_provider.isClassDictCreation():
+            while parent_provider.isExpressionClassBody():
                 parent_provider = parent_provider.getParentVariableProvider()
 
             if parent_provider.isExpressionFunctionBody() and \
                parent_provider.isUnqualifiedExec():
-                SyntaxErrors.raiseSyntaxError(
-                    reason       = PythonVersions.\
-                                     getErrorMessageExecWithNestedFunction() % \
-                                     parent_provider.getName(),
-                    source_ref   = parent_provider.getExecSourceRef(),
-                    col_offset   = None,
-                    display_file = True,
-                    display_line = True,
-                    source_line  = None
+                raiseSyntaxError(
+                    getErrorMessageExecWithNestedFunction() % parent_provider.getName(),
+                    node.getSourceReference(),
+                    display_line = False # Wrong line anyway
                 )
 
+        return variable
 
     def onEnterNode(self, node):
-        if node.isExpressionVariableRef():
+        if node.isExpressionVariableNameRef():
             provider = node.getParentVariableProvider()
 
-            if node.getVariable() is None:
-                self._attachVariable(node, provider)
+            try:
+                variable = self._attachVariable(node, provider)
+            except MaybeLocalVariableUsage:
+                variable_name = node.getVariableName()
 
-            addVariableUsage(node.getVariable(), provider)
-        elif node.isExpressionTempVariableRef():
+                node.replaceWith(
+                    ExpressionLocalsVariableRef(
+                        variable_name     = variable_name,
+                        fallback_variable = node.getParentModule().getVariableForReference(variable_name),
+                        source_ref        = node.getSourceReference()
+                    )
+                )
+            else:
+                node.replaceWith(
+                    ExpressionVariableRef(
+                        variable   = variable,
+                        source_ref = node.source_ref
+                    )
+                )
+
+                variable.addVariableUser(provider)
+
+        elif node.isExpressionVariableRef():
             provider = node.getParentVariableProvider()
 
-            addVariableUsage(node.getVariable(), provider)
+            variable = node.getVariable()
 
+            if variable is None:
+                try:
+                    variable = self._attachVariable(node, provider)
+                except MaybeLocalVariableUsage:
+                    variable_name = node.getVariableName()
 
-    # For Python3, every function in a class is supposed to take "__class__" as
-    # a reference, so make sure that happens.
-    if python_version >= 300:
-        def onLeaveNode(self, node):
-            if node.isExpressionFunctionBody() and node.isClassClosureTaker():
-                if python_version < 340 or True: # TODO: Temporarily reverted:
-                    node.getVariableForReference(
-                        variable_name = "__class__"
+                    node.replaceWith(
+                        ExpressionLocalsVariableRef(
+                            variable_name     = variable_name,
+                            fallback_variable = node.getParentModule().getVariableForReference(variable_name),
+                            source_ref        = node.getSourceReference()
+                        )
                     )
                 else:
-                    parent_provider = node.getParentVariableProvider()
-
-                    variable = parent_provider.getTempVariable(
-                        temp_scope = None,
-                        name       = "__class__"
+                    node.setVariable(
+                        variable
                     )
 
-                    node.addClosureVariable(variable)
+                    variable.addVariableUser(provider)
+            else:
+                variable.addVariableUser(provider)
 
 
 class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
@@ -378,23 +411,33 @@ class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
 
     def onEnterNode(self, node):
         if python_version < 300 and node.isStatementDelVariable():
-            variable = node.getTargetVariableRef().getVariable()
+            variable = node.getVariable()
 
             if not variable.isModuleVariable() and \
-               isSharedLogically(variable):
-                SyntaxErrors.raiseSyntaxError(
-                    reason       = """\
+               variable.isSharedAmongScopes():
+                raiseSyntaxError(
+                    """\
 can not delete variable '%s' referenced in nested scope""" % (
                        variable.getName()
                     ),
-                    source_ref   = (
-                        None if isFullCompat() else node.getSourceReference()
-                    ),
-                    display_file = not isFullCompat(),
-                    display_line = not isFullCompat()
+                    node.getSourceReference()
                 )
         elif node.isStatementsFrame():
-            node.updateVarNames()
+            node.updateLocalNames()
+        elif node.isExpressionFunctionBodyBase():
+            addFunctionVariableReleases(node)
+
+            # Python3 is influenced by the mere use of a variable named as
+            # "super". So we need to prepare ability to take closure.
+            if node.hasFlag("has_super"):
+                if not node.hasVariableName("__class__"):
+                    class_var = node.takeVariableForClosure("__class__")
+                    class_var.addVariableUser(node)
+
+                    node.registerProvidedVariable(class_var)
+                    while node != class_var.getOwner():
+                        node = node.getParentVariableProvider()
+                        node.registerProvidedVariable(class_var)
 
 
 def completeVariableClosures(tree):
@@ -406,15 +449,3 @@ def completeVariableClosures(tree):
 
     for visitor in visitors:
         visitTree(tree, visitor)
-
-        if tree.isPythonModule():
-            for function in tree.getFunctions():
-                visitTree(function, visitor)
-
-    if tree.isPythonModule():
-        for function in tree.getFunctions():
-            addFunctionVariableReleases(function)
-    else:
-        # For "eval" code, this will need to be done differently, not done
-        # yet-
-        assert False
