@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -319,3 +319,9 @@ def execWithShortTuple():
         return "gives exception: " + repr(e)
 
 print "Exec with too short tuple argument:", execWithShortTuple()
+
+if str is not bytes:
+    def evalMemoryView(value):
+        return eval(memoryview(value))
+
+    print "Eval with memory view:", evalMemoryView(b"27")
