@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -47,7 +47,8 @@ def checkFromFunction():
             d = dict(d)
             d["__file__"] = "<__file__ removed>"
 
-        return repr(d)
+        import pprint
+        return pprint.pformat(d)
 
     print("Globals", displayDict(frame.f_globals))
     print("Locals", displayDict(frame.f_locals))
