@@ -17,24 +17,24 @@
 #
 import sys, os
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
-    executeReferenceChecked,  # @UnresolvedImport
-    my_print,                 # @UnresolvedImport
+from nuitka.tools.testing.Common import (
+    executeReferenceChecked,
+    checkDebugPython
 )
 
-if not hasattr(sys, "gettotalrefcount"):
-    my_print("Warning, using non-debug Python makes this test ineffective.")
-    sys.gettotalrefcount = lambda : 0
+checkDebugPython()
+
 
 x = 17
 
@@ -899,6 +899,19 @@ def simpleFunction109():
     # Call a PyCFunction that looks like a method call.
     sys.exc_info()
 
+def simpleFunction110():
+    def my_open(*args, **kwargs):
+        return(args, kwargs)
+
+    __builtins__.open = my_open
+
+    open("me", buffering = True)
+
+u = u'__name__'
+
+def simpleFunction111():
+    return getattr(simpleFunction111, u)
+
 ####################################
 
 
