@@ -17,35 +17,25 @@
 #
 import sys, os, types
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
+from nuitka.tools.testing.Common import (
     executeReferenceChecked,
-    my_print,
+    checkDebugPython,
+    run_async
 )
 
-if not hasattr(sys, "gettotalrefcount"):
-    my_print("Warning, using non-debug Python makes this test ineffective.")
-    sys.gettotalrefcount = lambda : 0
+checkDebugPython()
 
-def run_async(coro):
-    values = []
-    result = None
-    while True:
-        try:
-            values.append(coro.send(None))
-        except StopIteration as ex:
-            result = ex.args[0] if ex.args else None
-            break
-    return values, result
 
 def raisy():
     raise TypeError
