@@ -1,4 +1,4 @@
-#     Copyright 2016, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python test originally created or extracted from other peoples work. The
 #     parts from me are licensed as below. It is at least Free Software where
@@ -28,4 +28,5 @@ print(len(range(22266,9998,-3)))
 print(len(range(22266,9997,-3)))
 print(len(range(22266,9996,-3)))
 print(len(range(0,3,3)))
-print(len(iter((1, 2))))
+print(len([3,3.3]))
+print(len((3,3.3)))
