@@ -93,7 +93,9 @@ def isConstantExpression(expression):
 
     return kind.startswith("Constant") or \
            kind in ("ImportModuleHard",
-                    "ModuleFileAttributeRef")
+                    "ImportName",
+                    "ModuleFileAttributeRef",
+                    "ModuleLoaderRef")
 
 def checkSequence(statements):
     for statement in statements:
@@ -104,7 +106,7 @@ def checkSequence(statements):
 
             if not isConstantExpression(print_arg):
                 sys.exit(
-                    "%s: Error, print of non-constant %s." % (
+                    "%s: Error, print of non-constant '%s'." % (
                         getSourceRef(statement),
                         getKind(print_arg)
                     )
@@ -132,13 +134,20 @@ def checkSequence(statements):
             continue
 
         if kind == "AssignmentVariable":
+            variable_name = statement.attrib["variable_name"]
+
+            # Ignore "__spec__" assignment for Python3.4, it is not going
+            # to be static.
+            if variable_name == "__spec__":
+                continue
+
             assign_source, = getRole(statement, "source")
 
             if getKind(assign_source) == "FunctionCreation":
                 continue
 
             elif not isConstantExpression(assign_source):
-                sys.exit("Error, assignment from non-constant %s." % getKind(assign_source))
+                sys.exit("Error, assignment from non-constant '%s'." % getKind(assign_source))
 
             continue
 
@@ -220,18 +229,22 @@ for filename in sorted(os.listdir('.')):
         assert len(module_statements_sequence) == 1
         module_statements = next(iter(module_statements_sequence))
 
-        checkSequence(module_statements)
+        try:
+            checkSequence(module_statements)
 
-        for function in root.xpath('role[@name="functions"]/node'):
-            function_body, = function.xpath('role[@name="body"]')
-            function_statements_sequence = function_body[0]
-            assert len(function_statements_sequence) == 1
-            function_statements = next(iter(function_statements_sequence))
+            for function in root.xpath('role[@name="functions"]/node'):
+                function_body, = function.xpath('role[@name="body"]')
+                function_statements_sequence = function_body[0]
+                assert len(function_statements_sequence) == 1
+                function_statements = next(iter(function_statements_sequence))
 
-            checkSequence(function_statements)
+                checkSequence(function_statements)
 
-        if changed:
-            os.unlink(filename)
+            if changed:
+                os.unlink(filename)
+        except SystemExit:
+            my_print("FAIL.")
+            raise
 
         my_print("OK.")
     else:
