@@ -17,19 +17,21 @@
 #     limitations under the License.
 #
 
-import os, sys
+import os
+import sys
 
-# Find common code relative in file system. Not using packages for test stuff.
+# Find nuitka package relative to us.
 sys.path.insert(
     0,
     os.path.normpath(
         os.path.join(
             os.path.dirname(os.path.abspath(__file__)),
+            "..",
             ".."
         )
     )
 )
-from test_common import (
+from nuitka.tools.testing.Common import (
     my_print,
     setup,
     createSearchMode,
